package com.tofly.xrtygis.custom.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.xrtygis.custom.entity.dto.DmaAlertReportDto;
import com.tofly.xrtygis.custom.entity.vo.DmaAlertReportVo;
import com.tofly.xrtygis.custom.entity.vo.DmaSalesAmountVo;
import com.tofly.xrtygis.custom.service.LscreenService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 智慧营销接口
 */
@RestController
@RequestMapping("/Lscreen")
@Api(tags = "大屏展示_智慧营销接口")
public class LscreenController {

    @Autowired
    private  LscreenService lscreenService;
    @Value("${waterSupplyChu.codes}")
    private String waterSupplyChu;

    @GetMapping("/GetUseWater")
    @ApiOperation("用户统计")
    public ResultRespone getUseWater(){
        return ResultRespone.success(lscreenService.getUseWater());
    }

    @GetMapping("/GetUserCount")
    @ApiOperation("近12月用户总数和增长数")
    public ResultRespone getUserCountByMonth(){
        return ResultRespone.success(lscreenService.getUserCount());
    }

    @GetMapping("/GetNearNumber")
    @ApiOperation("售水量统计")
    public ResultRespone getNearNumber(){

        return ResultRespone.success(lscreenService.getNearNumber());
    }

    @GetMapping("/GetLargeUseWater")
    @ApiOperation("大用户用水排名统计")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "topNum",value = "大用户用水前几名，默认取前10名",required = false,dataType = "INTEGER")
    })
    public ResultRespone getLargeUseWater(Integer topNum){
       if(topNum==null){
            topNum=10;
        }
        return ResultRespone.success(lscreenService.getLargeUseWater(topNum));
    }



    @GetMapping("/GetUserYsXzAndMoney")
    @ApiOperation("上月用水性质、多种分类的收费数据统计")
    public ResultRespone getUserYsXzAndMoney(){
        return ResultRespone.success(lscreenService.getUserYsXzAndMoney());
    }


    @GetMapping("/GetQf3Data")
    @ApiOperation("欠费超过3个月数据统计")
    public ResultRespone getQf3Data(){

        return ResultRespone.success(lscreenService.getQf3Data());
    }

    @GetMapping("/GetLastSfQf")
    @ApiOperation("上月收费，欠费统计")
    public ResultRespone getLastSfQf(){
        return ResultRespone.success(lscreenService.getLastSfQf());
    }

    @GetMapping("/MeterReadingRate")
    @ApiOperation("抄表率统计")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "num",value = "取前几个月的数据，默认取最近6个月的数据",required = false,dataType = "INTEGER")
    })
    public ResultRespone GetMeterReadingRate(Integer num){
        if(num==null || num==0){
            num=6;
        }
        return ResultRespone.success(lscreenService.GetMeterReadingRate(num));
    }



    @GetMapping("/GetCollectionRate")
    @ApiOperation("今年，去年，回款率统计")
    public ResultRespone GetCollectionRate(){

        return ResultRespone.success(lscreenService.GetCollectionRate());
    }


    @GetMapping("/GetNearPay")
    @ApiOperation("今年，去年，收费金额统计")
    public ResultRespone getNearPay(){

        return ResultRespone.success(lscreenService.getNearPay());
    }

    @GetMapping("/GetqdPay")
    @ApiOperation("近12月收费方式")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "num",value = "取前几个月的数据，默认取最近12个月的数据",required = false,dataType = "INTEGER")
    })
    public ResultRespone getqdPay(Integer num){
       if(num==null || num==0){
           num=12;
       }
        return ResultRespone.success(lscreenService.getqdPay(num));
    }

    @PostMapping("/saveCustomerData")
    @ApiOperation("按照年月同步营销有效用户总数")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "recordMonth",value = "同步年月，yyyyMM格式，多个逗号分离",required = true,dataType = "String")
    })
    public ResultRespone saveCustomerData(String  recordMonth){
        return ResultRespone.success(lscreenService.saveCustomerData(recordMonth));
    }


    @GetMapping("/userWaterType")
    @ApiOperation("领导驾驶舱_用水类型占比统计")
    public ResultRespone getUserWaterType(){
        return ResultRespone.success(lscreenService.getUserWaterType());
    }


    @GetMapping("/salesWaterTotal")
    @ApiOperation("智慧决策_售水量构成统计")
    public ResultRespone getSalesWaterTotal(){
        return ResultRespone.success(lscreenService.getSalesWaterTotal());
    }

    @GetMapping("/getPayInfo")
    @ApiOperation("智慧决策_今年收费数据展示")
    public ResultRespone getPayInfo(){
        return ResultRespone.success(lscreenService.getPayInfo());
    }

    @GetMapping("/getPayMoneyInfo")
    @ApiOperation("智慧决策_年度同期收费情况")
    public ResultRespone getPayMoneyInfo(){
        return ResultRespone.success(lscreenService.getPayMoneyInfo());
    }


    @GetMapping("/getPayUserInfo")
    @ApiOperation("智慧决策_年度同期用户收费情况")
    public ResultRespone getPayUserInfo(){
        return ResultRespone.success(lscreenService.getPayUserInfo());
    }

    @GetMapping("/getWaterType")
    @ApiOperation("智慧运营_用户水表类型占比统计")
    public ResultRespone getWaterType(){
        return ResultRespone.success(lscreenService.getWaterType());
    }

    @GetMapping("/getSalesWaterCount")
    @ApiOperation("慧决策_近12月生产销售水量")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "month",value = "统计近几月的数据，默认month值为12",required = false,dataType = "Integer")
    })
    public ResultRespone getSalesWaterCount(Integer month){
        if(month==null || month==0){
            month=12;
        }
        return ResultRespone.success(lscreenService.getSalesWaterCount(month,waterSupplyChu));
    }

    @GetMapping("/getLastMonthSalesWater")
    @ApiOperation("慧决策_上月生产销售水量")
    public ResultRespone getLastMonthSalesWater(){
        return ResultRespone.success(lscreenService.getLastMonthSalesWater());
    }

    @GetMapping("/getUserAndMoneyReport")
    @ApiOperation("领导驾驶舱_今年/去年同期营收总额；当前/去年同期用户数量，水厂数量")
    public ResultRespone getUserAndMoneyReport(){
        return ResultRespone.success(lscreenService.getUserAndMoneyReport());
    }

    @GetMapping("/getDmaSalesAmountReport")
    @ApiOperation("Dma售水量统计分析报表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "startDate",value = "开始月份格式 yyyy-MM",required = true,dataType = "String"),
            @ApiImplicitParam(name = "endDate",value = "结束月份格式 yyyy-MM",required = true,dataType = "String"),
            @ApiImplicitParam(name = "dmaId",value = "DmaId值", dataType = "Long")
    })
    public ResultRespone<List<DmaSalesAmountVo>> getUserAndMoneyReport(Long dmaId, String startDate, String endDate){
        return ResultRespone.success(lscreenService.getDmaSalesAmountReport(dmaId,startDate,endDate));
    }

    @GetMapping("/getDmaAlertReport")
    @ApiOperation("Dma售水量统计分析报表")
    public ResultRespone<Page<DmaAlertReportVo>> getDmaAlertReport(Page page, DmaAlertReportDto dto){
        return ResultRespone.success(lscreenService.getDmaAlertReport(page,  dto));
    }

    @GetMapping("/getDmaAlertReportTotal")
    @ApiOperation("Dma售水量统计分析报表总数统计")
    public ResultRespone getDmaAlertReportTotal(DmaAlertReportDto dto){
        Map<String,Object> map=new HashMap<>();
        Page page=new Page();
        page.setSize(1);
        page.setCurrent(1);

        dto.setAlertType("设备报警");
        Page<DmaAlertReportVo> pageListSb= lscreenService.getDmaAlertReport(page,  dto);
        Long deviceCount= pageListSb.getTotal();
        dto.setAlertType("漏损报警");
        Page<DmaAlertReportVo> pageListLs= lscreenService.getDmaAlertReport(page,  dto);
        Long lsCount= pageListLs.getTotal();
        map.put("deviceTotal",deviceCount);
        map.put("lsTotal",lsCount);
        map.put("total",deviceCount+lsCount);
        return ResultRespone.success(map);
    }

}
