package com.tofly.xrtygis.custom.controller.dma;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.xrtygis.commons.Constant;
import com.tofly.xrtygis.custom.entity.dto.ScadaDataDto;
import com.tofly.xrtygis.custom.entity.vo.*;
import com.tofly.xrtygis.custom.service.dma.CustomDmaScadaService;
import com.tofly.xrtygis.entity.vo.DmaAboutVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author JF
 * @date 2024-03-12
 */
@RestController
@RequestMapping("/customDmaScada")
@Api(tags = "DMA分区和SCADA系统综合展示")
public class CustomDmaScadaController {

    @Autowired
    private CustomDmaScadaService customDmaScadaService;

    @PostMapping("/viewScadaData")
    @ApiOperation(value = "查询SCADA指标数据")
    @ToFlyAppLog(title = "查询SCADA指标数据")
    public ResultRespone<List<ScadaIndexVo>> viewScadaData(ScadaDataDto dto) {
        try {
            return ResultRespone.success(customDmaScadaService.viewScadaData(dto));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    @GetMapping("/viewMeterAbout")
    @ApiOperation(value = "查询抄表到户相关数据")
    @ToFlyAppLog(title = "查询抄表到户相关数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "year", value = "年份（yyyy）", required = true, dataType = "String")
    })
    public ResultRespone<MeterAboutVo> viewMeterAbout(String year) {
        try {
            return ResultRespone.success(customDmaScadaService.viewMeterAbout(year));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    @GetMapping("/getDmaSupplyWater")
    @ApiOperation(value = "获取DMA分区供水量分析")
    @ToFlyAppLog(title = "获取DMA分区供水量分析")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dmaId", value = "DMA分区ID", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "statisticsType", value = "统计类型（0小时统计;1天统计;2月统计;3年统计）", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "startDate", value = "开始时间（yyyy-MM-dd HH:mm:ss）", required = true, dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "结束时间（yyyy-MM-dd HH:mm:ss）", required = true, dataType = "String"),
    })
    public ResultRespone<List<DmaSupplyWaterVo>> getDmaSupplyWater(Long dmaId, Integer statisticsType, String startDate, String endDate) {
        try {
            return ResultRespone.success(customDmaScadaService.getDmaSupplyWater(dmaId, statisticsType, startDate, endDate));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    @GetMapping("/getMeterWater")
    @ApiOperation(value = "获取水表水量")
    @ToFlyAppLog(title = "获取水表水量")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dmaId", value = "DMA分区ID", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "statisticsType", value = "统计类型（0小时统计;1天统计;2月统计;3年统计）", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "startDate", value = "开始时间（yyyy-MM-dd HH:mm:ss）", required = true, dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "结束时间（yyyy-MM-dd HH:mm:ss）", required = true, dataType = "String"),
    })
    public ResultRespone<List<MeterWaterVo>> getMeterWater(Long dmaId, Integer statisticsType, String startDate, String endDate) {
        try {
            return ResultRespone.success(customDmaScadaService.getMeterWater(dmaId, statisticsType, startDate, endDate));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    @GetMapping("/viewCruxIndex")
    @ApiOperation(value = "关键指标展示")
    public ResultRespone<CruxIndexVo> viewCruxIndex(){
        try {
            return ResultRespone.success(customDmaScadaService.viewCruxIndex());
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    @GetMapping("/chargeWaterAnalyse")
    @ApiOperation(value = "计费水量分析")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dmaId", value = "DMA分区ID", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "statisticsType", value = "统计类型（0小时统计;1天统计;2月统计;3年统计）", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "startDate", value = "开始时间（yyyy-MM-dd HH:mm:ss）", required = true, dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "结束时间（yyyy-MM-dd HH:mm:ss）", required = true, dataType = "String"),
    })
    public ResultRespone<List<DmaSupplyWaterVo>> chargeWaterAnalyse(Long dmaId, Integer statisticsType, String startDate, String endDate){
        try {
            return ResultRespone.success(customDmaScadaService.chargeWaterAnalyse(dmaId, statisticsType, startDate, endDate));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    @GetMapping("/leakageRateAnalyse")
    @ApiOperation("漏损率分析")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "level", value = "分区等级", dataType = "Integer"),
            @ApiImplicitParam(name = "dmaId", value = "分区Id", dataType = "Long"),
            @ApiImplicitParam(name = "isLeakageWarn", value = "漏损是否告警（0否1是）", dataType = "String"),
            @ApiImplicitParam(name = "isMinNightWarn", value = "夜间最小流量是否告警（0否1是）", dataType = "String"),
            @ApiImplicitParam(name = "startDate", value = "开始时间 yyyy-MM-dd HH:mm:ss", required = true, dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "结束时间 yyyy-MM-dd HH:mm:ss", required = true, dataType = "String"),
            @ApiImplicitParam(name = "statisticsType", value = "统计类型（0小时统计;1天统计;2月统计;3年统计）", required = true, dataType = "Integer"),
    })
    public ResultRespone<List<DmaLeakageVo>> leakageRateAnalyse(Integer level, Long dmaId, String isLeakageWarn, String isMinNightWarn, String startDate, String endDate, Integer statisticsType){
        return ResultRespone.success(customDmaScadaService.leakageRateAnalyse(level, dmaId, isLeakageWarn, isMinNightWarn, startDate, endDate, statisticsType));
    }

    @GetMapping("/regionAboutInfo")
    @ApiOperation("分区相关信息查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dmaIds", value = "分区Id（多个用,分隔）", dataType = "String"),
    })
    public ResultRespone<List<DmaAboutVo>> regionAboutInfo(String dmaIds){
        List<Long> dmaIdList = null;
        if (StringUtils.isNotBlank(dmaIds)) {
            dmaIdList = Arrays.stream(dmaIds.split(Constant.CSV)).map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
        }
        return ResultRespone.success(customDmaScadaService.regionAboutInfo(dmaIdList));
    }

    @GetMapping("/getDmaWaterAbout")
    @ApiOperation(value = "获取DMA分区相关水量分析报表")
    @ToFlyAppLog(title = "获取DMA分区相关水量分析报表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dmaId", value = "DMA分区ID", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "statisticsType", value = "统计类型（0小时统计;1天统计;2月统计;3年统计）", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "startDate", value = "开始时间（yyyy-MM-dd HH:mm:ss）", required = true, dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "结束时间（yyyy-MM-dd HH:mm:ss）", required = true, dataType = "String"),
    })
    public ResultRespone<List<DmaWaterAboutVo>> getDmaWaterAbout(Long dmaId, Integer statisticsType, String startDate, String endDate) {
        try {
            return ResultRespone.success(customDmaScadaService.getDmaWaterAbout(dmaId, statisticsType, startDate, endDate));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    @GetMapping("/nightFlowMonitor")
    @ApiOperation(value = "夜间最小流量监测")
    @ToFlyAppLog(title = "夜间最小流量监测")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "deviceId", value = "设备ID", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "startDate", value = "开始时间（yyyy-MM-dd HH:mm:ss）", required = true, dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "结束时间（yyyy-MM-dd HH:mm:ss）", required = true, dataType = "String"),
    })
    public ResultRespone<List<NightFlowMonitorVo>> nightFlowMonitor(Long deviceId, String startDate, String endDate) {
        try {
            return ResultRespone.success(customDmaScadaService.nightFlowMonitor(deviceId, startDate, endDate));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }
}
