package com.tofly.xrtygis.custom.mapper.dma;

import com.tofly.xrtygis.custom.entity.dto.ScadaDataDto;
import com.tofly.xrtygis.custom.entity.vo.ScadaIndexVo;
import com.tofly.xrtygis.custom.entity.vo.TableUsageVo;
import com.tofly.xrtygis.custom.entity.vo.YxMeterVo;
import com.tofly.xrtygis.entity.DmaUnfilledWater;
import com.tofly.xrtygis.entity.vo.PipeMaterialStatisticalVo;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

public interface CustomDmaScadaMapper {

    /**
     * 获取SCADA设备指标相关数据
     * @param dto 开始时间
     * @return StationIndexVo 综合展示请求参数
     */
    List<ScadaIndexVo> viewScadaData(ScadaDataDto dto);

    /**
     * 获取SCADA设备指标相关数据（时间分组）
     * @param dto 开始时间
     * @return StationIndexVo 综合展示请求参数
     */
    List<ScadaIndexVo> viewScadaDataGroupTime(ScadaDataDto dto);

    /**
     * 获取抄表用水总量
     * @param startDate 开始时间
     * @param endDate 结束时间
     * @return
     */
    BigDecimal getMeterAmount(@Param("startDate") String startDate, @Param("endDate") String endDate);

    /**
     * 获取DMA分区未上盘水量填报数据总量
     * @param startDate 开始时间
     * @param endDate 结束时间
     * @return DmaUnfilledWater DMA分区未上盘水量数据填报记录表
     */
    DmaUnfilledWater getUnfilledWaterTotal(@Param("startDate") String startDate, @Param("endDate") String endDate);

    /**
     * 获取水厂总设计供水能力
     * @return
     */
    BigDecimal getDesignWaterCapacity();

    /**
     * 获取水厂数量
     * @return
     */
    Integer getWaterStationNum();

    /**
     * 获取用户数量
     * @return
     */
    Integer getUserNum();

    /**
     * 获取管网总长度
     * @return
     */
    @Select("SELECT MATERIAL, NUM " +
            "FROM " +
            "(SELECT MATERIAL, ROUND( SUM( PIPELENGTH ), 2 ) NUM " +
            "FROM njsw_sde.TF_JSJS_PIPE_B " +
            "GROUP BY MATERIAL)")
    List<PipeMaterialStatisticalVo> getPipeLength();

    /**
     * 获取分区数量
     * @param level 分区等级
     * @param type 分区类型
     * @return
     */
    Integer getRegionNum(@Param("level") String level, @Param("type") String type);

    /**
     * 获取设备数量
     * @param status 设备状态
     * @return
     */
    Integer getDeviceNum(@Param("status") String status);

    /**
     * 获取漏点数量
     * @param
     * @return
     */
    Integer getLeakNum();

    /**
     * 获取漏点数量
     * @param
     * @return
     */
    List<Map<String,Object>> getLeakNumGroupTime(@Param("dmaId") Long dmaId, @Param("statisticsType") Integer statisticsType);

    /**
     * 获取昨日水量
     * @param dataType 指标类型
     * @return
     */
    List<Map<String,Object>> getYesterdayWater(@Param("dataType") String dataType);

    /**
     * 获取上月水量
     * @param dataType 指标类型
     * @return
     */
    List<Map<String,Object>> getLastMonthWater(@Param("dataType") String dataType);

    /**
     * 获取设备水量
     * @param dataType 指标类型
     * @return
     */
    List<Map<String,Object>> getDeviceWater(@Param("statisticsType") Integer statisticsType,
                                            @Param("startDate") String startDate,
                                            @Param("endDate") String endDate,
                                            @Param("dataType") String dataType);

    /**
     * 获取营销抄表数据
     * @param dateType 统计类型（0小时统计;1天统计;2月统计;3年统计）
     * @param startDate 开始时间
     * @param endDate 结束时间
     * @return ScadaIndexVo 营销抄表数据
     */
    List<YxMeterVo> viewYxData(@Param("dateType") Integer dateType,
                               @Param("startDate") String startDate,
                               @Param("endDate") String endDate);

    /**
     * 获取设备供水量
     * @param startDate 开始时间
     * @param endDate 结束时间
     * @param indexType 指标类型
     * @param statisticsType 统计类型（0小时统计;1天统计;2月统计;3年统计）
     * @return TableUsageVo 水表设备用水量
     */
    List<TableUsageVo> getDiffByDevice(@Param("startDate") String startDate,
                                          @Param("endDate") String endDate,
                                          @Param("indexType") String indexType,
                                          @Param("statisticsType") Integer statisticsType);

    /**
     * 获取夜间最小流量
     * @param startDate 开始时间
     * @param endDate 结束时间
     * @return
     */
    List<Map<String, Object>> getMinNightFlow(@Param("startDate") String startDate, @Param("endDate") String endDate);

    /**
     * 获取夜间最小流量列表
     * @param startDate 开始时间
     * @param endDate 结束时间
     * @return
     */
    List<Map<String, Object>> getMinNightFlowList(@Param("deviceId") Long deviceId, @Param("startDate") String startDate, @Param("endDate") String endDate);
}
