package com.tofly.xrtygis.custom.service.dma.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.tofly.xrtygis.custom.entity.dto.ScadaDataDto;
import com.tofly.xrtygis.custom.entity.vo.*;
import com.tofly.xrtygis.custom.mapper.dma.CustomDmaScadaMapper;
import com.tofly.xrtygis.custom.service.dma.CustomDmaScadaService;
import com.tofly.xrtygis.em.IndexTypeEnum;
import com.tofly.xrtygis.em.StatisticsTypeEnum;
import com.tofly.xrtygis.entity.*;
import com.tofly.xrtygis.entity.scada.ScadaReport;
import com.tofly.xrtygis.entity.vo.DmaAboutVo;
import com.tofly.xrtygis.service.*;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
public class CustomDmaScadaServiceImpl implements CustomDmaScadaService {

    @Autowired
    private CustomDmaScadaMapper customDmaScadaMapper;
    @Autowired
    private DmaandobserService dmaandobserService;
    @Autowired
    private ScadaReportService scadaReportService;
    @Autowired
    private DmaregionService dmaregionService;
    @Autowired
    private TfYwpnDmaregionotherWService dmaregionotherService;
    @Autowired
    private DmanightflowService dmanightflowService;
    @Autowired
    private TfDeviceArchiveManageService deviceArchiveManageService;

    @Override
    public List<ScadaIndexVo> viewScadaData(ScadaDataDto dto) {
        switch (dto.getTableType()) {
            case 1:
                dto.setTableName("SCADA_HISTORY");
                break;
            case 2:
                dto.setTableName("STATISTICS_SCADA");
                break;
            case 3:
                dto.setTableName("SCADA_MONITOR");
                break;
        }
        if (CollUtil.isNotEmpty(dto.getDmaIdList())) {
            List<Dmaandobser> dmaandobserList = dmaandobserService.list(new QueryWrapper<Dmaandobser>().lambda()
                    .eq(Dmaandobser::getIsout, "0")
                    .in(Dmaandobser::getDmaId, dto.getDmaIdList()));
            if (CollUtil.isNotEmpty(dmaandobserList)) {
                dto.setDeviceIdList(dmaandobserList.stream().map(Dmaandobser::getTableId).collect(Collectors.toList()));
            }
        }
        List<ScadaIndexVo> scadaIndexVoList;
        if (StringUtils.isNotEmpty(dto.getIsGroupTime()) && dto.getIsGroupTime().equals("1")) {
            scadaIndexVoList = customDmaScadaMapper.viewScadaDataGroupTime(dto);
        } else {
            scadaIndexVoList = customDmaScadaMapper.viewScadaData(dto);
        }
        List<ScadaReport> scadaReportList = scadaReportService.list(new QueryWrapper<ScadaReport>().lambda().eq(ScadaReport::getIsDel, "0"));
        for (ScadaIndexVo scadaIndexVo : scadaIndexVoList) {
            scadaIndexVo.setIndexStatus("0");
            if (StringUtils.isBlank(scadaIndexVo.getReportIds())) {
                continue;
            }
            List<Long> reportIdList = Arrays.stream(scadaIndexVo.getReportIds().split(",")).map(Long::parseLong).collect(Collectors.toList());
            List<ScadaReport> reportList = scadaReportList.stream().filter(e -> reportIdList.contains(e.getId())).collect(Collectors.toList());
            if (CollUtil.isEmpty(reportList)) {
                continue;
            }
            scadaIndexVo.setReportList(reportList);
            for (ScadaReport report : reportList) {
                Double value = Double.parseDouble(scadaIndexVo.getIndexValue());
                if (value.compareTo(Double.parseDouble(report.getReportLower())) >= 0
                        && value.compareTo(Double.parseDouble(report.getReportUpper())) <= 0) {
                    scadaIndexVo.setIndexStatus("1");
                    scadaIndexVo.setReportLevel(report.getReportLevel());
                    break;
                }
            }
        }
        return scadaIndexVoList;
    }

    @Override
    public MeterAboutVo viewMeterAbout(String year) {
        MeterAboutVo res = new MeterAboutVo();

        String yearFirstDate = year + "01-01 00:00:00";
        String yearLastDate = year + "12-31 23:59:59";

        // 计算年居民抄表到户水量（m³）
        BigDecimal meterAmount = customDmaScadaMapper.getMeterAmount(yearFirstDate, yearLastDate);
        res.setYearResidentMeterWater(meterAmount);

        // 计算年填报免费用水量（m³）
        DmaUnfilledWater dmaUnfilledWater = customDmaScadaMapper.getUnfilledWaterTotal(yearFirstDate, yearLastDate);
        res.setYearFillFreeWater(dmaUnfilledWater == null ? BigDecimal.ZERO : dmaUnfilledWater.getFreeWaterVolume());

        // 计算年取水栓消防栓用水量（m³）
        ScadaDataDto dto = new ScadaDataDto();
        List<String> deviceTypeList = new ArrayList<>();
        deviceTypeList.add("6");
        deviceTypeList.add("7");
        dto.setDeviceTypeList(deviceTypeList);
        dto.setTableType(2);
        dto.setTableName("STATISTICS_SCADA");
        dto.setStatisticsType(StatisticsTypeEnum.THREE_YEAR.getDateType());
        dto.setIndexType(IndexTypeEnum.CUM_POS_FLOW.getIndexType());
        dto.setStartDate(yearFirstDate);
        dto.setEndDate(yearLastDate);
        List<ScadaIndexVo> waterFireScadaList = customDmaScadaMapper.viewScadaData(dto);
        res.setYearWaterFireWater(BigDecimal.ZERO);
        if (CollUtil.isNotEmpty(waterFireScadaList)) {
            BigDecimal totalWater = BigDecimal.valueOf(waterFireScadaList.stream().mapToDouble(s -> Double.parseDouble(s.getIndexValue())).sum());
            res.setYearWaterFireWater(totalWater);
        }

        // 计算年供水量（m³）
        ScadaDataDto dto1 = new ScadaDataDto();
        dto1.setTableType(2);
        dto1.setTableName("STATISTICS_SCADA");
        dto1.setStatisticsType(StatisticsTypeEnum.THREE_YEAR.getDateType());
        dto1.setIndexType(IndexTypeEnum.CUM_POS_FLOW.getIndexType());
        dto1.setStartDate(yearFirstDate);
        dto1.setEndDate(yearLastDate);
        dto1.setIsBindWaterStation(1);
        List<ScadaIndexVo> supplyWaterList = customDmaScadaMapper.viewScadaData(dto1);
        res.setYearSupplyWater(BigDecimal.ZERO);
        if (CollUtil.isNotEmpty(supplyWaterList)) {
            BigDecimal totalSupplyWater = BigDecimal.valueOf(supplyWaterList.stream().mapToDouble(s -> Double.parseDouble(s.getIndexValue())).sum());
            res.setYearSupplyWater(totalSupplyWater);
        }

        return res;
    }

    @Override
    public List<DmaSupplyWaterVo> getDmaSupplyWater(Long dmaId, Integer statisticsType, String startDate, String endDate) {
        List<DmaSupplyWaterVo> res = new ArrayList<>();

        ScadaDataDto dto = new ScadaDataDto();
        dto.setTableType(2);
        dto.setTableName("STATISTICS_SCADA");
        dto.setStatisticsType(statisticsType);
        dto.setIndexType(IndexTypeEnum.CUM_NET_FLOW.getIndexType());
        dto.setStartDate(startDate);
        dto.setEndDate(endDate);
        dto.setIsBindWaterStation(1);
        List<Dmaandobser> dmaandobserList = dmaandobserService.list(new QueryWrapper<Dmaandobser>().lambda()
                .eq(Dmaandobser::getDmaId, dmaId));
        if (CollUtil.isEmpty(dmaandobserList)) {
            return null;
        }
        dto.setDeviceIdList(dmaandobserList.stream().map(Dmaandobser::getTableId).collect(Collectors.toList()));
        List<ScadaIndexVo> scadaIndexList = customDmaScadaMapper.viewScadaData(dto);
        if (CollUtil.isEmpty(scadaIndexList)) {
            return null;
        }

        List<Date> betweenDate = getBetweenDate(startDate, endDate, statisticsType);
        for (Date date : betweenDate) {
            DmaSupplyWaterVo vo = new DmaSupplyWaterVo();
            vo.setScadaTime(DateUtil.format(date, "yyyy-MM-dd HH:mm:ss"));

            List<ScadaIndexVo> dateScadaIndexList = scadaIndexList.stream()
                    .filter(e -> e.getScadaTime().equals(DateUtil.format(date, "yyyy-MM-dd HH:mm:ss")))
                    .collect(Collectors.toList());
            if (CollUtil.isEmpty(dateScadaIndexList)) {
                continue;
            }
            List<Long> deviceInList = dmaandobserList.stream().filter(e -> e.getIsout().equals("0")).map(Dmaandobser::getTableId).collect(Collectors.toList());
            BigDecimal totalSupplyWater = BigDecimal.valueOf(dateScadaIndexList.stream().filter(e -> deviceInList.contains(e.getDeviceId())).mapToDouble(s -> Double.parseDouble(s.getIndexValue())).sum());
            vo.setSupplyWater(totalSupplyWater);

            Map<String, List<ScadaIndexVo>> scadaIndexCollect = dateScadaIndexList.stream().collect(Collectors.groupingBy(ScadaIndexVo::getDeviceCode));
            List<MeterWaterVo> meterList = new ArrayList<>();
            scadaIndexCollect.forEach((k, v)->{
                MeterWaterVo meter = new MeterWaterVo();
                meter.setMeterCode(k);
                meter.setMeterName(v.get(0).getDeviceName());
                BigDecimal meterSupplyWater = BigDecimal.valueOf(v.stream().mapToDouble(e -> Double.parseDouble(e.getIndexValue())).sum());
                meter.setSupplyWater(meterSupplyWater);
                meter.setProp(meterSupplyWater.divide(totalSupplyWater, 4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)));
                meterList.add(meter);
            });
            vo.setMeterList(meterList);

            res.add(vo);
        }

        return res;
    }

    @Override
    public List<MeterWaterVo> getMeterWater(Long dmaId, Integer statisticsType, String startDate, String endDate) {
        List<MeterWaterVo> res = new ArrayList<>();

        ScadaDataDto dto = new ScadaDataDto();
        dto.setTableType(2);
        dto.setTableName("STATISTICS_SCADA");
        dto.setStatisticsType(statisticsType);
        dto.setIndexType(IndexTypeEnum.CUM_NET_FLOW.getIndexType());
        dto.setStartDate(startDate);
        dto.setEndDate(endDate);
        dto.setIsBindWaterStation(1);
        List<Dmaandobser> dmaandobserList = dmaandobserService.list(new QueryWrapper<Dmaandobser>().lambda()
                .eq(Dmaandobser::getDmaId, dmaId));
        if (CollUtil.isEmpty(dmaandobserList)) {
            return null;
        }
        dto.setDeviceIdList(dmaandobserList.stream().map(Dmaandobser::getTableId).collect(Collectors.toList()));
        List<ScadaIndexVo> scadaIndexList = customDmaScadaMapper.viewScadaData(dto);
        if (CollUtil.isEmpty(scadaIndexList)) {
            return null;
        }

        List<Long> deviceInList = dmaandobserList.stream().filter(e -> e.getIsout().equals("0")).map(Dmaandobser::getTableId).collect(Collectors.toList());
        BigDecimal totalSupplyWater = BigDecimal.valueOf(scadaIndexList.stream().filter(e -> deviceInList.contains(e.getDeviceId())).mapToDouble(s -> Double.parseDouble(s.getIndexValue())).sum());
        Map<String, List<ScadaIndexVo>> scadaIndexCollect = scadaIndexList.stream().collect(Collectors.groupingBy(ScadaIndexVo::getDeviceCode));
        scadaIndexCollect.forEach((k, v)->{
            MeterWaterVo meter = new MeterWaterVo();
            meter.setMeterCode(k);
            meter.setMeterName(v.get(0).getDeviceName());
            BigDecimal meterSupplyWater = BigDecimal.valueOf(v.stream().mapToDouble(e -> Double.parseDouble(e.getIndexValue())).sum());
            meter.setSupplyWater(meterSupplyWater);
            meter.setProp(meterSupplyWater.divide(totalSupplyWater, 4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)));
            res.add(meter);
        });

        return res;
    }

    @Override
    public CruxIndexVo viewCruxIndex() {
        CruxIndexVo res = new CruxIndexVo();

        // 计算用户数量
        res.setUserNum(customDmaScadaMapper.getUserNum());

        // 计算管网总长度
//        List<PipeMaterialStatisticalVo> pipeMaterialStatisticalVos = customDmaScadaMapper.getPipeLength();
//        BigDecimal pipeLength = BigDecimal.ZERO;
//        for(PipeMaterialStatisticalVo one : pipeMaterialStatisticalVos){
//            if(one.getNum() != null){
//                pipeLength = pipeLength.add(one.getNum());
//            }
//        }
//        pipeLength = pipeLength.setScale(2, RoundingMode.HALF_UP);
//        res.setPipeLength(pipeLength);

        // 计算设计供水能力
        res.setDesignWaterCapacity(customDmaScadaMapper.getDesignWaterCapacity());

        // 计算水厂数量
        res.setWaterStationNum(customDmaScadaMapper.getWaterStationNum());

        // 计算一级分区数量
        res.setOneRegionNum(customDmaScadaMapper.getRegionNum("1", null));

        // 计算二级分区数量
        res.setTwoRegionNum(customDmaScadaMapper.getRegionNum("2", null));

        // 计算DMA分区数量
        res.setTwoRegionNum(customDmaScadaMapper.getRegionNum(null, "2"));

        // 获取分区列表
        List<Dmaregion> regionList = dmaregionService.list();
        if (CollUtil.isNotEmpty(regionList)) {
            res.setRegionList(regionList.stream().sorted(Comparator.comparing(Dmaregion::getPartitionLevel)).collect(Collectors.toList()));
        }

        // 计算设备数量
        res.setDeviceNum(customDmaScadaMapper.getDeviceNum(null));

        // 计算在线设备数量
        res.setOnLineDeviceNum(customDmaScadaMapper.getDeviceNum("1"));

        // 计算告警设备数量
        res.setWarnDeviceNum(customDmaScadaMapper.getDeviceNum("2"));

        // 计算漏点数量
        res.setLeakNum(customDmaScadaMapper.getLeakNum());

        // 计算昨日漏损率和漏损量
        res.setYesterdayLeakWater(BigDecimal.ZERO);
        res.setYesterdayLeakRate(00.00f);
        res.setLastYearYesterdayLeakWater(BigDecimal.ZERO);
        List<Map<String,Object>> yesterdayWaters = customDmaScadaMapper.getYesterdayWater(IndexTypeEnum.CUM_NET_FLOW.getIndexType());
        List<Dmaregion> dmaregions = dmaregionService.list(new QueryWrapper<Dmaregion>().lambda()
                .eq(Dmaregion::getPartitionLevel, 1));
        if (CollectionUtils.isNotEmpty(dmaregions) && CollectionUtils.isNotEmpty(yesterdayWaters)) {
            List<Long> dmaregionIds = dmaregions.stream().map(Dmaregion::getId).collect(Collectors.toList());
            List<Dmaandobser> inDmaandobsers = dmaandobserService.list(new QueryWrapper<Dmaandobser>().lambda()
                    .in(Dmaandobser::getDmaId, dmaregionIds)
                    .eq(Dmaandobser::getIsout, "0"));
            List<Dmaandobser> outDmaandobsers = dmaandobserService.list(new QueryWrapper<Dmaandobser>().lambda()
                    .in(Dmaandobser::getDmaId, dmaregionIds)
                    .eq(Dmaandobser::getIsout, "1"));
            if (CollectionUtils.isNotEmpty(inDmaandobsers) && CollectionUtils.isNotEmpty(outDmaandobsers)) {
                List<Long> inDeviceIds = inDmaandobsers.stream().map(Dmaandobser::getTableId).collect(Collectors.toList());
                List<Long> outDeviceIds = outDmaandobsers.stream().map(Dmaandobser::getTableId).collect(Collectors.toList());
                List<Map<String,Object>> inYesterdayWaters = yesterdayWaters.stream()
                        .filter(e -> e.get("deviceId") != null
                                && inDeviceIds.contains(Long.parseLong(String.valueOf(e.get("deviceId")))))
                        .collect(Collectors.toList());
                List<Map<String,Object>> outYesterdayWaters = yesterdayWaters.stream()
                        .filter(e -> e.get("deviceId") != null
                                && outDeviceIds.contains(Long.parseLong(String.valueOf(e.get("deviceId")))))
                        .collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(inYesterdayWaters) && CollectionUtils.isNotEmpty(outYesterdayWaters)) {
                    BigDecimal inWater = BigDecimal.valueOf(inYesterdayWaters.stream()
                            .filter(e -> e.get("diff") != null)
                            .mapToDouble(e -> Double.parseDouble(String.valueOf(e.get("diff")))).sum());
                    BigDecimal outWater = BigDecimal.valueOf(outYesterdayWaters.stream()
                            .filter(e -> e.get("diff") != null)
                            .mapToDouble(e -> Double.parseDouble(String.valueOf(e.get("diff")))).sum());
                    res.setYesterdayLeakWater(inWater.subtract(outWater));
                    res.setYesterdayLeakRate(inWater.subtract(outWater).divide(inWater, 4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).floatValue());
                    // 计算去年昨日漏损量
                    Calendar cal = Calendar.getInstance();
                    cal.add(Calendar.YEAR, -1);
                    cal.add(Calendar.DATE, -1);
                    cal.set(Calendar.HOUR_OF_DAY, 0);
                    cal.set(Calendar.MINUTE, 0);
                    cal.set(Calendar.SECOND, 0);
                    Date startTime = cal.getTime();
                    cal.set(Calendar.HOUR_OF_DAY, 23);
                    cal.set(Calendar.MINUTE, 59);
                    cal.set(Calendar.SECOND, 59);
                    Date endTime = cal.getTime();
                    List<Map<String,Object>> lastWaters = customDmaScadaMapper.getDeviceWater(0,
                            DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss"),
                            DateUtil.format(endTime, "yyyy-MM-dd HH:mm:ss"),
                            IndexTypeEnum.CUM_NET_FLOW.getIndexType());
                    if (CollUtil.isNotEmpty(lastWaters)) {
                        List<Map<String,Object>> lastInYesterdayWaters = yesterdayWaters.stream()
                                .filter(e -> e.get("deviceId") != null
                                        && inDeviceIds.contains(Long.parseLong(String.valueOf(e.get("deviceId")))))
                                .collect(Collectors.toList());
                        List<Map<String,Object>> lastOutYesterdayWaters = yesterdayWaters.stream()
                                .filter(e -> e.get("deviceId") != null
                                        && outDeviceIds.contains(Long.parseLong(String.valueOf(e.get("deviceId")))))
                                .collect(Collectors.toList());
                        if (CollectionUtils.isNotEmpty(lastInYesterdayWaters) && CollectionUtils.isNotEmpty(lastOutYesterdayWaters)) {
                            BigDecimal lastInWater = BigDecimal.valueOf(lastInYesterdayWaters.stream()
                                    .filter(e -> e.get("diff") != null)
                                    .mapToDouble(e -> Double.parseDouble(String.valueOf(e.get("diff")))).sum());
                            BigDecimal lastOutWater = BigDecimal.valueOf(lastOutYesterdayWaters.stream()
                                    .filter(e -> e.get("diff") != null)
                                    .mapToDouble(e -> Double.parseDouble(String.valueOf(e.get("diff")))).sum());
                            res.setLastYearYesterdayLeakWater(lastInWater.subtract(lastOutWater));
                        }
                    }
                }
            }
        }

        // 计算上月漏损率和漏损量/产销差和产销差水量
        res.setLastMonthLeakWater(BigDecimal.ZERO);
        res.setLastMonthLeakRate(00.00f);
        res.setLastYearLastMonthLeakWater(BigDecimal.ZERO);
        res.setLastMonthNrwWater(BigDecimal.ZERO);
        res.setLastMonthNrw(00.00f);
        res.setLastYearLastMonthNrwWater(BigDecimal.ZERO);
        List<Map<String,Object>> monthWaters = customDmaScadaMapper.getLastMonthWater(IndexTypeEnum.CUM_NET_FLOW.getIndexType());
        if (CollectionUtils.isNotEmpty(dmaregions) && CollectionUtils.isNotEmpty(monthWaters)) {
            List<Long> dmaregionIds = dmaregions.stream().map(Dmaregion::getId).collect(Collectors.toList());
            List<Dmaandobser> inDmaandobsers = dmaandobserService.list(new QueryWrapper<Dmaandobser>().lambda()
                    .in(Dmaandobser::getDmaId, dmaregionIds)
                    .eq(Dmaandobser::getIsout, "0"));
            List<Dmaandobser> outDmaandobsers = dmaandobserService.list(new QueryWrapper<Dmaandobser>().lambda()
                    .in(Dmaandobser::getDmaId, dmaregionIds)
                    .eq(Dmaandobser::getIsout, "1"));
            if (CollectionUtils.isNotEmpty(inDmaandobsers) && CollectionUtils.isNotEmpty(outDmaandobsers)) {
                List<Long> inDeviceIds = inDmaandobsers.stream().map(Dmaandobser::getTableId).collect(Collectors.toList());
                List<Long> outDeviceIds = outDmaandobsers.stream().map(Dmaandobser::getTableId).collect(Collectors.toList());
                List<Map<String,Object>> inYesterdayWaters = monthWaters.stream()
                        .filter(e -> e.get("deviceId") != null
                                && inDeviceIds.contains(Long.parseLong(String.valueOf(e.get("deviceId")))))
                        .collect(Collectors.toList());
                List<Map<String,Object>> outYesterdayWaters = monthWaters.stream()
                        .filter(e -> e.get("deviceId") != null
                                && outDeviceIds.contains(Long.parseLong(String.valueOf(e.get("deviceId")))))
                        .collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(inYesterdayWaters) && CollectionUtils.isNotEmpty(outYesterdayWaters)) {
                    BigDecimal inWater = BigDecimal.valueOf(inYesterdayWaters.stream()
                            .filter(e -> e.get("diff") != null)
                            .mapToDouble(e -> Double.parseDouble(String.valueOf(e.get("diff")))).sum());
                    BigDecimal outWater = BigDecimal.valueOf(outYesterdayWaters.stream()
                            .filter(e -> e.get("diff") != null)
                            .mapToDouble(e -> Double.parseDouble(String.valueOf(e.get("diff")))).sum());
                    res.setLastMonthLeakWater(inWater.subtract(outWater));
                    res.setLastMonthLeakRate(inWater.subtract(outWater).divide(inWater, 4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).floatValue());
                    res.setLastMonthNrwWater(inWater.subtract(outWater));
                    res.setLastMonthNrw(inWater.subtract(outWater).divide(inWater, 4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).floatValue());
                    // 计算去年上月漏损量/产销差水量
                    Calendar cal = Calendar.getInstance();
                    cal.add(Calendar.YEAR, -1);
                    cal.add(Calendar.MONTH, -1);
                    cal.set(Calendar.DAY_OF_MONTH, 1);
                    cal.set(Calendar.HOUR_OF_DAY, 0);
                    cal.set(Calendar.MINUTE, 0);
                    cal.set(Calendar.SECOND, 0);
                    Date startTime = cal.getTime();
                    int lastDay;
                    if(DateUtil.format(cal.getTime(),"MM").equals("02")) {
                        lastDay = cal.getLeastMaximum(Calendar.DAY_OF_MONTH);
                    }else {
                        lastDay = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
                    }
                    cal.set(Calendar.DAY_OF_MONTH, lastDay);
                    cal.set(Calendar.HOUR_OF_DAY, 23);
                    cal.set(Calendar.MINUTE, 59);
                    cal.set(Calendar.SECOND, 59);
                    Date endTime = cal.getTime();
                    List<Map<String,Object>> lastWaters = customDmaScadaMapper.getDeviceWater(1,
                            DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss"),
                            DateUtil.format(endTime, "yyyy-MM-dd HH:mm:ss"),
                            IndexTypeEnum.CUM_NET_FLOW.getIndexType());
                    if (CollUtil.isNotEmpty(lastWaters)) {
                        List<Map<String,Object>> lastInYesterdayWaters = yesterdayWaters.stream()
                                .filter(e -> e.get("deviceId") != null
                                        && inDeviceIds.contains(Long.parseLong(String.valueOf(e.get("deviceId")))))
                                .collect(Collectors.toList());
                        List<Map<String,Object>> lastOutYesterdayWaters = yesterdayWaters.stream()
                                .filter(e -> e.get("deviceId") != null
                                        && outDeviceIds.contains(Long.parseLong(String.valueOf(e.get("deviceId")))))
                                .collect(Collectors.toList());
                        if (CollectionUtils.isNotEmpty(lastInYesterdayWaters) && CollectionUtils.isNotEmpty(lastOutYesterdayWaters)) {
                            BigDecimal lastInWater = BigDecimal.valueOf(lastInYesterdayWaters.stream()
                                    .filter(e -> e.get("diff") != null)
                                    .mapToDouble(e -> Double.parseDouble(String.valueOf(e.get("diff")))).sum());
                            BigDecimal lastOutWater = BigDecimal.valueOf(lastOutYesterdayWaters.stream()
                                    .filter(e -> e.get("diff") != null)
                                    .mapToDouble(e -> Double.parseDouble(String.valueOf(e.get("diff")))).sum());
                            res.setLastYearLastMonthLeakWater(lastInWater.subtract(lastOutWater));
                            res.setLastYearLastMonthNrwWater(lastInWater.subtract(lastOutWater));
                        }
                    }
                }
            }
        }

        res.setHandleRate(75.11f);
        res.setWarnMessageNum(32);
        res.setLastMonthLeakWater(BigDecimal.valueOf(32211.71));
        res.setYesterdayLeakRate(15.01f);
        res.setYesterdayLeakWater(BigDecimal.valueOf(66.754));
        res.setLastYearYesterdayLeakWater(BigDecimal.valueOf(56.14));
        res.setLastMonthLeakRate(11.08f);
        res.setLastYearLastMonthLeakWater(BigDecimal.valueOf(30178.71));
        res.setLastMonthNrwWater(BigDecimal.valueOf(52.312));
        res.setLastMonthNrw(14.42f);
        res.setLastYearLastMonthNrwWater(BigDecimal.valueOf(42167.33));
        return res;
    }

    @Override
    public List<DmaSupplyWaterVo> chargeWaterAnalyse(Long dmaId, Integer statisticsType, String startDate, String endDate) {
        List<DmaSupplyWaterVo> res = new ArrayList<>();

        ScadaDataDto dto = new ScadaDataDto();
        dto.setTableType(2);
        dto.setTableName("STATISTICS_SCADA");
        dto.setStatisticsType(statisticsType);
        dto.setIndexType(IndexTypeEnum.CUM_NET_FLOW.getIndexType());
        dto.setStartDate(startDate);
        dto.setEndDate(endDate);
        dto.setIsBindWaterStation(1);
        List<Dmaandobser> dmaandobserList = dmaandobserService.list(new QueryWrapper<Dmaandobser>().lambda()
                .eq(Dmaandobser::getDmaId, dmaId)
                .eq(Dmaandobser::getIsout, "0"));
        if (CollUtil.isEmpty(dmaandobserList)) {
            return null;
        }
        dto.setDeviceIdList(dmaandobserList.stream().map(Dmaandobser::getTableId).collect(Collectors.toList()));
        List<ScadaIndexVo> scadaIndexList = customDmaScadaMapper.viewScadaData(dto);
        if (CollUtil.isEmpty(scadaIndexList)) {
            return null;
        }

        List<Date> betweenDate = getBetweenDate(startDate, endDate, statisticsType);
        for (Date date : betweenDate) {
            DmaSupplyWaterVo vo = new DmaSupplyWaterVo();
            vo.setScadaTime(DateUtil.format(date, "yyyy-MM-dd HH:mm:ss"));

            List<ScadaIndexVo> dateScadaIndexList = scadaIndexList.stream()
                    .filter(e -> e.getScadaTime().equals(DateUtil.format(date, "yyyy-MM-dd HH:mm:ss")))
                    .collect(Collectors.toList());
            if (CollUtil.isEmpty(dateScadaIndexList)) {
                continue;
            }
            List<Long> deviceInList = dmaandobserList.stream().filter(e -> e.getIsout().equals("0")).map(Dmaandobser::getTableId).collect(Collectors.toList());
            BigDecimal totalSupplyWater = BigDecimal.valueOf(dateScadaIndexList.stream().filter(e -> deviceInList.contains(e.getDeviceId())).mapToDouble(s -> Double.parseDouble(s.getIndexValue())).sum());
            vo.setSupplyWater(totalSupplyWater);

            Map<String, List<ScadaIndexVo>> scadaIndexCollect = dateScadaIndexList.stream().collect(Collectors.groupingBy(ScadaIndexVo::getDeviceCode));
            List<MeterWaterVo> meterList = new ArrayList<>();
            scadaIndexCollect.forEach((k, v)->{
                MeterWaterVo meter = new MeterWaterVo();
                meter.setMeterCode(k);
                meter.setMeterName(v.get(0).getDeviceName());
                BigDecimal meterSupplyWater = BigDecimal.valueOf(v.stream().mapToDouble(e -> Double.parseDouble(e.getIndexValue())).sum());
                meter.setSupplyWater(meterSupplyWater);
                meter.setProp(meterSupplyWater.divide(totalSupplyWater, 4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)));
                meterList.add(meter);
            });
            vo.setMeterList(meterList);

            res.add(vo);
        }

        return res;
    }

    @Override
    public List<DmaLeakageVo> leakageRateAnalyse(Integer level, Long dmaId, String isLeakageWarn, String isMinNightWarn, String startDate, String endDate, Integer statisticsType) {
        List<DmaLeakageVo> res = new ArrayList<>();
        List<Dmaregion> dmaRegionAll = dmaregionService.list(new QueryWrapper<Dmaregion>().lambda()
                        .eq(dmaId != null, Dmaregion::getId, dmaId)
                        .eq(level != null, Dmaregion::getPartitionLevel, level));
        List<TfYwpnDmaregionotherW> dmaRegionOtherAll = dmaregionotherService.list(new QueryWrapper<TfYwpnDmaregionotherW>().lambda()
                .eq(dmaId != null, TfYwpnDmaregionotherW::getDmaId, dmaId));
        List<TableUsageVo> list = customDmaScadaMapper.getDiffByDevice(startDate, endDate, IndexTypeEnum.CUM_NET_FLOW.getIndexType(), statisticsType);
        List<YxMeterVo> yxMeterVoList = customDmaScadaMapper.viewYxData(statisticsType, startDate, endDate);
        // 计算年取水栓消防栓用水量（m³）
        ScadaDataDto dto = new ScadaDataDto();
        List<String> deviceTypeList = new ArrayList<>();
        deviceTypeList.add("6");
        deviceTypeList.add("7");
        dto.setDeviceTypeList(deviceTypeList);
        dto.setTableType(2);
        dto.setTableName("STATISTICS_SCADA");
        dto.setStatisticsType(statisticsType);
        dto.setIndexType(IndexTypeEnum.CUM_NET_FLOW.getIndexType());
        dto.setStartDate(startDate);
        dto.setEndDate(endDate);
        List<ScadaIndexVo> waterFireScadaList = customDmaScadaMapper.viewScadaData(dto);
        List<Map<String, Object>> minNightFlowList = customDmaScadaMapper.getMinNightFlow(startDate, endDate);
        List<Dmaandobser> dmaandobserList = dmaandobserService.list();
        List<Date> dates = getBetweenDate(startDate, endDate, statisticsType);
        dmaRegionAll.forEach(info -> {
            List<Dmaandobser> dmaandobserFinalIn = dmaandobserList.stream()
                    .filter(e -> Objects.equals(e.getDmaId(), info.getId()) && e.getIsout().equals("0"))
                    .collect(Collectors.toList());
            List<Dmaandobser> dmaandobserFinalOut = dmaandobserList.stream()
                    .filter(e -> Objects.equals(e.getDmaId(), info.getId()) && e.getIsout().equals("1"))
                    .collect(Collectors.toList());
            if (CollUtil.isEmpty(dmaandobserFinalIn) || CollUtil.isEmpty(dmaandobserFinalOut)) {
                return;
            }
            List<String> dmaTableIds = dmaandobserList.stream()
                    .filter(e -> Objects.equals(e.getDmaId(), info.getId()))
                    .map(e -> String.valueOf(e.getTableId()))
                    .collect(Collectors.toList());
            List<Long> deviceListOut = dmaandobserFinalOut.stream().map(Dmaandobser::getTableId).collect(Collectors.toList());
            List<Long> deviceListIn = dmaandobserFinalIn.stream().map(Dmaandobser::getTableId).collect(Collectors.toList());
            for (Date date : dates) {
                String dateQuery = "";
                switch (statisticsType) {
                    case 0:
                        dateQuery = DateUtil.format(date, "yyyy-MM-dd HH");
                        break;
                    case 1:
                        dateQuery = DateUtil.format(date, "yyyy-MM-dd");
                        break;
                    case 2:
                        dateQuery = DateUtil.format(date, "yyyy-MM");
                        break;
                    case 3:
                        dateQuery = DateUtil.format(date, "yyyy");
                        break;
                }
                String finalDateQuery = dateQuery;
                List<TableUsageVo> listFinalOut = list.stream()
                        .filter(e -> e.getDeviceId() != null && e.getScadaTime() != null
                                && deviceListOut.contains(Long.parseLong(String.valueOf(e.getDeviceId()))) && finalDateQuery.equals(String.valueOf(e.getScadaTime())))
                        .collect(Collectors.toList());
                List<TableUsageVo> listFinalIn = list.stream()
                        .filter(e -> e.getDeviceId() != null && e.getScadaTime() != null
                                && deviceListIn.contains(Long.parseLong(String.valueOf(e.getDeviceId()))) && finalDateQuery.equals(String.valueOf(e.getScadaTime())))
                        .collect(Collectors.toList());
                DmaLeakageVo dmaLeakageVo = new DmaLeakageVo();
                dmaLeakageVo.setDmaId(info.getId());
                dmaLeakageVo.setDmaName(info.getDmaName());
                dmaLeakageVo.setPartitionLevel(info.getPartitionLevel());
                dmaLeakageVo.setPid(info.getPid());
                List<TfYwpnDmaregionotherW> dmaRegionOtherAllFinal = dmaRegionOtherAll.stream().filter(e -> e.getDmaId().equals(info.getId())).collect(Collectors.toList());
                if (CollUtil.isNotEmpty(dmaRegionOtherAllFinal)) {
                    dmaLeakageVo.setNightJusticeWater(dmaRegionOtherAllFinal.get(0).getNightJusticeWater());
                }
                dmaLeakageVo.setLeakDown(info.getLeakDown());
                dmaLeakageVo.setLeakUp(info.getLeakUp());
                dmaLeakageVo.setType(info.getType());
                dmaLeakageVo.setScadaTime(dateQuery);
                dmaLeakageVo.setIsLeakageWarn("0");
                dmaLeakageVo.setLeakageRate(0.0f);
                if (CollUtil.isNotEmpty(listFinalOut) && CollUtil.isNotEmpty(listFinalIn)) {
                    dmaLeakageVo.setOutWaterMeterInfos(listFinalOut);
                    dmaLeakageVo.setInWaterMeterInfos(listFinalIn);
                    dmaLeakageVo.setOutWaterMeterValue(BigDecimal.valueOf(listFinalOut.stream().mapToDouble(e -> Double.parseDouble(String.valueOf(e.getTableUsage()))).sum()));
                    dmaLeakageVo.setInWaterMeterValue(BigDecimal.valueOf(listFinalIn.stream().mapToDouble(e -> Double.parseDouble(String.valueOf(e.getTableUsage()))).sum()));
                    BigDecimal diff = dmaLeakageVo.getInWaterMeterValue().subtract(dmaLeakageVo.getOutWaterMeterValue());
                    dmaLeakageVo.setDiffValue(diff);
                    float leakageRate = 0.0f;
                    if (dmaLeakageVo.getInWaterMeterValue().compareTo(BigDecimal.ZERO) != 0) {
                        leakageRate = diff.divide(dmaLeakageVo.getInWaterMeterValue(), 4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).floatValue();
                    }
                    dmaLeakageVo.setLeakageRate(leakageRate);
                    if (leakageRate < info.getLeakDown() || leakageRate > info.getLeakUp()) {
                        dmaLeakageVo.setIsLeakageWarn("1");
                    }
                }

                if (StringUtils.isNotEmpty(isLeakageWarn) && !isLeakageWarn.equals(dmaLeakageVo.getIsLeakageWarn())) {
                    continue;
                }

                List<YxMeterVo> yxMeterVoListFinal = yxMeterVoList.stream()
                        .filter(e -> e.getMeterCode() != null && e.getCensusDate() != null
                                && dmaTableIds.contains(String.valueOf(e.getMeterCode()))
                                && finalDateQuery.equals(String.valueOf(e.getCensusDate())))
                        .collect(Collectors.toList());
                if (CollUtil.isNotEmpty(yxMeterVoListFinal)) {
                    dmaLeakageVo.setRegisterUserWater(new BigDecimal(yxMeterVoListFinal.get(0).getMeterAmount()));
                }

                List<ScadaIndexVo> waterFireScadaListFinal = waterFireScadaList.stream()
                        .filter(e -> e.getDeviceId() != null && e.getScadaTime() != null
                                && dmaTableIds.contains(String.valueOf(e.getDeviceId()))
                                && DateUtil.format(date, "yyyy-MM-dd HH:mm:ss").equals(String.valueOf(e.getScadaTime())))
                        .collect(Collectors.toList());
                if (CollUtil.isNotEmpty(waterFireScadaListFinal)) {
                    dmaLeakageVo.setFreeWater(new BigDecimal(waterFireScadaListFinal.get(0).getIndexValue()));
                }

                List<Map<String, Object>> minNightFlowListFinal = minNightFlowList.stream().filter(e -> dmaTableIds.contains(String.valueOf(e.get("deviceId")))).collect(Collectors.toList());
                if (CollUtil.isNotEmpty(minNightFlowListFinal)) {
                    Map<String, List<Map<String, Object>>> minNightFlowListGroup = minNightFlowListFinal.stream().collect(Collectors.groupingBy(e -> String.valueOf(e.get("minTime"))));
                    final BigDecimal[] minValF = {BigDecimal.ZERO};
                    minNightFlowListGroup.forEach((k, v) -> {
                        List<Map<String, Object>> minVals = minNightFlowListGroup.get(k);
                        final BigDecimal[] minVal = {BigDecimal.ZERO};
                        minVals.forEach(minNightFlow -> minVal[0] = minVal[0].add(new BigDecimal(String.valueOf(minNightFlow.get("minVal")))));
                        if (minValF[0].compareTo(BigDecimal.ZERO) == 0 || minValF[0].compareTo(minVal[0]) > 0) {
                            minValF[0] = minVal[0];
                        }
                    });
                    dmaLeakageVo.setMinNightFlow(minValF[0]);
                    dmaLeakageVo.setIsMinNightWarn("0");
                    if (dmaLeakageVo.getNightJusticeWater() != null
                            && dmaLeakageVo.getMinNightFlow().compareTo(BigDecimal.valueOf(dmaLeakageVo.getNightJusticeWater())) > 0) {
                        dmaLeakageVo.setIsMinNightWarn("1");
                    }
                }
                if (StringUtils.isNotBlank(isMinNightWarn) && !Objects.equals(isMinNightWarn, dmaLeakageVo.getIsMinNightWarn())) {
                    continue;
                }

                res.add(dmaLeakageVo);
            }
        });
        if (CollUtil.isNotEmpty(res)) {
            return res.stream().sorted(Comparator.comparing(DmaLeakageVo::getLeakageRate).reversed()).collect(Collectors.toList());
        } else {
            return null;
        }
    }

    @Override
    public List<DmaAboutVo> regionAboutInfo(List<Long> dmaIdList) {
        List<DmaAboutVo> res = new ArrayList<>();
        List<Dmaregion> dmaRegionAll = dmaregionService.list(new QueryWrapper<Dmaregion>().lambda()
                .in(CollUtil.isNotEmpty(dmaIdList), Dmaregion::getId, dmaIdList));
        List<TfYwpnDmaregionotherW> dmaRegionOtherAll = dmaregionotherService.list(new QueryWrapper<TfYwpnDmaregionotherW>().lambda()
                .in(CollUtil.isNotEmpty(dmaIdList), TfYwpnDmaregionotherW::getDmaId, dmaIdList));
        List<Dmaandobser> dmaandobserList = dmaandobserService.list();
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.MONTH, -1);
        cal.set(Calendar.DAY_OF_MONTH, 1);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        Date startTime = cal.getTime();
        int lastDay;
        if(DateUtil.format(cal.getTime(),"MM").equals("02")) {
            lastDay = cal.getLeastMaximum(Calendar.DAY_OF_MONTH);
        }else {
            lastDay = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
        }
        cal.set(Calendar.DAY_OF_MONTH, lastDay);
        cal.set(Calendar.HOUR_OF_DAY, 23);
        cal.set(Calendar.MINUTE, 59);
        cal.set(Calendar.SECOND, 59);
        Date endTime = cal.getTime();
        List<TableUsageVo> lastMonthList = customDmaScadaMapper.getDiffByDevice(DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss"), DateUtil.format(endTime, "yyyy-MM-dd HH:mm:ss"), IndexTypeEnum.CUM_NET_FLOW.getIndexType(), 1);

        cal.setTime(new Date());
        cal.add(Calendar.DATE, -1);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        startTime = cal.getTime();
        cal.set(Calendar.HOUR_OF_DAY, 23);
        cal.set(Calendar.MINUTE, 59);
        cal.set(Calendar.SECOND, 59);
        endTime = cal.getTime();
        List<Dmanightflow> yesterdayNightFlows = dmanightflowService.list(new QueryWrapper<Dmanightflow>()
                .le("TO_CHAR(MINTIME, 'yyyy-mm-dd hh24:mi:ss')", DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss"))
                .ge("TO_CHAR(MINTIME, 'yyyy-mm-dd hh24:mi:ss')", DateUtil.format(endTime, "yyyy-MM-dd HH:mm:ss")));

        // 计算昨日用水量（m³）
        ScadaDataDto dto = new ScadaDataDto();
        dto.setTableType(2);
        dto.setTableName("STATISTICS_SCADA");
        dto.setStatisticsType(0);
        dto.setIndexType(IndexTypeEnum.CUM_NET_FLOW.getIndexType());
        dto.setStartDate(DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss"));
        dto.setEndDate(DateUtil.format(endTime, "yyyy-MM-dd HH:mm:ss"));
        List<ScadaIndexVo> yesterdayWaterList = customDmaScadaMapper.viewScadaData(dto);
        dmaRegionAll.forEach(dmaregion -> {
            DmaAboutVo dmaAboutVo = new DmaAboutVo();
            BeanUtils.copyProperties(dmaregion, dmaAboutVo);
            List<TfYwpnDmaregionotherW> dmaRegionOtherFinal = dmaRegionOtherAll.stream().filter(e -> e.getDmaId().equals(dmaregion.getId())).collect(Collectors.toList());
            if (CollUtil.isNotEmpty(dmaRegionOtherFinal)) {
                dmaAboutVo.setDmaregionother(dmaRegionOtherFinal.get(0));
            }

            List<Dmaandobser> dmaandobserFinalIn = dmaandobserList.stream()
                    .filter(e -> Objects.equals(e.getDmaId(), dmaregion.getId()) && e.getIsout().equals("0"))
                    .collect(Collectors.toList());
            List<Dmaandobser> dmaandobserFinalOut = dmaandobserList.stream()
                    .filter(e -> Objects.equals(e.getDmaId(), dmaregion.getId()) && e.getIsout().equals("1"))
                    .collect(Collectors.toList());
            if (CollUtil.isNotEmpty(dmaandobserFinalIn) && CollUtil.isNotEmpty(dmaandobserFinalOut)) {
                List<Long> dmaTableIds = dmaandobserList.stream()
                        .filter(e -> Objects.equals(e.getDmaId(), dmaregion.getId()))
                        .map(Dmaandobser::getTableId)
                        .collect(Collectors.toList());
                List<Long> deviceListOut = dmaandobserFinalOut.stream().map(Dmaandobser::getTableId).collect(Collectors.toList());
                List<Long> deviceListIn = dmaandobserFinalIn.stream().map(Dmaandobser::getTableId).collect(Collectors.toList());
                List<TableUsageVo> listFinalOut = lastMonthList.stream()
                        .filter(e -> e.getDeviceId() != null && e.getScadaTime() != null
                                && deviceListOut.contains(Long.parseLong(String.valueOf(e.getDeviceId()))))
                        .collect(Collectors.toList());
                List<TableUsageVo> listFinalIn = lastMonthList.stream()
                        .filter(e -> e.getDeviceId() != null && e.getScadaTime() != null
                                && deviceListIn.contains(Long.parseLong(String.valueOf(e.getDeviceId()))))
                        .collect(Collectors.toList());
                if (CollUtil.isNotEmpty(listFinalOut) && CollUtil.isNotEmpty(listFinalIn)) {
                    BigDecimal outWaterMeterValue = BigDecimal.valueOf(listFinalOut.stream().mapToDouble(e -> Double.parseDouble(String.valueOf(e.getTableUsage()))).sum());
                    BigDecimal inWaterMeterValue = BigDecimal.valueOf(listFinalIn.stream().mapToDouble(e -> Double.parseDouble(String.valueOf(e.getTableUsage()))).sum());
                    BigDecimal diff = inWaterMeterValue.subtract(outWaterMeterValue);
                    float leakageRate = 0.0f;
                    if (inWaterMeterValue.compareTo(BigDecimal.ZERO) != 0) {
                        leakageRate = diff.divide(inWaterMeterValue, 4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).floatValue();
                    }
                    dmaAboutVo.setLastMonthLeakage(leakageRate);
                    dmaAboutVo.setLastMonthNrw(leakageRate);
                }

                List<Dmanightflow> minNightFlowListFinal = yesterdayNightFlows.stream().filter(e -> dmaTableIds.contains(e.getTableId())).collect(Collectors.toList());
                if (CollUtil.isNotEmpty(minNightFlowListFinal)) {
                    Map<Date, List<Dmanightflow>> minNightFlowListGroup = minNightFlowListFinal.stream().collect(Collectors.groupingBy(Dmanightflow::getMintime));
                    final BigDecimal[] minValF = {BigDecimal.ZERO};
                    AtomicReference<Date> minTime = new AtomicReference<>(new Date());
                    minNightFlowListGroup.forEach((k, v) -> {
                        List<Dmanightflow> minVals = minNightFlowListGroup.get(k);
                        final BigDecimal[] minVal = {BigDecimal.ZERO};
                        minVals.forEach(minNightFlow -> minVal[0] = minVal[0].add(BigDecimal.valueOf(minNightFlow.getMinval())));
                        if (minValF[0] == null || minValF[0].compareTo(minVal[0]) > 0) {
                            minValF[0] = minVal[0];
                            minTime.set(k);
                        }
                    });
                    dmaAboutVo.setYesterdayMinNightFlow(minValF[0]);
                    dmaAboutVo.setYesterdayMinNightTime(minTime.get());
                    dmaAboutVo.setYesterdayMinNightWarn(minNightFlowListFinal.stream().filter(e -> e.getUsfl().equals("1")).count());
                }

                if (CollUtil.isNotEmpty(yesterdayWaterList)) {
                    List<ScadaIndexVo> yesterdayWaterListFinal = yesterdayWaterList.stream().filter(e -> deviceListIn.contains(e.getDeviceId())).collect(Collectors.toList());
                    dmaAboutVo.setYesterdayWater(CollUtil.isNotEmpty(yesterdayWaterListFinal) ? BigDecimal.valueOf(yesterdayWaterListFinal.stream().mapToDouble(e -> Double.parseDouble(String.valueOf(e.getIndexValue()))).sum()):BigDecimal.ZERO);
                }
            }

            res.add(dmaAboutVo);
        });
        return res;
    }

    @Override
    public List<DmaWaterAboutVo> getDmaWaterAbout(Long dmaId, Integer statisticsType, String startDate, String endDate) {
        List<DmaWaterAboutVo> res = new ArrayList<>();

        Dmaregion dmaregion = dmaregionService.getById(dmaId);

        // 计算供水量
        ScadaDataDto dto = new ScadaDataDto();
        dto.setTableType(2);
        dto.setTableName("STATISTICS_SCADA");
        dto.setStatisticsType(statisticsType);
        dto.setIndexType(IndexTypeEnum.CUM_NET_FLOW.getIndexType());
        dto.setStartDate(startDate);
        dto.setEndDate(endDate);
        dto.setIsBindWaterStation(1);
        List<Dmaandobser> dmaandobserList = dmaandobserService.list(new QueryWrapper<Dmaandobser>().lambda().eq(Dmaandobser::getDmaId, dmaId));
        dto.setDeviceIdList(dmaandobserList.stream().map(Dmaandobser::getTableId).collect(Collectors.toList()));
        List<ScadaIndexVo> scadaIndexListAll = customDmaScadaMapper.viewScadaDataGroupTime(dto);
        dto.setDeviceIdList(dmaandobserList.stream().filter(e -> e.getIsout().equals("0")).map(Dmaandobser::getTableId).collect(Collectors.toList()));
        List<ScadaIndexVo> scadaIndexListIn = customDmaScadaMapper.viewScadaDataGroupTime(dto);

        // 计算计费用水量
        List<YxMeterVo> yxMeterVoList = customDmaScadaMapper.viewYxData(statisticsType, startDate, endDate);

        // 计算年取水栓消防栓用水量（m³）
        ScadaDataDto dto1 = new ScadaDataDto();
        List<String> deviceTypeList = new ArrayList<>();
        deviceTypeList.add("6");
        deviceTypeList.add("7");
        dto1.setDeviceTypeList(deviceTypeList);
        dto1.setTableType(2);
        dto1.setTableName("STATISTICS_SCADA");
        dto1.setStatisticsType(statisticsType);
        dto1.setIndexType(IndexTypeEnum.CUM_NET_FLOW.getIndexType());
        dto1.setStartDate(startDate);
        dto1.setEndDate(endDate);
        List<ScadaIndexVo> waterFireScadaList = customDmaScadaMapper.viewScadaDataGroupTime(dto1);

        List<Long> deviceIdsAll = dmaandobserList.stream().map(Dmaandobser::getTableId).collect(Collectors.toList());
        List<String> deviceCodesAll = deviceArchiveManageService.list(new QueryWrapper<TfDeviceArchiveManage>().lambda()
                .in(TfDeviceArchiveManage::getId, deviceIdsAll))
                .stream().map(TfDeviceArchiveManage::getCode).collect(Collectors.toList());
        List<Long> deviceIdsIn = dmaandobserList.stream().filter(e -> e.getIsout().equals("0")).map(Dmaandobser::getTableId).collect(Collectors.toList());
        List<Long> deviceIdsOut = dmaandobserList.stream().filter(e -> e.getIsout().equals("1")).map(Dmaandobser::getTableId).collect(Collectors.toList());

        List<Map<String,Object>> leakNumAll = customDmaScadaMapper.getLeakNumGroupTime(dmaId, statisticsType);

        List<Date> betweenDate = getBetweenDate(startDate, endDate, statisticsType);
        betweenDate.forEach(date -> {
            DmaWaterAboutVo vo = new DmaWaterAboutVo();
            vo.setRegionId(dmaId);
            vo.setRegionName(dmaregion.getDmaName());

            String dateQuery = "";
            switch (statisticsType) {
                case 0:
                    dateQuery = DateUtil.format(date, "yyyy-MM-dd HH");
                    break;
                case 1:
                    dateQuery = DateUtil.format(date, "yyyy-MM-dd");
                    break;
                case 2:
                    dateQuery = DateUtil.format(date, "yyyy-MM");
                    break;
                case 3:
                    dateQuery = DateUtil.format(date, "yyyy");
                    break;
            }
            String finalDateQuery = dateQuery;

            List<ScadaIndexVo> scadaIndexListInFinal = scadaIndexListIn.stream()
                    .filter(e -> finalDateQuery.equals(String.valueOf(e.getScadaTime())))
                    .collect(Collectors.toList());
            if (CollUtil.isNotEmpty(scadaIndexListInFinal)) {
                vo.setSupplyWater(BigDecimal.valueOf(scadaIndexListInFinal.stream().mapToDouble(s -> Double.parseDouble(s.getIndexValue())).sum()));
            }

            List<ScadaIndexVo> waterFireScadaListFinal = waterFireScadaList.stream()
                    .filter(e -> finalDateQuery.equals(String.valueOf(e.getScadaTime())))
                    .collect(Collectors.toList());
            if (CollUtil.isNotEmpty(waterFireScadaListFinal)) {
                vo.setFreeUseWater(BigDecimal.valueOf(waterFireScadaListFinal.stream().mapToDouble(s -> Double.parseDouble(s.getIndexValue())).sum()));
            }

            List<YxMeterVo> yxMeterVoListFinal = yxMeterVoList.stream()
                    .filter(e -> deviceCodesAll.contains(String.valueOf(e.getMeterCode()))
                            && finalDateQuery.equals(String.valueOf(e.getCensusDate())))
                    .collect(Collectors.toList());
            if (CollUtil.isNotEmpty(yxMeterVoListFinal)) {
                vo.setChargeUseWater(new BigDecimal(yxMeterVoListFinal.get(0).getMeterAmount()));
            }

            // 计算漏失水量、产销差水量、漏损率、产销差率
            List<ScadaIndexVo> listFinalOut = scadaIndexListAll.stream()
                    .filter(e -> deviceIdsOut.contains(Long.parseLong(String.valueOf(e.getDeviceId()))))
                    .collect(Collectors.toList());
            List<ScadaIndexVo> listFinalIn = scadaIndexListAll.stream()
                    .filter(e -> deviceIdsIn.contains(Long.parseLong(String.valueOf(e.getDeviceId()))))
                    .collect(Collectors.toList());
            if (CollUtil.isNotEmpty(listFinalOut) && CollUtil.isNotEmpty(listFinalIn)) {
                BigDecimal outWaterMeterValue = BigDecimal.valueOf(listFinalOut.stream().mapToDouble(e -> Double.parseDouble(String.valueOf(e.getIndexValue()))).sum());
                BigDecimal inWaterMeterValue = BigDecimal.valueOf(listFinalIn.stream().mapToDouble(e -> Double.parseDouble(String.valueOf(e.getIndexValue()))).sum());
                BigDecimal diff = inWaterMeterValue.subtract(outWaterMeterValue);
                float leakageRate = 0.0f;
                if (inWaterMeterValue.compareTo(BigDecimal.ZERO) != 0) {
                    leakageRate = diff.divide(inWaterMeterValue, 4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).floatValue();
                }
                vo.setLeakageWater(diff);
                vo.setNrwWater(diff);
                vo.setLeakageRate(leakageRate);
                vo.setNrw(leakageRate);
            }

            List<Map<String,Object>> leakNumFinal = leakNumAll.stream().filter(e -> finalDateQuery.equals(String.valueOf(e.get("censusTime")))).collect(Collectors.toList());
            if (CollUtil.isNotEmpty(leakNumFinal)) {
                vo.setLeakNum(Integer.valueOf(String.valueOf(leakNumFinal.get(0).get("num"))));
            }

            res.add(vo);
        });
        return res;
    }

    @Override
    public List<NightFlowMonitorVo> nightFlowMonitor(Long deviceId, String startDate, String endDate) {
        List<NightFlowMonitorVo> res = new ArrayList<>();

        ScadaDataDto dto = new ScadaDataDto();
        dto.setTableType(2);
        dto.setTableName("STATISTICS_SCADA");
        dto.setStatisticsType(1);
        List<Long> deviceIdList = new ArrayList<>();
        deviceIdList.add(deviceId);
        dto.setDeviceIdList(deviceIdList);
        dto.setIndexType(IndexTypeEnum.INSTANTANEOUS_FLOW.getIndexType());
        dto.setStartDate(startDate);
        dto.setEndDate(endDate);
        List<ScadaIndexVo> scadaIndexList = customDmaScadaMapper.viewScadaData(dto);

        List<Map<String, Object>> minNightFlowList = customDmaScadaMapper.getMinNightFlowList(deviceId, startDate, endDate);

        List<Date> betweenDate = getBetweenDate(startDate, endDate, 1);
        betweenDate.forEach(date -> {
            String time = DateUtil.format(date, "yyyy-MM-dd") + " 00:00:00";

            List<ScadaIndexVo> scadaIndexListFinal = scadaIndexList.stream().filter(e -> e.getScadaTime().equals(time)).collect(Collectors.toList());
            List<Map<String, Object>> minNightFlowListFinal = minNightFlowList.stream()
                    .filter(e -> String.valueOf(e.get("minTime")).substring(0, 10).equals(DateUtil.format(date, "yyyy-MM-dd")))
                    .collect(Collectors.toList());
            if (CollUtil.isEmpty(minNightFlowListFinal)) {
                return;
            }

            NightFlowMonitorVo vo = new NightFlowMonitorVo();
            vo.setScadaTime(String.valueOf(minNightFlowListFinal.get(0).get("minTime")));
            vo.setAveDayFlow(new BigDecimal(String.valueOf(minNightFlowListFinal.get(0).get("minVal"))));
            if (CollUtil.isNotEmpty(scadaIndexListFinal)) {
                vo.setAveDayFlow(new BigDecimal(scadaIndexListFinal.get(0).getIndexValueAvg()));
            }
            if (vo.getMinFlow() != null && vo.getAveDayFlow() != null) {
                vo.setProp(vo.getMinFlow().divide(vo.getAveDayFlow(), 4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).floatValue());
            }
            res.add(vo);
        });
        return res;
    }


    /* ***************************************************自定义方法******************************************** */


    /**
     * 获取两个日期之间的所有日期
     * @param start 开始日期
     * @param end 结束日期
     * @param type 0小时 1天 2月 3年
     * @return 日期列表
     */
    List<Date> getBetweenDate(String start, String end, Integer type) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(DateUtil.parse(start, "yyyy-MM-dd HH:mm:ss"));
        List<Date> dateList = new ArrayList<>();
        while (DateUtil.format(cal.getTime(), "yyyy-MM-dd HH:mm:ss").compareTo(end) < 0) {
            switch (type) {
                case 0:
                    dateList.add(cal.getTime());
                    cal.add(Calendar.HOUR, +1);
                    break;
                case 1:
                    dateList.add(cal.getTime());
                    cal.add(Calendar.DATE, +1);
                    break;
                case 2:
                    dateList.add(cal.getTime());
                    cal.add(Calendar.MONTH, +1);
                    break;
                case 3:
                    dateList.add(cal.getTime());
                    cal.add(Calendar.YEAR, +1);
                    break;
            }
        }
        return dateList;
    }
}
