package com.tofly.gpsboot.service.audit.impl;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.tofly.api.UserTaskClient;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.gpsboot.common.Constant;
import com.tofly.gpsboot.common.system.*;
import com.tofly.gpsboot.exception.BusinessException;
import com.tofly.gpsboot.service.audit.IAuditHiddenTroubleEliminationServer;
import com.tofly.gpsboot.service.mq.IAuditMessageService;
import com.tofly.gpsboot.service.paln.CheckhistoryService;
import com.tofly.gpsboot.service.paln.PlantypeService;
import com.tofly.gpsboot.service.paln.task.PlanInspectUserService;
import com.tofly.gpsboot.service.paln.task.PlanService;
import com.tofly.gpsboot.service.paln.task.PlanchildService;
import com.tofly.gpsboot.service.report.IReportHiddenTroubleServer;
import com.tofly.gpsboot.service.trouble.ToubleclearService;
import com.tofly.gpsboot.service.trouble.ToubleplanService;
import com.tofly.gpsboot.service.trouble.TroubleService;
import com.tofly.gpsboot.util.DateTimeUtil;
import com.tofly.pojo.message.Usertask;
import com.tofly.pojo.plan.Plan;
import com.tofly.pojo.plan.Planchild;
import com.tofly.pojo.plan.Plantype;
import com.tofly.pojo.plan.dto.ReportAuditDto;
import com.tofly.pojo.trouble.Toubleclear;
import com.tofly.pojo.trouble.Toubleplan;
import com.tofly.pojo.trouble.Trouble;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;


/**
 * 隐患消除审核
 * @author Zev
 */
@Service("AUDIT_TYPE_HIDDEN_TROUBLE_ELIMINATION")
public class AuditHiddenTroubleEliminationServiceImpl implements IAuditHiddenTroubleEliminationServer {

    @Autowired
    private ToubleclearService toubleclearService;

    @Autowired
    private TroubleService troubleService;

    @Autowired
    private CheckhistoryService checkhistoryService;

    @Autowired
    private IAuditMessageService iAuditMessageService;

    @Autowired
    private UserTaskClient userTaskClient;
    @Autowired
    private PlanService planService;

    @Autowired
    private ToubleplanService toubleplanService;
    @Autowired
    private PlanInspectUserService planInspectUserService;

    @Autowired
    private PlanchildService planchildService;

    @Autowired
    private PlantypeService plantypeService;

    @Autowired
    private IReportHiddenTroubleServer iReportHiddenTroubleServer;

    @Override
    public void audit(ReportAuditDto dto) throws BusinessException {
        Long id = dto.getId();

        Toubleclear toubleclear = toubleclearService.getById(id);
        Long toubleId = toubleclear.getToubleId();
        Trouble info = troubleService.getById(toubleId);
        // 当前隐患消除记录审核状态排除已审核通过数据
        if (AuditSateEnum.AUDIT_YES.getCode().equals(toubleclear.getAuditResult())) {
            throw new BusinessException(Constant.AUDIT_NOT_UNCOMMITTED_CODE, Constant.AUDIT_NOT_UNCOMMITTED);
        }
        String auditResult = dto.getAuditResult();
        Date date = DateTimeUtil.convertLDTToDate(LocalDateTime.now());
        String auditNotes = dto.getAuditNotes();
        Long auditId = info.getAuditUser();
        // 审核时间
        toubleclear.setAuditTime(date);
        // 审核人(同派工人)
        toubleclear.setAuditUser(auditId);
        // 审核状态
        toubleclear.setAuditResult(auditResult);
        // 审核意见
        toubleclear.setAuditNotes(auditNotes);

        if(AuditSateEnum.AUDIT_YES.getCode().equals(auditResult)){
            // 隐患主计划状态变为
            info.setState(TroubleSateEnum.TROUBLE_SATE_YES_ELIMINATE.getCode());
            info.setAuditResult(auditResult);
            // 对应的巡检计划状态全部变成4 计划完成
            Set<Long> toublePlanIdList = toubleplanService.list(new QueryWrapper<Toubleplan>().lambda().eq(Toubleplan::getTroubleId, toubleId)).stream().map(Toubleplan::getPlanId).collect(Collectors.toSet());
            List<Plan> list = planService.list(new QueryWrapper<Plan>().lambda().in(Plan::getId, toublePlanIdList));
            list.forEach(i->i.setPlanState(PlanStateEnum.PLAN_STATE_PLAN_COMPLETE.getCode()));
            planService.updateBatchById(list);


            //计划汇总
            List<Plantype> planTypeList=plantypeService.list(new QueryWrapper<Plantype>().lambda()
                    .in(Plantype::getPlanId,toublePlanIdList));

            //子计划完成情况，归属到消除人上面
            List<Planchild> planChildList=planchildService.list(new QueryWrapper<Planchild>().lambda()
                    .in(Planchild::getPlanId,toublePlanIdList).eq(Planchild::getRespUser, toubleclear.getSubmitUser()));
            if(planTypeList!=null && planChildList!=null &&planChildList.size()>0){
                planChildList.forEach(a->{
                    Plantype plantypeFinish= planTypeList.stream().filter(m->m.getPlanId().equals(a.getPlanId())).findFirst().orElse(new Plantype());
                    if(plantypeFinish!=null && plantypeFinish.getPlanId()!=null){
                        BigDecimal LastLength=plantypeFinish.getTotalLength().subtract(plantypeFinish.getOverLength());
                        BigDecimal LastCount=plantypeFinish.getTotalCount().subtract(plantypeFinish.getOverCount());
                        //有剩余未完成的
                        if(LastCount.compareTo(BigDecimal.ZERO)>0){
                            BigDecimal overCount=a.getOverCount().add(LastCount);
                            if(overCount.compareTo(plantypeFinish.getTotalCount())>0){
                                overCount=plantypeFinish.getTotalCount();
                            }
                            a.setOverCount(overCount);

                        }
                        if(LastLength.compareTo(BigDecimal.ZERO)>0){
                            BigDecimal overLength=a.getOverLen().add(LastLength);
                            if(overLength.compareTo(plantypeFinish.getTotalLength())>0){
                                overLength=plantypeFinish.getTotalLength();
                            }
                            a.setOverLen(overLength);

                        }
                    }
                    a.setPlanState(PlanStateEnum.PLAN_STATE_PLAN_COMPLETE.getCode());

                });
                planchildService.updateBatchById(planChildList);
            }
            if(planTypeList!=null && planChildList.size()>0){
                //总计划
                planTypeList.forEach(a->{
                    a.setOverLength(a.getTotalLength());
                    a.setOverCount(a.getTotalCount());
                    a.setPercent(a.getCompletionRate());

                });
                plantypeService.updateBatchById(planTypeList);
            }


            // 完成处理当前隐患待办计划任务信息
            toubleclearService.clearTroubleUserTaskPlan(toubleId);

        }else {
            // 审核不通过,上报状态再设置为 待上报
           // info.setIsSubmit(ReportSateEnum.REPORT_SUBMITTED.getCode());
            // 隐患主计划状态变为
            info.setState(TroubleSateEnum.TROUBLE_SATE_PATROL.getCode());
            //不同意，巡检，修改待办任务
            // 自动上报

            Set<Long> toublePlanIdList = toubleplanService.list(new QueryWrapper<Toubleplan>().lambda().eq(Toubleplan::getTroubleId, toubleId)).stream().map(Toubleplan::getPlanId).collect(Collectors.toSet());
            toublePlanIdList.forEach(item->{
                // 消除申请上报时候，消除对应的申请上报待办
                Usertask usertaskClear = new Usertask();
                usertaskClear.setTableName(MessageTopicEnum.TASK_PLAN_HIDDEN_TROUBLE_SEND.getTableName());
                usertaskClear.setTableId(item);
                List<Usertask> result = userTaskClient.getUserTaskInfo(usertaskClear).getResult();
                if(null != result && result.size()>0){
                    result.forEach(task->{
                        task.setIshandle(UserTaskStateEnumEnum.IS_HANDLER_NOT_STARTED.getCode());
                        userTaskClient.updateById(task);
                    });

                }
            });

        }
        troubleService.updateById(info);
        toubleclearService.updateById(toubleclear);
        // 存入当前计划审批历史信息
        checkhistoryService.recordHis(AuditTypeEnum.AUDIT_TYPE_HIDDEN_TROUBLE_ELIMINATION.getCode(), id, auditResult, auditNotes, auditId, date);


        // 发送mq消息(计划审核成功/失败)给上报对象到消息提示列表
        // 通知审核对象mq消息
        iAuditMessageService.auditTypeHiddenTroubleElimination(JSON.toJSONString(info));
        // 待办事项处理
        Usertask usertask = new Usertask();
        usertask.setTableName(MessageTopicEnum.REPORT_HIDDEN_TROUBLE_ELIMINATION.getTableName());
        usertask.setTableId(id);
        List<Usertask> result = userTaskClient.getUserTaskInfo(usertask).getResult();
        if(null != result && result.size()>0){
            result.forEach(task->{
                task.setHandleTime(DateTimeUtil.convertLDTToDate(LocalDateTime.now()));
                task.setHandleUser(SecurityUtils.getUserId());
                task.setIshandle(UserTaskStateEnumEnum.IS_HANDLER_COMPLETED.getCode());
                userTaskClient.updateById(task);
            });

        }
    }
}
