package com.tofly.watermeter.controller;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.watermeter.custom.dto.AiReadMeterDto;
import com.tofly.watermeter.entity.AiReadMeter;
import com.tofly.watermeter.service.AiReadMeterService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

/**
 * @author Bubble
 */
@RestController
@AllArgsConstructor
@RequestMapping("/aiReadMeter")
@Api(tags = "ai识别报表")
public class AiReadMeterController {

    private final AiReadMeterService aiReadMeterService;


    @GetMapping("/page")
    @ApiOperation("抄表异常明细报表")
    public ResultRespone page(Page page, AiReadMeterDto dto) {
        String month = dto.getMonth();
        String input = dto.getInput();
        Integer user = dto.getUser();
        Page result = aiReadMeterService.page(page, Wrappers.<AiReadMeter>lambdaQuery()
                .and(StringUtils.isNotBlank(input), ai -> ai.like(AiReadMeter::getYhbh, input).or().eq(AiReadMeter::getYhxm, input).or()
                        .like(AiReadMeter::getSbbh, input).or().like(AiReadMeter::getYhdz, input))
                .in(StringUtils.isNotBlank(month), AiReadMeter::getMonthPlan, month)
                .isNotNull(AiReadMeter::getCyl)
                .and(dto.getAbnormalType() == 0, w -> w.gt(AiReadMeter::getCyl, 1).or().lt(AiReadMeter::getCyl, -1))
                .and(dto.getAbnormalType() == 1, w -> w.eq(AiReadMeter::getState, -3).or().eq(AiReadMeter::getState, -1))
                .in(user != null, AiReadMeter::getCbry, user).orderByDesc(AiReadMeter::getCyl));
        return ResultRespone.success(result);
    }

    @GetMapping("/countAiIdentify")
    @ApiOperation("抄表照片识别率报表")
    public ResultRespone countAiIdentify(AiReadMeterDto dto) {
        return ResultRespone.success(aiReadMeterService.countAiIdentify(dto));
    }

    @GetMapping("/countMatching")
    @ApiOperation("抄表质检匹配率报表")
    public ResultRespone countMatching(AiReadMeterDto dto) {
        return ResultRespone.success(aiReadMeterService.countMatching(dto));
    }

    @GetMapping("/countError")
    @ApiOperation("抄表质检差异量统计报表")
    public ResultRespone countError(AiReadMeterDto dto) {
        return ResultRespone.success(aiReadMeterService.countError(dto));
    }

    @GetMapping("/insetAiReadMeter")
    @ApiOperation("插入ai识别数据")
    public ResultRespone insetAiReadMeter() {
        return ResultRespone.success(aiReadMeterService.insetAiReadMeter());
    }

    @GetMapping("/countAiIdentifyTotal")
    @ApiOperation("ai识别总报表")
    public ResultRespone countAiIdentifyTotal(String month) {
        return ResultRespone.success(aiReadMeterService.countAiIdentifyTotal(month));
    }

    @GetMapping("/countAiIdentifyTrend")
    @ApiOperation("ai识别趋势报表")
    public ResultRespone countAiIdentifyTrend(String startMonth, String endMonth) {
        return ResultRespone.success(aiReadMeterService.countAiIdentifyTrend(startMonth, endMonth));
    }

    @GetMapping("/countAiSuccess")
    @ApiOperation(("ai抄表员成功率趋势报表"))
    public ResultRespone countAiSuccess(String startMonth, String endMonth) {
        return ResultRespone.success(aiReadMeterService.countAiSuccess(startMonth, endMonth));
    }

    @GetMapping("/customsOfficerPage")
    @ApiOperation("ai稽查查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "month", value = "月份 yyyyMM",dataType = "string"),
            @ApiImplicitParam(name = "status", value = "0 已稽核 1 未稽查",dataType = "string")
    })
    public ResultRespone customsOfficerPage(Page page, String month, String status) {
        return ResultRespone.success(aiReadMeterService.customsOfficerPage(page, month, status));
    }

    @PutMapping(("/updateAiReadWater"))
    @ApiOperation("抄表稽查")
    public ResultRespone updateAiReadWater(@RequestBody AiReadMeter aiReadMeter) {
        return ResultRespone.success(aiReadMeterService.updateAiReadWater(aiReadMeter));
    }

}
