package com.tofly.watermeter.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.watermeter.entity.CensusPlanDetails;
import com.tofly.watermeter.service.CensusPlanDetailsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

/**
 * @author Bubble
 */
@RestController
@AllArgsConstructor
@RequestMapping("/censusPlanDetails")
@Api(tags = "普查计划详情接口")
public class CensusPlanDetailsController {

    private final CensusPlanDetailsService censusPlanDetailsService;

    @GetMapping("/queryByPlanId")
    @ApiOperation("根据计划id查询普查计划")
    public ResultRespone queryById(Long id) {
        return ResultRespone.success(censusPlanDetailsService.list(Wrappers.<CensusPlanDetails>lambdaQuery().eq(CensusPlanDetails::getPlanId, id)));
    }

    @PostMapping("/insert")
    @ApiOperation("普查计划详情添加")
    public ResultRespone insert(@RequestBody CensusPlanDetails censusPlanDetails) {
        return ResultRespone.success(censusPlanDetailsService.insert(censusPlanDetails));
    }

    @PutMapping("/update")
    @ApiOperation("普查计划修改")
    public ResultRespone update(@RequestBody CensusPlanDetails censusPlanDetails) {
        return ResultRespone.success(censusPlanDetailsService.update(censusPlanDetails));
    }

    @ApiOperation("删除计划详情")
    @DeleteMapping("/delete")
    public ResultRespone delete(String ids) {
        return ResultRespone.success(censusPlanDetailsService.deletePlanDetail(ids));
    }

    @PostMapping("/planStart")
    @ApiOperation("普查计划启用")
    public ResultRespone planStart(String ids,String year) {
        return ResultRespone.success(censusPlanDetailsService.planStart(ids,year));
    }

    @GetMapping("/getQyInfoByPlan")
    @ApiOperation("查询已制定计划区段")
    public ResultRespone getQyInfoByPlan(Page page, Long planId, String cbqyBm){
        return ResultRespone.success(censusPlanDetailsService.getQyInfoByPlan(page, planId, cbqyBm));
    }


    @GetMapping("/censusZoneCollect")
    @ApiOperation("普查计划--区域任务汇总")
    public ResultRespone censusZoneCollect(){
        return ResultRespone.success(censusPlanDetailsService.censusZoneCollect());
    }

    @GetMapping("/censusPlanCollect")
    @ApiOperation("普查计划--区域段任务汇总")
    public ResultRespone censusPlanCollect(String qybm){
        return ResultRespone.success(censusPlanDetailsService.censusPlanCollect(qybm));
    }

    @DeleteMapping("/deletePlanDetail")
    @ApiOperation("删除计划详情及普查记录")
    public ResultRespone deletePlanDetail(String ids){
        return ResultRespone.success();
    }

}
