package com.tofly.watermeter.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.watermeter.custom.entity.CensusQueryDto;
import com.tofly.watermeter.entity.TableCensus;
import com.tofly.watermeter.service.TableCensusService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

@RestController
@AllArgsConstructor
@RequestMapping("/tableCensus")
@Api(tags = "水表普查数据")
public class TableCensusController {

    private final TableCensusService tableCensusService;

    @PostMapping("/saveTableCensus")
    @ApiOperation("添加普查信息")
    public ResultRespone saveTableCensus(TableCensus tableCensus, @RequestParam(value = "electronicFiles", required = false) List<MultipartFile> electronicFiles, @RequestParam(value = "mechanicalFiles", required = false)List<MultipartFile> mechanicalFiles,@RequestParam(value = "tableFiles", required = false)List<MultipartFile> tableFiles) {
        return ResultRespone.success(tableCensusService.saveTableCensus(tableCensus, electronicFiles, mechanicalFiles,tableFiles));
    }

    @DeleteMapping("/deleteTableCensus")
    @ApiOperation("删除普查信息")
    public ResultRespone deleteTableCensus(String ids) {
        return ResultRespone.success(tableCensusService.deleteTableCensus(ids));
    }

    @PutMapping("/updateTableCensus")
    @ApiOperation("修改普查信息")
    public ResultRespone updateTableCensus(TableCensus tableCensus, @RequestParam(value = "electronicFiles", required = false) List<MultipartFile> electronicFiles, @RequestParam(value = "mechanicalFiles", required = false)List<MultipartFile> mechanicalFiles,@RequestParam(value = "tableFiles", required = false)List<MultipartFile> tableFiles) {
        return ResultRespone.success(tableCensusService.updateTableCensus(tableCensus, electronicFiles, mechanicalFiles, tableFiles));
    }


    @GetMapping("/planDownLoad")
    @ApiOperation("普查记录下载")
    public ResultRespone planDownLoad(Page page){
        return ResultRespone.success(tableCensusService.planDownLoad(page));
    }

    @GetMapping("/censusRecordsCount")
    @ApiOperation("普查记录统计")
    public ResultRespone censusRecordsCount(CensusQueryDto dto){
        return ResultRespone.success(tableCensusService.censusRecordsCount(dto));
    }

    @GetMapping("/censusRecordPage")
    @ApiOperation("普查记录列表")
    public ResultRespone censusRecordPage(Page page,CensusQueryDto dto){
        return ResultRespone.success(tableCensusService.censusRecordPage(page, dto));
    }


}
