package com.tofly.watermeter.custom.service.meterBusiness.export.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.alibaba.fastjson.JSONArray;
import com.tofly.watermeter.custom.service.meterBusiness.export.ExportCommonCodeService;
import com.tofly.watermeter.em.Constant;
import com.tofly.watermeter.em.ExcelExportFileTypeEnum;
import com.tofly.watermeter.strategy.ExcelStrategyContext;
import com.tofly.watermeter.util.ExcelUtil;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;

@Service
@AllArgsConstructor
public class ExportCommonCodeServiceImpl implements ExportCommonCodeService {

    private final ExcelStrategyContext excelStrategyContext;

    @Override
    public Object exportCode(HttpServletResponse response,String service, List list) {
        // 根据templateServiceType,templateFlag获取对应模板数据
        ExcelExportFileTypeEnum excelExportFileTypeEnum = ExcelExportFileTypeEnum.getExcelExportFileTypeEnum(service, "1");
        if (null == excelExportFileTypeEnum) {
            return null;
        }
        //将查询的结果转json字符串的格式，因为导出的时候只能传一个字符串
        String resultListToString = JSONArray.toJSONStringWithDateFormat(list,"yyyy-MM-dd HH:mm:ss");
        // 从模型类型枚举根据templateServiceType获取策略对应模板数据
        List<Map<String, Object>> dataList = excelStrategyContext.getExportData(service, "1", resultListToString);
        ExcelUtil.export(dataList.get(0), excelExportFileTypeEnum, Constant.FILE_PATH_TEST_EXPORT_EXCEL, response);
        return null;
    }

    /**
     * easyExcel导出
     *
     * @param fileName 文件名
     * @param list     数据
     * @param clazs    模板
     * @param response response
     */
    @Override
    public void easyExcelExport(String fileName, List list, Class clazs, HttpServletResponse response) throws Exception {
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/vnd.ms-excel");
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName + ".xlsx", "UTF-8"));
        EasyExcel.write(response.getOutputStream(), clazs)
                .excelType(ExcelTypeEnum.XLSX).autoCloseStream(Boolean.TRUE)
                .sheet(fileName).doWrite(list);
    }
}
