package com.tofly.watermeter.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.watermeter.custom.dto.AiReadMeterDto;
import com.tofly.watermeter.entity.AiReadMeter;
import com.tofly.watermeter.mapper.AiReadMeterMapper;
import com.tofly.watermeter.service.AiReadMeterService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author Bubble
 */
@Service
public class AiReadMeterServiceImpl extends ServiceImpl<AiReadMeterMapper, AiReadMeter> implements AiReadMeterService {
    /**
     * ai识别率统计
     *
     * @param dto dto
     * @return list
     */
    @Override
    public List<Map<String, Object>> countAiIdentify(AiReadMeterDto dto) {
        return baseMapper.countAiIdentify(dto);
    }

    /**
     * ai匹配率统计
     *
     * @param dto dto
     * @return list
     */
    @Override
    public List<Map<String, Object>> countMatching(AiReadMeterDto dto) {
        return baseMapper.countMatching(dto);
    }

    /**
     * 差异值统计
     *
     * @param dto dto
     * @return list
     */
    @Override
    public List<Map<String, Object>> countError(AiReadMeterDto dto) {
        return baseMapper.countError(dto);
    }

    /**
     * ai识别抄表数据添加
     *
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean insetAiReadMeter() {
        //需添加ai识别表数据
        List<AiReadMeter> aiReadMeters = baseMapper.quneryReadWater();
        if (CollectionUtils.isNotEmpty(aiReadMeters)) {
            saveBatch(aiReadMeters, 1000);
        }
        //路径为空的数据（路径为空说明是fileId是多个，单独处理）
        List<AiReadMeter> nullFilePath = list(Wrappers.<AiReadMeter>lambdaQuery().isNull(AiReadMeter::getFilePath));
        if (CollectionUtils.isNotEmpty(nullFilePath)) {
            String fileIdStr = nullFilePath.stream().map(AiReadMeter::getFileid).collect(Collectors.joining(","));
            if (StringUtils.isNotBlank(fileIdStr)) {
                List<Map> filePathMap = baseMapper.getFilePaths(fileIdStr);
                for (AiReadMeter rm : nullFilePath) {
                    String fileId = rm.getFileid();
                    if (StringUtils.isNotBlank(fileId)) {
                        List<String> fileids = Arrays.asList(fileId.split(","));
                        //拼接文件路径
                        String filePath = filePathMap.stream().filter(map -> fileids.contains(String.valueOf(map.get("id"))))
                                .map(map -> String.valueOf(map.get("path"))).collect(Collectors.joining(","));

                        update(Wrappers.<AiReadMeter>lambdaUpdate().eq(AiReadMeter::getYhbh, rm.getYhbh())
                                .eq(AiReadMeter::getMonthPlan, rm.getMonthPlan())
                                .set(AiReadMeter::getFilePath, filePath));
                    }
                }
            }
        }
        return true;
    }

    /**
     * ai识别总报表
     *
     * @param month 月份 yyyyMM
     * @return list
     */
    @Override
    public List<Map<String, Object>> countAiIdentifyTotal(String month) {
        return baseMapper.countAiIdentifyTotal(month);
    }

    /**
     * ai识别趋势报表
     *
     * @param startMonth 开始日期
     * @param endMonth   结束日期
     * @return list
     */
    @Override
    public List<Map<String, Object>> countAiIdentifyTrend(String startMonth, String endMonth) {
        return baseMapper.countAiIdentifyTrend(startMonth, endMonth);
    }

    /**
     * ai抄表员成功率趋势
     *
     * @param startMonth 开始日期
     * @param endMonth   结束日期
     * @return list
     */
    @Override
    public List<Map<String, Object>> countAiSuccess(String startMonth, String endMonth) {
        return baseMapper.countAiSuccess(startMonth, endMonth);
    }

    /**
     * 稽核列表数据分页查询
     *
     * @param page   page
     * @param month  月份 yyyyMM
     * @param status 稽核状态 1 已稽核 0 未稽核
     * @return page
     */
    @Override
    public Page<Map<String, Object>> customsOfficerPage(Page page, String month, String status) {
        return baseMapper.customsOfficerPage(page, month, status);
    }

    /**
     * 更新ai识别数据
     *
     * @param aiReadMeter aiReadMeter
     * @return boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateAiReadWater(AiReadMeter aiReadMeter) {
        return update(Wrappers.<AiReadMeter>lambdaUpdate()
                .eq(AiReadMeter::getYhbh,aiReadMeter.getYhbh())
                .eq(AiReadMeter::getMonthPlan,aiReadMeter.getMonthPlan())
                .set(AiReadMeter::getJczt,aiReadMeter.getJczt())
                .set(AiReadMeter::getJcjg,aiReadMeter.getJcjg())
                .set(AiReadMeter::getJcry, SecurityUtils.getUserId())
                .set(AiReadMeter::getJcryName,SecurityUtils.getUserName())
                .set(AiReadMeter::getJcsj,new Date())
                .set(StringUtils.isNotBlank(aiReadMeter.getJcremark()),AiReadMeter::getJcremark,aiReadMeter.getJcremark())
                .set(aiReadMeter.getJcds() != null,AiReadMeter::getJcds,aiReadMeter.getJcds()));
    }


}
