/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.watermeter.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.watermeter.custom.entity.KhSearch;
import com.tofly.watermeter.custom.entity.meterRead.CensusAllocation;
import com.tofly.watermeter.custom.entity.meterRead.CensusBhAndXhList;
import com.tofly.watermeter.custom.entity.meterRead.QyAndQd;
import com.tofly.watermeter.em.Constant;
import com.tofly.watermeter.em.ExcelExportFileTypeEnum;
import com.tofly.watermeter.entity.CensusBindZone;
import com.tofly.watermeter.entity.Qfkhjbxx;
import com.tofly.watermeter.mapper.CensusBindZoneMapper;
import com.tofly.watermeter.service.CensusBindZoneService;
import com.tofly.watermeter.util.JxlsUtils;
import com.tofly.watermeter.util.ThreadPoolUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 用户编号与区域、区域段绑定中间表
 *
 * @author wangjieliu
 * @date Mon Jun 07 00:00:00 CST 2021
 */
@Slf4j
@Service("censusBindZoneService")
@AllArgsConstructor
public class CensusBindZoneServiceImpl extends ServiceImpl<CensusBindZoneMapper, CensusBindZone> implements CensusBindZoneService {

    private final RedisTemplate redisTemplate;

    /**
     * 查询普查表绑定信息
     *
     * @param page     page
     * @param khSearch khSearch
     * @param response response
     * @return page
     */
    @Override
    public IPage<Qfkhjbxx> commonKh(Page<Qfkhjbxx> page, KhSearch khSearch, HttpServletResponse response) {
        String qybm = khSearch.getQybm();
        String qydbm = khSearch.getQydbm();
        String sbcj = khSearch.getSbcj();
        String yhzt = khSearch.getYhzt();
        String cbb = khSearch.getCbb();
        String ysxz = khSearch.getYsxz();
        List<String> cbqyList = null;
        List<String> cbqydList = null;
        List<String> sbcjList = null;
        List<String> yhztList = null;
        List<String> cbbList = null;
        List<String> ysxzList = null;
        String templateFlag = khSearch.getTemplateFlag();
        if (StringUtil.isNotEmpty(qybm)) {
            cbqyList = Arrays.asList(qybm.split(","));
        }
        if (StringUtil.isNotEmpty(qydbm)) {
            cbqydList = Arrays.asList(qydbm.split(","));
        }
        if (StringUtil.isNotEmpty(sbcj)) {
            sbcjList = Arrays.asList(sbcj.split(","));
        }
        if (StringUtil.isNotEmpty(yhzt)) {
            yhztList = Arrays.asList(yhzt.split(","));
        }
        if (StringUtil.isNotEmpty(cbb)) {
            cbbList = Arrays.asList(cbb.split(","));
        }
        if (StringUtil.isNotEmpty(ysxz)) {
            ysxzList = Arrays.asList(ysxz.split(","));
        }
        if ("1".equals(templateFlag)) {
            List<Qfkhjbxx> qfkhjbxxList = baseMapper.commonKh(ysxzList, cbqyList, cbqydList, khSearch, sbcjList, yhztList, cbbList);
            // 根据templateServiceType,templateFlag获取对应模板数据
            ExcelExportFileTypeEnum excelExportFileTypeEnum = ExcelExportFileTypeEnum.getExcelExportFileTypeEnum("clientInfoDeatils", "1");
            if (null == excelExportFileTypeEnum) {
                return null;
            }
            Map<String, Object> model = new HashMap<>(10);
            model.put("dataMap", qfkhjbxxList);
//            model.put("exportTime","导出时间");
            // 从模型类型枚举根据templateServiceType获取策略对应模板数据
            JxlsUtils.exportExcel(excelExportFileTypeEnum, model, response, Constant.FILE_PATH_TEST_EXPORT_EXCEL);
            return null;
        }
        IPage<Qfkhjbxx> qfkhjbxxPage = baseMapper.commonKh(page, ysxzList, cbqyList, cbqydList, khSearch, sbcjList, yhztList, cbbList);
        if (qfkhjbxxPage.getRecords().size() == 0) {
            throw new ToflyDeniedException("暂未查询到用户信息");
        }
        return qfkhjbxxPage;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean allocationManager(CensusAllocation allocation) {
        String yhbh = allocation.getYhbh();
        String twoType = allocation.getTwoType();
        if (StringUtil.isEmpty(yhbh)) {
            throw new ToflyDeniedException("请选择用户");
        } else if (StringUtil.isEmpty(twoType)) {
            throw new ToflyDeniedException("请传递操作类型");
        }
        List<String> selectYh = Arrays.asList(yhbh.split(","));
        if (!selectYh.isEmpty()) {
//            Connection connection =
            //获取选择用户的区域段信息(用于修改这些区段的顺序号)，并且删除选择用户的绑定信息
            Set<String> bingQyd = getSelectYhQyd(selectYh);

            //分配操作
            if ("0".equals(twoType)) {
                //目标区段
                String qydbm = allocation.getQydbm();
                String qybm = allocation.getQybm();
                //区域段编码是唯一的
                while (true) {
                    //加锁
                    if (redisTemplate.opsForValue().setIfAbsent(qydbm, UUID.randomUUID().toString(), 300, TimeUnit.MILLISECONDS)) {
                        if (StringUtil.isNull(qydbm)) {
                            throw new ToflyDeniedException("请选择区域段");
                        } else if (StringUtil.isNull(qybm)) {
                            throw new ToflyDeniedException("请选择区域");
                        }
                        bingQyd.add(qydbm);
                        //不能因为重新分配用户，就将原有的抄表序号给打乱了
                        //查询已经绑定了的用户最大序号
                        Integer cbxh = baseMapper.getMaxCbxh(qydbm);
                        if (StringUtil.isNotNull(cbxh)) {
                            cbxh = cbxh + 1;
                        } else {
                            cbxh = 1;
                        }
                        //对数据进行排序
                        List<CensusBindZone> censusBindZoneList = new ArrayList<>();
                        for (int i = 0; i < selectYh.size(); i++) {
                            CensusBindZone censusBindZone = new CensusBindZone();
                            censusBindZone.setCbxh(cbxh + i);
                            censusBindZone.setYhbh(selectYh.get(i));
                            censusBindZone.setQybm(qybm);
                            censusBindZone.setQydbm(qydbm);
                            censusBindZone.setUpdateFlag("1");
                            //子顺序号默认为0，用于之后的排序使用
                            censusBindZone.setChildSort(0);
                            censusBindZone.setUpdateUser(SecurityUtils.getUserId());
                            censusBindZone.setUpdateDate(new Date());
                            censusBindZoneList.add(censusBindZone);
                        }
                        //在插入前，查询是否已经有当前区段的这些用户已经落库了，如果已经存在了，则不能添加，但是我们这里是连续两次提交，两次进行的数据不是同一事务，上一个事务和下一个事务的数据是相互隔离不受影响的；好像需要设置隔离级别？
                        this.saveBatch(censusBindZoneList);
                        //释放锁
                        redisTemplate.opsForValue().getOperations().delete(qydbm);
                        break;
                    }
                }
            } else if (!"1".equals(twoType)) {
                throw new ToflyDeniedException("请正确操作");
            }
            //修改所有涉及到的区段序号
            aginSort(bingQyd);
        }
        return true;
    }

    /**
     * 调序
     *
     * @param yhbhAndCbxhList yhbhAndCbxhList
     * @return boolean
     */
    @Override
    public boolean orderManager(CensusBhAndXhList yhbhAndCbxhList) {
        //需要排序的用户信息
        List<CensusBindZone> yhbhAndCbxhs = yhbhAndCbxhList.getYhbhAndCbxhs();
        //标识往前往后
        String moveFlag = yhbhAndCbxhList.getMoveFlag();
        //目标用户；必须是已分配了的；不然没有顺序号
        String targetYh = yhbhAndCbxhList.getTargetYh();
        if (StringUtil.isEmpty(targetYh)) {
            throw new ToflyDeniedException("请选择目标用户");
        } else if (StringUtil.isEmpty(moveFlag)) {
            throw new ToflyDeniedException("请正确传递排序参数");
        } else if (StringUtil.isEmpty(yhbhAndCbxhs)) {
            throw new ToflyDeniedException("请正确传递排序用户");
        }
        //查询目标用户分配信息
        CensusBindZone targetYhbing = this.getOne(Wrappers.<CensusBindZone>lambdaQuery().eq(CensusBindZone::getYhbh, targetYh));
        if (StringUtil.isNotNull(targetYhbing)) {
            //目标用户的顺序号
            Integer targetCbxh;
            //区段
            String qydbm = targetYhbing.getQydbm();
            String qybm = targetYhbing.getQybm();
            //区域段编码是唯一的
            while (true) {
                //加锁
                if (redisTemplate.opsForValue().setIfAbsent(qydbm, UUID.randomUUID().toString(), 300, TimeUnit.MILLISECONDS)) {
                    //选择的用户编号
                    List<String> selectYh = yhbhAndCbxhs.stream().map(CensusBindZone::getYhbh).collect(Collectors.toList());
                    //获取选择用户的区域段信息(用于修改这些区段的顺序号)，并且删除选择用户的绑定信息
                    Set<String> bingQyd = getSelectYhQyd(selectYh);
                    bingQyd.add(qydbm);
                    //往后
                    if ("0".equals(moveFlag)) {
                        targetCbxh = targetYhbing.getCbxh();
                    }
                    //往前
                    else if ("1".equals(moveFlag)) {
                        targetCbxh = targetYhbing.getCbxh() - 1;
                    } else {
                        throw new ToflyDeniedException("请正确操作用户排序");
                    }
                    List<CensusBindZone> censusBindZoneList = new ArrayList<>();
                    for (int i = 0; i < selectYh.size(); i++) {
                        CensusBindZone censusBindZone = new CensusBindZone();
                        //目前先设置为与目标一样的顺序号，然后根据子顺序号，再做排序调整
                        censusBindZone.setCbxh(targetCbxh);
                        censusBindZone.setYhbh(selectYh.get(i));
                        censusBindZone.setQybm(qybm);
                        censusBindZone.setQydbm(qydbm);
                        censusBindZone.setUpdateFlag("1");
                        //子顺序号
                        censusBindZone.setChildSort(i + 1);
                        censusBindZoneList.add(censusBindZone);
                    }
                    this.saveBatch(censusBindZoneList);
                    //修改所有涉及到的区段序号
                    aginSort(bingQyd);
                    //释放锁
                    redisTemplate.opsForValue().getOperations().delete(qydbm);
                    break;
                }
            }
        } else {
            throw new ToflyDeniedException("选择的目标用户暂未分配");
        }
        return true;
    }

    /**
     * 获取区段
     *
     * @param isState isState
     * @return list
     */
    @Override
    public List<QyAndQd> getQyAndQd(String isState) {
        if ("1".equals(isState)){
            //查询必须分配的用户的区域和区域段信息
            return baseMapper.getQyAndQdAllocation();
        } else if ("2".equals(isState)) {
            //获取所有的树形结构
            return baseMapper.getQyAndQdPlan();
        } else {
            //获取已分配
            return baseMapper.getQyAndQd();
        }
    }


    /**
     * 获取选择用户的区域段信息(用于修改这些区段的顺序号)，并且删除选择用户的绑定信息
     *
     * @param selectYh 选择的用户编号集合
     * @return 区域段集合
     */
    public Set<String> getSelectYhQyd(List<String> selectYh) {
        //存放绑定表中需要修改的区段顺序号
        Set<String> bingQyd = new HashSet<>();
        //如果yhbhList长度大于1000，则无法使用in查询出来，会大于in的最大限制，所以需要进行分段
        int size = selectYh.size();
        //向上取整，获得次数
        int count = (int) Math.ceil((double) size / 1000d);
        //这里循环次数不多，估计就两三次
        for (int i = 0; i < count; i++) {
            //将上月的数据保存到抄表历史数据表中
            int startIndex = i * 1000;
            int endIndex = 1000 + i * 1000;
            List<String> subYhbhList;
            //判断是否截取的最后一次数据，如果是最后一次数据，则不能使用endIndex这个参数去截取了，有可能会存在endIndex参数值超过索引最大值的情况
            if (i < count - 1) {
                //分段采取数据 0~999 ； 1000~1999 , 2000~2999 .....//这里subList方法无法截取最后一位，所以需要多写一位索引
                subYhbhList = selectYh.subList(startIndex, endIndex);
            } else {
                //分段采取最后的数据
                subYhbhList = selectYh.subList(startIndex, selectYh.size());
            }
            List<CensusBindZone> qydList = this.list(Wrappers.<CensusBindZone>lambdaQuery().in(CensusBindZone::getYhbh, subYhbhList).groupBy(CensusBindZone::getQydbm).select(CensusBindZone::getQydbm));
            if (!qydList.isEmpty()) {
                //因为存在已分配的用户重新分配到目标区段，则需要重新排序的区段
                bingQyd.addAll(qydList.stream().map(CensusBindZone::getQydbm).collect(Collectors.toSet()));
                //删除绑定表中已分配了的用户，因为这些用户也会被重新分配到另外的目标区段
                baseMapper.deleteBindInfo(subYhbhList);
            }
        }
        return bingQyd;
    }

    @Transactional(rollbackFor = Exception.class)
    public void aginSort(Set<String> bingQyd) {
        try {
            ThreadPoolUtil.executorService.execute(() -> {
                //首先清除绑定的重复数据，保证下方排序不出错
                baseMapper.deleteDuplicateData();
                for (String qydbm : bingQyd) {
                    baseMapper.aginSort(qydbm);
                }
            });
        } catch (Exception e) {
            throw new ToflyDeniedException("排序出现异常:" + e.getMessage());
        }
    }

}
