package com.tofly.watermeter.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.watermeter.entity.Cbqydbm;
import com.tofly.watermeter.entity.CensusPlanDetails;
import com.tofly.watermeter.entity.Qfcbplan;
import com.tofly.watermeter.entity.TableCensus;
import com.tofly.watermeter.mapper.CensusPlanDetailsMapper;
import com.tofly.watermeter.service.CbqydbmService;
import com.tofly.watermeter.service.CensusPlanDetailsService;
import com.tofly.watermeter.service.QfcbplanService;
import com.tofly.watermeter.service.TableCensusService;
import com.tofly.watermeter.util.ThreadPoolUtil;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Bubble
 */
@Service
@AllArgsConstructor
public class CensusPlanDetailsServiceImpl extends ServiceImpl<CensusPlanDetailsMapper, CensusPlanDetails> implements CensusPlanDetailsService {

    private final QfcbplanService qfcbplanService;

    private final TableCensusService tableCensusService;

    private final CbqydbmService cbqydbmService;

    /**
     * 启用计划
     *
     * @param ids 计划详情id
     * @return boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean planStart(String ids,String year) {

        List<String> planInfoIdList;

        if (StringUtil.isEmpty(ids)){
            if (StringUtil.isBlank(year)){
                year = DateUtil.format(new Date(), "yyyy");
            }
            //查询当前年计划
            Qfcbplan yearPlan = qfcbplanService.getOne(Wrappers.<Qfcbplan>lambdaQuery()
                    .eq(Qfcbplan::getIsGeneral, "2")
                    .eq(Qfcbplan::getIsState,"1")
                    .eq(Qfcbplan::getYear, year));
            if (yearPlan == null){
                throw new ToflyDeniedException(year + "年计划不存在或未启用");
            }
            List<CensusPlanDetails> censusPlanList = list(Wrappers.<CensusPlanDetails>lambdaQuery().eq(CensusPlanDetails::getIsState, "0")
                    .eq(CensusPlanDetails::getPlanId, yearPlan.getId()));
            if (CollectionUtils.isEmpty(censusPlanList)){
                throw new ToflyDeniedException(year + "年未下发计划或所有计划已启用");
            }
            planInfoIdList = censusPlanList.stream().map(c->String.valueOf(c.getId())).collect(Collectors.toList());
        }else {
            planInfoIdList = Arrays.asList(ids.split(","));
        }
        List<CensusPlanDetails> censusPlanDetailsList = list(Wrappers.<CensusPlanDetails>lambdaQuery()
                .in(CensusPlanDetails::getId, planInfoIdList));
        if (CollectionUtils.isEmpty(censusPlanDetailsList)){
            throw new ToflyDeniedException("计划不存在");
        }
        //年计划id
        List<Long> planIdList = censusPlanDetailsList.stream().distinct().map(CensusPlanDetails::getPlanId).collect(Collectors.toList());
        //查询抄表年计划是否有禁用的
        List<Qfcbplan> disablePlanList = qfcbplanService.list(Wrappers.<Qfcbplan>lambdaQuery()
                .in(Qfcbplan::getId, planIdList)
                .eq(Qfcbplan::getIsGeneral,"2")
                .eq(Qfcbplan::getIsState, "0"));

        if (CollectionUtils.isNotEmpty(disablePlanList)){
            throw new ToflyDeniedException(disablePlanList.stream().map(Qfcbplan::getYear).collect(Collectors.joining(",")) + "年计划已禁用，无法启用计划");
        }

        ThreadPoolUtil.executorService.execute(() -> {
            List<TableCensus> tableCensuses = baseMapper.intiCensusInfo(planInfoIdList);
            tableCensusService.saveBatch(tableCensuses);
        });
        return update(Wrappers.<CensusPlanDetails>lambdaUpdate().set(CensusPlanDetails::getIsState, "1").in(CensusPlanDetails::getId, planInfoIdList));
    }

    /**
     * 普查计划添加
     *
     * @param censusPlanDetails censusPlanDetails
     * @return boolean
     */
    @Override
    public boolean insert(CensusPlanDetails censusPlanDetails) {
        Long planId = censusPlanDetails.getPlanId();
        String qybm = censusPlanDetails.getCbqyBm();
        String qydbm = censusPlanDetails.getCbqydBm();
        String isState = censusPlanDetails.getIsState();
        Long cbry = censusPlanDetails.getCbry();
        if (StringUtil.isNull(planId)) {
            throw new ToflyDeniedException("请选择年计划");
        } else if (StringUtil.isEmpty(qybm)) {
            throw new ToflyDeniedException("请选择区域");
        }
        List<String> qybmList = Arrays.asList(qybm.split(","));
        //保存计划
        List<CensusPlanDetails> detailsList = new ArrayList<>();
        qybmList.forEach(k -> {
            //最终需要制定计划的区域段编码
            List<String> queryQydbmList = null;
            //判断有没有选择区域段
            if (StringUtil.isEmpty(qydbm)) {
                //相同区域下面只能有一个区域段计划(注意区分普查或人工计划类型)
                List<CensusPlanDetails> qfcbplandetailsList = list(Wrappers.<CensusPlanDetails>lambdaQuery()
                        .eq(CensusPlanDetails::getPlanId, planId)
                        .eq(CensusPlanDetails::getCbqyBm, k)
                        .select(CensusPlanDetails::getCbqyBm, CensusPlanDetails::getCbqydBm));
                List<String> qydbmList = null;
                if (StringUtil.isNotNull(qfcbplandetailsList) && qfcbplandetailsList.size() > 0) {
                    qydbmList = qfcbplandetailsList.stream().map(CensusPlanDetails::getCbqydBm).collect(Collectors.toList());
                }
                //如果没有传递区域段编码，则是要所有区域下的区域段；但是如果已经有制定计划了，则排除当前这个计划区域段
                List<Cbqydbm> cbqydbmList = cbqydbmService.list(Wrappers.<Cbqydbm>lambdaQuery().eq(Cbqydbm::getQybm, k)
                        .notIn(StringUtil.isNotNull(qydbmList), Cbqydbm::getQydbm, qydbmList).eq(Cbqydbm::getYxbz, "0")
                        .select(Cbqydbm::getQydbm, Cbqydbm::getQybm));
                if (StringUtil.isNotNull(cbqydbmList) && cbqydbmList.size() > 0) {
                    queryQydbmList = cbqydbmList.stream().map(Cbqydbm::getQydbm).collect(Collectors.toList());
                }
            } else {
                queryQydbmList = Arrays.asList(qydbm.split(","));
            }
            //如果区域段是一个的情况；需要判断是否有重复的
            if (StringUtil.isNotNull(queryQydbmList) && queryQydbmList.size() == 1) {
                //相同区域下面只能有一个区域段计划(注意区分普查或人工计划类型)  备注：现在只能有一个   卡表计划或机械表计划中的区段不能有重复的，所以取消这个判断 .eq(Qfcbplandetails::getIsGeneral, isGeneral)
                int count = count(Wrappers.<CensusPlanDetails>lambdaQuery().eq(CensusPlanDetails::getPlanId, planId)
                        .eq(CensusPlanDetails::getCbqyBm, k)
                        .eq(CensusPlanDetails::getCbqydBm, queryQydbmList.get(0)));
                if (count > 0) {
                    Cbqydbm cbqydbm = cbqydbmService.getOne(Wrappers.<Cbqydbm>lambdaQuery().eq(Cbqydbm::getQydbm, queryQydbmList.get(0)).select(Cbqydbm::getQydmc));
                    throw new ToflyDeniedException("已有：" + cbqydbm.getQydmc() + "计划");
                }
            }
            if (StringUtil.isEmpty(isState)) {
                //如果没有设置启用状态，默认为启用
                censusPlanDetails.setIsState("0");
            }
            if (StringUtil.isNotNull(queryQydbmList) && queryQydbmList.size() > 0) {
                //查询区域段/区域名称
                List<Cbqydbm> cbqydbm = cbqydbmService.list(Wrappers.<Cbqydbm>lambdaQuery().in(Cbqydbm::getQydbm, queryQydbmList)
                        .select(Cbqydbm::getQydbm, Cbqydbm::getQydmc, Cbqydbm::getQybm));
                cbqydbm.forEach(v -> {
                    CensusPlanDetails cp = new CensusPlanDetails();
                    cp.setCbqyBm(k);
                    cp.setCbqyMc(v.getQymc());
                    cp.setQydmc(v.getQydmc());
                    cp.setCbqydBm(v.getQydbm());
                    cp.setIsState(censusPlanDetails.getIsState());
                    cp.setPlanId(planId);
                    cp.setCbry(cbry);
                    detailsList.add(cp);
                });
            }
        });
        if (detailsList.size() > 0) {
            Long maxId = baseMapper.getMaxId();
            Long userId = SecurityUtils.getUserId();
            for (int i = 0; i < detailsList.size(); i++) {
                detailsList.get(i).setId(maxId + i);
                detailsList.get(i).setCreateUser(userId);
                detailsList.get(i).setCreateTime(new Date());
            }
        }
        //没有抄表人员的情况
        if (StringUtil.isNull(cbry)) {
            //保存计划
            baseMapper.savePlanBatch(detailsList);
        } else {
            baseMapper.savePlanCbryBatch(detailsList);
        }
        return true;
    }

    /**
     * 修改普查计划详情
     *
     * @param censusPlanDetails censusPlanDetails
     * @return Boolean.TRUE
     */
    @Override
    public boolean update(CensusPlanDetails censusPlanDetails) {
        Long planId = censusPlanDetails.getPlanId();
        String qybm = censusPlanDetails.getCbqyBm();
        String qydbm = censusPlanDetails.getCbqydBm();
        Long cbry = censusPlanDetails.getCbry();
        Date startDay = censusPlanDetails.getStartDay();
        Date endDay = censusPlanDetails.getEndDay();
        String isState = censusPlanDetails.getIsState();
        String isUpdateType = censusPlanDetails.getIsUpdateType();
        if (StringUtil.isEmpty(isUpdateType)) {
            throw new ToflyDeniedException("请传递设置类型");
        } else if (StringUtil.isNull(planId)) {
            throw new ToflyDeniedException("请选择年计划");
        } else if (StringUtil.isEmpty(qybm)) {
            throw new ToflyDeniedException("请选择区域");
        }
        //区域集合
        List<String> qybmList = Arrays.asList(qybm.split(","));
        //区域段集合
        List<String> qydbmList = null;

        if (StringUtil.isNotEmpty(qydbm)) {
            qydbmList = Arrays.asList(qydbm.split(","));
        } else {
            //多个区域编码的情况，查询编码下面所有的区域段
            List<Cbqydbm> cbqydbmList = cbqydbmService.list(Wrappers.<Cbqydbm>lambdaQuery()
                    .in(Cbqydbm::getQybm, qybmList).select(Cbqydbm::getQydbm));
            qydbmList = cbqydbmList.stream().map(Cbqydbm::getQydbm).collect(Collectors.toList());
        }

        //抄表人员设置
        if ("0".equals(isUpdateType)) {
            if (StringUtil.isNull(cbry)) {
                throw new ToflyDeniedException("请选择抄表人员");
            }
        }
        //抄表时间设置
        else if ("1".equals(isUpdateType)) {
            if (StringUtil.isNull(startDay) || StringUtil.isNull(endDay)) {
                throw new ToflyDeniedException("请选择抄表开始日和结束日");
            }
        }
        //启用或禁用
        else if ("2".equals(isUpdateType)) {
            if (StringUtil.isEmpty(isState)) {
                throw new ToflyDeniedException("请正确选择启用或禁用状态");
            }
        } else {
            throw new ToflyDeniedException("修改计划时请正确操作");
        }

        if (qydbmList.size() > 0) {
            update(Wrappers.<CensusPlanDetails>lambdaUpdate()
                    .eq(CensusPlanDetails::getPlanId, planId)
                    .in(CensusPlanDetails::getCbqydBm, qydbmList)
                    .set(StringUtil.isNotNull(cbry), CensusPlanDetails::getCbry, cbry)
                    .set(StringUtil.isNotNull(startDay), CensusPlanDetails::getStartDay, startDay)
                    .set(StringUtil.isNotNull(endDay), CensusPlanDetails::getEndDay, endDay)
                    .set(StringUtil.isNotEmpty(isState), CensusPlanDetails::getIsState, isState));
        } else {
            throw new ToflyDeniedException("所选区域下面暂无抄表段计划");
        }
        return true;
    }

    /**
     * 普查计划--区域段任务汇总
     *
     * @param qybm 区域编码
     * @return list
     */
    @Override
    public List<Map> censusPlanCollect(String qybm) {
        if (StringUtil.isEmpty(qybm)) {
            throw new ToflyDeniedException("请选择区域");
        }
        //抄表人员ID
        Long cbry = SecurityUtils.getUserId();

        List<Map> mapList = baseMapper.censusPlanCollect(cbry,qybm);
        for (Map map : mapList) {
            map.put("isQy", "0");
            map.put("cbqyBm", qybm);
        }
        return mapList;
    }

    /**
     * 区域任务汇总
     *
     * @return list
     */
    @Override
    public List<Map> censusZoneCollect() {
        //抄表人员ID
        Long cbry = SecurityUtils.getUserId();

        List<Map> mapList = baseMapper.censusZoneCollect(cbry);
        for (Map map : mapList) {
            map.put("isQy", "1");
        }
        //查询属于该抄表员的计划
        return mapList;
    }

    /**
     * 删除计划详情及普查记录
     *
     * @param ids ids
     * @return boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean deletePlanDetail(String ids) {
        List<String> idList = Arrays.asList(ids.split(","));
        List<CensusPlanDetails> censusPlanDetails = listByIds(idList);
        if (CollectionUtils.isNotEmpty(censusPlanDetails)){
            List<Long> planIdList = censusPlanDetails.stream().map(CensusPlanDetails::getPlanId).collect(Collectors.toList());
            for (Qfcbplan plan : qfcbplanService.listByIds(planIdList)) {
                String year = plan.getYear();
                for (CensusPlanDetails censusPlanDetail : censusPlanDetails) {
                    tableCensusService.remove(Wrappers.<TableCensus>lambdaQuery()
                            .eq(TableCensus::getCensusYear,year)
                            .eq(TableCensus::getCbqyd,censusPlanDetail.getCbqydBm())
                            .eq(TableCensus::getCbqy,censusPlanDetail.getCbqyBm()));
                }
            }
        }
        return removeByIds(idList);
    }

    @Override
    public Page getQyInfoByPlan(Page page, Long planId, String cbqyBm) {
        if (StringUtil.isNull(planId)) {
            throw new ToflyDeniedException("请选择年计划");
        }
        //如果没有传递区域编码，则是查询区域编码数据
        if (StringUtil.isEmpty(cbqyBm)) {
            return page(page, Wrappers.<CensusPlanDetails>lambdaQuery()
                    .eq(CensusPlanDetails::getPlanId, planId)
                    .groupBy(CensusPlanDetails::getCbqyBm, CensusPlanDetails::getCbqyMc)
                    .select(CensusPlanDetails::getCbqyBm, CensusPlanDetails::getCbqyMc));
        } else if (StringUtil.isNotEmpty(cbqyBm)) {
            //如果传递了区域编码，只能查找区域编码下的区域段信息
            return page(page, Wrappers.<CensusPlanDetails>lambdaQuery()
                    .eq(CensusPlanDetails::getPlanId, planId).eq(CensusPlanDetails::getCbqyBm, cbqyBm)
                    .groupBy(CensusPlanDetails::getCbqydBm, CensusPlanDetails::getQydmc)
                    .select(CensusPlanDetails::getCbqydBm, CensusPlanDetails::getQydmc));
        } else {
            return null;
        }
    }

}
