package com.tofly.watermeter.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.watermeter.em.GetIndexEnum;
import com.tofly.watermeter.entity.GetData;
import com.tofly.watermeter.entity.GetIndexConfig;
import com.tofly.watermeter.entity.GetParamConfig;
import com.tofly.watermeter.mapper.GetDataMapper;
import com.tofly.watermeter.service.GetDataService;
import com.tofly.watermeter.service.GetIndexConfigService;
import com.tofly.watermeter.service.GetParamConfigService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author Bubble
 */
@Service
@AllArgsConstructor
public class GetDataServiceImpl extends ServiceImpl<GetDataMapper, GetData> implements GetDataService {

    private final GetIndexConfigService getIndexConfigService;
    private final GetParamConfigService getParamConfigService;

    /**
     * 截取指标数据
     *
     * @return boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean getIndexValue() {
        Date nowDate = new Date();
        DateTime lastDay = DateUtil.offsetDay(nowDate, -1);

        DateTime getData = DateUtil.parseDate(DateUtil.format(lastDay, "yyyy-MM-dd") + " " + "00:00:00");
        //查询是否有昨日数据生成
        List<GetData> dataList = list(Wrappers.<GetData>lambdaQuery().eq(GetData::getGetDate, getData));
        if (CollectionUtil.isEmpty(dataList)){
            List<GetIndexConfig> indexConfigList = getIndexConfigService.list();
            //取水量液位
            double intakeWaterOne = 0D;
            double intakeWaterTwo = 0D;
            double intakeWaterTotal = 0D;
            //电耗
            double electricityOne = 0D;
            double electricityTwo = 0D;
            double electricityHead = 0D;

            //pac、食盐、次钠、电单耗
            double pacUnitOne = 0D;
            double pacUnitTwo = 0D;
            double saltUnit = 0D;
            double hyposodiumUnit = 0D;
            //电耗
            double electricityOneUnit = 0D;
            double electricityTwoUnit = 0D;
            double electricityTotalUnit = 0D;

            //配置的截取指标不为空
            if (CollectionUtil.isNotEmpty(indexConfigList)) {
                //所有配置指标
                String indexTotal = indexConfigList.stream().map(GetIndexConfig::getIndexCode).collect(Collectors.joining(","));
                //取水量配置
                List<GetIndexConfig> intakeWaterConfig = indexConfigList.stream().filter(g -> g.getTypeId().equals(GetIndexEnum.INDEX_INTAKE_WATER.getCode())).collect(Collectors.toList());
                //pac配置
                List<GetIndexConfig> pacConfig = indexConfigList.stream().filter(g -> g.getTypeId().equals(GetIndexEnum.INDEX_PAC.getCode())).collect(Collectors.toList());
                //食盐配置
                List<GetIndexConfig> saltConfig = indexConfigList.stream().filter(g -> g.getTypeId().equals(GetIndexEnum.INDEX_SALT.getCode())).collect(Collectors.toList());
                //次钠配置
                List<GetIndexConfig> hyposodiumConfig = indexConfigList.stream().filter(g -> g.getTypeId().equals(GetIndexEnum.INDEX_HYPOSODIUM.getCode())).collect(Collectors.toList());
                //电耗配置
                List<GetIndexConfig> electricityConfig = indexConfigList.stream().filter(g -> g.getTypeId().equals(GetIndexEnum.INDEX_ELECTRICITY.getCode())).collect(Collectors.toList());

                //pac液位
                double pacOneLiquidLevel;
                double pacTwoLiquidLevel;
                //食盐液位
                double saltLiquidLevel = 0D;
                //次钠液位
                double hyposodiumLiquidLevel = 0D;

                String interceptHour = "08:00:00";
                //开始结束时间
                Date startDate = DateUtil.parse(DateUtil.format(lastDay, "yyyy-MM-dd") + " " + interceptHour,"yyyy-MM-dd HH:mm:ss");
                Date endDate = DateUtil.parse(DateUtil.format(nowDate, "yyyy-MM-dd") + " " + interceptHour,"yyyy-MM-dd HH:mm:ss");
                //所有指标值信息
                List<Map<String, Object>> allIndexInfoList = baseMapper.getIndexValues(startDate, endDate, indexTotal);

                //取水量配置不为空
                if (CollectionUtil.isNotEmpty(intakeWaterConfig)) {
                    //二厂取水指标
                    List<String> factoryTwoIndex = intakeWaterConfig.stream().filter(g -> GetIndexEnum.FACTORY_TWO.getCode()
                            .equals(g.getFactoryId()) && g.getTypeId().equals(GetIndexEnum.INDEX_INTAKE_WATER.getCode())).map(GetIndexConfig::getIndexCode).collect(Collectors.toList());
                    //总取水指标
                    List<String> factoryTotalIndex = intakeWaterConfig.stream().filter(g -> g.getFactoryId() == null && g.getTypeId().equals(GetIndexEnum.INDEX_INTAKE_WATER.getCode()))
                            .map(GetIndexConfig::getIndexCode).collect(Collectors.toList());
                    //一厂二厂取水量
                    intakeWaterTwo = liquidLevelCompute(allIndexInfoList, factoryTwoIndex);
                    intakeWaterTotal = liquidLevelCompute(allIndexInfoList, factoryTotalIndex);
                    intakeWaterOne = intakeWaterTotal - intakeWaterTwo;

                }
                //保存一厂液位
                GetData intakeOne = new GetData();
                intakeOne.setGetValue(intakeWaterOne);
                intakeOne.setGetDate(getData);
                intakeOne.setDataType(GetIndexEnum.INTAKE_WATER_ONE_TYPE.getCode());
                intakeOne.setDataName(GetIndexEnum.INTAKE_WATER_ONE_TYPE.getCodeName());
                save(intakeOne);
                //保存二厂液位
                GetData intakeTwo = new GetData();
                intakeTwo.setGetValue(intakeWaterTwo);
                intakeTwo.setGetDate(getData);
                intakeTwo.setDataType(GetIndexEnum.INTAKE_WATER_TWO_TYPE.getCode());
                intakeTwo.setDataName(GetIndexEnum.INTAKE_WATER_TWO_TYPE.getCodeName());
                save(intakeTwo);
                //保持总液位
                GetData intakeTotal = new GetData();
                intakeTotal.setGetValue(intakeWaterTotal);
                intakeTotal.setGetDate(getData);
                intakeTotal.setDataType(GetIndexEnum.INTAKE_WATER_TOTAL_TYPE.getCode());
                intakeTotal.setDataName(GetIndexEnum.INTAKE_WATER_TOTAL_TYPE.getCodeName());
                save(intakeTotal);


                //参与计算参数配置
                List<GetParamConfig> paramConfigs = getParamConfigService.list();
                //pac配置不为空
                if (CollectionUtils.isNotEmpty(paramConfigs)) {
                    if (CollectionUtils.isNotEmpty(pacConfig)) {
                        List<GetParamConfig> pacParam = paramConfigs.stream().filter(g -> g.getParamType().equals(GetIndexEnum.PARAM_PAC.getCode())).collect(Collectors.toList());
                        if (CollectionUtils.isNotEmpty(pacParam)) {
                            //pac液位指标配置
                            List<String> factoryOneIndex = pacConfig.stream().filter(g -> GetIndexEnum.FACTORY_ONE.getCode().equals(g.getFactoryId()) && g.getTypeId().equals(GetIndexEnum.INDEX_PAC.getCode()))
                                    .map(GetIndexConfig::getIndexCode).collect(Collectors.toList());
                            List<String> factoryTwoIndex = pacConfig.stream().filter(g -> GetIndexEnum.FACTORY_TWO.getCode().equals(g.getFactoryId()) && g.getTypeId().equals(GetIndexEnum.INDEX_PAC.getCode()))
                                    .map(GetIndexConfig::getIndexCode).collect(Collectors.toList());
                            //一二厂pac液位
                            pacOneLiquidLevel = complexLiquidLevelCompute(allIndexInfoList, factoryOneIndex);
                            pacTwoLiquidLevel = complexLiquidLevelCompute(allIndexInfoList, factoryTwoIndex);
                            //密度
                            Double density = pacParam.get(0).getDensity();
                            //计算单耗
                            double pacOneArea = 10.38D;
                            pacUnitOne = intakeWaterOne == 0 ? 0D : BigDecimal.valueOf(((pacOneLiquidLevel * pacOneArea * density * 1000) / intakeWaterOne) * 1000).setScale(4, RoundingMode.HALF_EVEN).doubleValue();
                            double pacTwoArea = 16D;
                            pacUnitTwo = intakeWaterTwo == 0 ? 0D : BigDecimal.valueOf(((pacTwoLiquidLevel * pacTwoArea * density * 1000) / intakeWaterTwo) * 1000).setScale(4, RoundingMode.HALF_EVEN).doubleValue();
                        }
                    }
                }

                //保存一厂液位
                GetData pacOne = new GetData();
                pacOne.setGetValue(pacUnitOne);
                pacOne.setGetDate(getData);
                pacOne.setDataType(GetIndexEnum.PAC_ONE_TYPE.getCode());
                pacOne.setDataName(GetIndexEnum.PAC_ONE_TYPE.getCodeName());
                save(pacOne);
                //保存二厂液位
                GetData pacTwo = new GetData();
                pacTwo.setGetValue(pacUnitTwo);
                pacTwo.setGetDate(getData);
                pacTwo.setDataType(GetIndexEnum.PAC_TWO_TYPE.getCode());
                pacTwo.setDataName(GetIndexEnum.PAC_TWO_TYPE.getCodeName());
                save(pacTwo);

                //食盐配置不为空
                if (CollectionUtils.isNotEmpty(saltConfig)) {

                }
                //次钠配置不为空
                if (CollectionUtils.isNotEmpty(hyposodiumConfig)) {
                }
                //电耗配置不为空
                if (CollectionUtils.isNotEmpty(electricityConfig)){
                    //一厂累计电耗
                    List<String> factoryOneIndex = electricityConfig.stream().filter(g -> GetIndexEnum.FACTORY_ONE.getCode()
                            .equals(g.getFactoryId()) && g.getTypeId().equals(GetIndexEnum.INDEX_ELECTRICITY.getCode())).map(GetIndexConfig::getIndexCode).collect(Collectors.toList());
                    //二厂累计电耗
                    List<String> factoryTwoIndex = electricityConfig.stream().filter(g -> GetIndexEnum.FACTORY_TWO.getCode()
                            .equals(g.getFactoryId()) && g.getTypeId().equals(GetIndexEnum.INDEX_ELECTRICITY.getCode())).map(GetIndexConfig::getIndexCode).collect(Collectors.toList());
                    //取水头累计电耗
                    List<String> headIndex = electricityConfig.stream().filter(g -> GetIndexEnum.WATER_INTAKE_HEAD.getCode()
                            .equals(g.getFactoryId()) && g.getTypeId().equals(GetIndexEnum.INDEX_ELECTRICITY.getCode())).map(GetIndexConfig::getIndexCode).collect(Collectors.toList());
                    electricityOne = liquidLevelCompute(allIndexInfoList, factoryOneIndex);
                    electricityTwo = liquidLevelCompute(allIndexInfoList, factoryTwoIndex);
                    electricityHead = liquidLevelCompute(allIndexInfoList, headIndex);
                    Map<String, Object> waterSupply = baseMapper.getWaterSupply();
                    Double waterSupplyOne = CollectionUtils.isNotEmpty(waterSupply) ? Double.parseDouble(String.valueOf(waterSupply.get("one"))) : 0D;
                    Double waterSupplyTwo = CollectionUtils.isNotEmpty(waterSupply) ? Double.parseDouble(String.valueOf(waterSupply.get("two"))) : 0D;
                    electricityOneUnit = calElectricityUnit(electricityOne,waterSupplyOne);
                    electricityTwoUnit = calElectricityUnit(electricityTwo,waterSupplyTwo);
                    electricityTotalUnit = calElectricityUnit(electricityHead + electricityOne + electricityTwo,waterSupplyOne + waterSupplyTwo);
                }
                //保存一厂电耗
                GetData electricityUnitOne = new GetData();
                electricityUnitOne.setGetValue(electricityOneUnit);
                electricityUnitOne.setGetDate(getData);
                electricityUnitOne.setDataType(GetIndexEnum.ELECTRICITY_ONE_TYPE.getCode());
                electricityUnitOne.setDataName(GetIndexEnum.ELECTRICITY_ONE_TYPE.getCodeName());
                save(electricityUnitOne);
                //保存二厂电耗
                GetData electricityUnitTwo = new GetData();
                electricityUnitTwo.setGetValue(electricityTwoUnit);
                electricityUnitTwo.setGetDate(getData);
                electricityUnitTwo.setDataType(GetIndexEnum.ELECTRICITY_TWO_TYPE.getCode());
                electricityUnitTwo.setDataName(GetIndexEnum.ELECTRICITY_TWO_TYPE.getCodeName());
                save(electricityUnitTwo);
                //保存总电耗
                GetData electricityUnitTotal = new GetData();
                electricityUnitTotal.setGetValue(electricityTotalUnit);
                electricityUnitTotal.setGetDate(getData);
                electricityUnitTotal.setDataType(GetIndexEnum.ELECTRICITY_TOTAL_TYPE.getCode());
                electricityUnitTotal.setDataName(GetIndexEnum.ELECTRICITY_TOTAL_TYPE.getCodeName());
                save(electricityUnitTotal);
            }
        }
        return true;
    }

    /**
     * 处理普通液位数据
     */
    private static Double liquidLevelCompute(List<Map<String, Object>> allIndexInfoList, List<String> index) {
        double num = 0D;
        for (String twoIndex : index) {
            List<Map<String, Object>> dataList = allIndexInfoList.stream().filter(map -> String.valueOf(map.get("itcd")).equals(twoIndex))
                    .collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(dataList)) {
                double start = Double.parseDouble(String.valueOf(dataList.get(0).get("val")));
                double end = Double.parseDouble(String.valueOf(dataList.get(dataList.size() - 1).get("val")));
                num += BigDecimal.valueOf(end).subtract(BigDecimal.valueOf(start)).doubleValue();
            }
        }
        return num;
    }

    /**
     * 处理复杂液位数据(改类型液位是池内液体逐渐减少，存在加液情况)
     *
     * @param allIndexInfoList 所有指标信息
     * @param index            配置指标
     * @return double
     */
    private static Double complexLiquidLevelCompute(List<Map<String, Object>> allIndexInfoList, List<String> index) {
        double num = 0D;
        for (String twoIndex : index) {
            List<Map<String, Object>> dataList = allIndexInfoList.stream().filter(map -> String.valueOf(map.get("itcd")).equals(twoIndex))
                    .collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(dataList)) {
                double start = Double.parseDouble(String.valueOf(dataList.get(0).get("val")));
                double end = Double.parseDouble(String.valueOf(dataList.get(dataList.size() - 1).get("val")));
                if (start > end) {
                    num += BigDecimal.valueOf(start).subtract(BigDecimal.valueOf(end)).doubleValue();
                } else {
                    double max = Double.parseDouble(String.valueOf(dataList.stream()
                            .max(Comparator.comparingDouble(o -> Double.parseDouble(String.valueOf(o.get("val")))))
                            .get().get("val")));
                    double min = Double.parseDouble(String.valueOf(dataList.stream()
                            .min(Comparator.comparingDouble(o -> Double.parseDouble(String.valueOf(o.get("val")))))
                            .get().get("val")));
                    num += (BigDecimal.valueOf(start).subtract(BigDecimal.valueOf(min))).add (BigDecimal.valueOf(max).subtract(BigDecimal.valueOf(end))).doubleValue();
                }
            }
        }
        return num;
    }

    private static Double calElectricityUnit(Double electricity, Double water){
       return water == 0D ? 0D : BigDecimal.valueOf(electricity).divide(BigDecimal.valueOf(water), 5, BigDecimal.ROUND_HALF_UP).multiply(BigDecimal.valueOf(1000)).doubleValue();
    }
}
