package com.tofly.watermeter.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.watermeter.custom.entity.CensusQueryDto;
import com.tofly.watermeter.entity.Qfcbplan;
import com.tofly.watermeter.entity.TableCensus;
import com.tofly.watermeter.mapper.TableCensusMapper;
import com.tofly.watermeter.service.QfcbplanService;
import com.tofly.watermeter.service.TableCensusService;
import com.tofly.watermeter.util.service.FileUpLoadService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author bubble
 */
@Service
@AllArgsConstructor
public class TableCensusServiceImpl extends ServiceImpl<TableCensusMapper, TableCensus> implements TableCensusService {

    private final FileUpLoadService fileUpLoadService;

    private final QfcbplanService qfcbplanService;

    /**
     * 添加普查信息
     *
     * @param tableCensus 普查信息
     * @return Boolean.TRUE
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean saveTableCensus(TableCensus tableCensus, List<MultipartFile> electronicFiles, List<MultipartFile> mechanicalFiles, List<MultipartFile> tableFiles) {

        Date fhsj = tableCensus.getFhsj();

        if (fhsj == null) {
            throw new ToflyDeniedException("复核时间不允许为空");
        }

        DateTime begin = DateUtil.beginOfYear(fhsj);
        DateTime end = DateUtil.endOfYear(fhsj);

        List<TableCensus> tableCensusList = list(Wrappers.<TableCensus>lambdaQuery().between(TableCensus::getFhsj, begin, end).eq(TableCensus::getYhbh, tableCensus.getYhbh()));
        if (CollectionUtil.isNotEmpty(tableCensusList)) {
            throw new ToflyDeniedException("当前年份该用户已进行普查");
        }
        fileSave(tableCensus, mechanicalFiles, electronicFiles, tableFiles);
        return save(tableCensus);
    }

    /**
     * 删除普查信息
     *
     * @param ids id
     * @return boolean
     */
    @Override
    public boolean deleteTableCensus(String ids) {
        return removeByIds(Arrays.asList(ids.split(",")));
    }

    /**
     * 修改普查信息
     *
     * @param tableCensus 普查信息
     * @return Boolean.TRUE
     */
    @Override
    public boolean updateTableCensus(TableCensus tableCensus, List<MultipartFile> electronicFiles, List<MultipartFile> mechanicalFiles, List<MultipartFile> tableFiles) {
        fileSave(tableCensus, electronicFiles, mechanicalFiles, tableFiles);
        String nowYear = DateUtil.format(new Date(), "yyyy");
        if (!tableCensus.getCensusYear().equals(nowYear)){
            throw new ToflyDeniedException("记录年份与当前年份不一致，不允许上传");
        }
        return update(Wrappers.<TableCensus>lambdaUpdate()
                .set(tableCensus.getZdfh() != null,TableCensus::getZdfh,tableCensus.getZdfh())
                .set(tableCensus.getJxzdfh() != null,TableCensus::getJxzdfh,tableCensus.getJxzdfh())
                .set(tableCensus.getSjcsfh() != null,TableCensus::getSjcsfh,tableCensus.getSjcsfh())
                .set(TableCensus::getFhry,SecurityUtils.getUserId())
                .set(TableCensus::getFhsj,new Date())
                .set(StringUtils.isNotBlank(tableCensus.getYsxzfh()),TableCensus::getYsxzfh,tableCensus.getYsxzfh())
                .set(StringUtils.isNotBlank(tableCensus.getBwfh()),TableCensus::getBwfh,tableCensus.getBwfh())
                .set(StringUtils.isNotBlank(tableCensus.getJdyz()),TableCensus::getJdyz,tableCensus.getJdyz())
                .set(StringUtils.isNotBlank(tableCensus.getPxzt()),TableCensus::getPxzt,tableCensus.getPxzt())
                .set(StringUtils.isNotBlank(tableCensus.getBz()),TableCensus::getBz,tableCensus.getBz())
                .set(StringUtils.isNotBlank(tableCensus.getJxwj()),TableCensus::getJxwj,tableCensus.getJxwj())
                .set(StringUtils.isNotBlank(tableCensus.getDzwj()),TableCensus::getDzwj,tableCensus.getDzwj())
                .set(StringUtils.isNotBlank(tableCensus.getSbwj()),TableCensus::getSbwj,tableCensus.getSbwj())
                .eq(TableCensus::getCensusYear,tableCensus.getCensusYear())
                .eq(TableCensus::getYhbh,tableCensus.getYhbh()));
    }

    /**
     * 普查计划下载
     *
     * @param page page
     * @return page
     */
    @Override
    public Page<TableCensus> planDownLoad(Page page) {
        Long userId = SecurityUtils.getUserId();
        String year = DateUtil.format(new Date(), "yyyy");
        List<Qfcbplan> planList = qfcbplanService.list(Wrappers.<Qfcbplan>lambdaQuery()
                .eq(Qfcbplan::getIsState,"1")
                .eq(Qfcbplan::getIsGeneral, "2").eq(Qfcbplan::getYear, year));

        if (CollectionUtils.isEmpty(planList)) {
            throw new ToflyDeniedException("未制定" + year + "年计划或该计划未启用");
        }

        Page<TableCensus> mapPage = baseMapper.planDownLoad(page, userId);

        List<TableCensus> records = mapPage.getRecords();
        if (CollectionUtils.isNotEmpty(records)){
            List<String> codeList = records.stream().map(TableCensus::getYhbh).collect(Collectors.toList());
            //修改下载标识
            update(Wrappers.<TableCensus>lambdaUpdate().in(TableCensus::getYhbh, codeList)
                    .eq(TableCensus::getCensusYear,DateUtil.format(new Date(),"yyyy"))
                    .set(TableCensus::getXzbz,1));
        }

        return mapPage;
    }


    /**
     * 普查记录统计
     *
     * @param dto dto
     * @return list
     */
    @Override
    public List<Map> censusRecordsCount(CensusQueryDto dto) {
        List<Map> dataMap = baseMapper.censusRecordsCount(dto);
        if (CollectionUtils.isNotEmpty(dataMap)) {
            Map<String, List<Map>> groupMap = dataMap.stream().collect(Collectors.groupingBy(m -> String.valueOf(m.get("cbqy"))));
            for (Map map : dataMap) {
                String zone = String.valueOf(map.get("cbqy"));
                List<Map> groupList = groupMap.get(zone);
                int allNum = 0;
                for (Map group : groupList) {
                    allNum += Integer.parseInt(String.valueOf(group.get("totalNum")));
                }
                map.put("all", allNum);
            }
        }
        return dataMap;
    }

    /**
     * 普查记录列表
     *
     * @param dto dto
     * @return page
     */
    @Override
    public Page<Map> censusRecordPage(Page page, CensusQueryDto dto) {
        return baseMapper.censusRecordPage(page, dto);
    }


    /*-------------------------------自定义-----------------------------*/

    /**
     * 自定义方法
     *
     * @param tableCensus     普查记录
     * @param electronicFiles 文件
     * @param mechanicalFiles 文件
     * @param tableFiles      表文件
     * @return TableCensus
     */
    private TableCensus fileSave(TableCensus tableCensus, List<MultipartFile> electronicFiles, List<MultipartFile> mechanicalFiles, List<MultipartFile> tableFiles) {
        if (!CollectionUtils.isEmpty(electronicFiles)) {
            StringBuilder electrinic = fileUpLoadService.saveFileList(electronicFiles, "/xrty/tableCensus/", "TF_YWPN_TABLECENSUS_W");
            if (electrinic != null) {
                tableCensus.setJxwj(electrinic.toString());
            }
        }
        if (!CollectionUtils.isEmpty(mechanicalFiles)) {
            StringBuilder mechanical = fileUpLoadService.saveFileList(mechanicalFiles, "/xrty/tableCensus/", "TF_YWPN_TABLECENSUS_W");
            if (mechanical != null) {
                tableCensus.setDzwj(mechanical.toString());
            }
        }
        if (!CollectionUtils.isEmpty(tableFiles)) {
            StringBuilder tablefile = fileUpLoadService.saveFileList(tableFiles, "/xrty/tableCensus/", "TF_YWPN_TABLECENSUS_W");
            if (tablefile != null) {
                tableCensus.setSbwj(tablefile.toString());
            }
        }
        return tableCensus;
    }
}
