/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.watermeter.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.watermeter.entity.*;
import com.tofly.watermeter.mapper.WateryieldMapper;
import com.tofly.watermeter.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 取供水量数据存储表
 *
 * @author admin
 * @date Sat May 28 00:00:00 CST 2022
 */
@Service("wateryieldService")
public class WateryieldServiceImpl extends ServiceImpl<WateryieldMapper, Wateryield> implements WateryieldService {


    @Autowired
    private WateryieldconfigService wateryieldconfigService;
    @Autowired
    private WateryieldMapper wateryieldMapper;
    @Autowired
    private HourwateryieldService hourwateryieldService;
    @Autowired
    private DaywateryieldService daywateryieldService;
    @Autowired
    private MonthwateryieldService monthwateryieldService;
    @Autowired
    private ReadmeterhisService readmeterhisService;

    private final static String FORMAT_HOUR = "yyyy-mm-dd hh24";
    private final static String FORMAT_DAY = "yyyy-mm-dd";
    private final static String FORMAT_MONTH = "yyyy-mm";

//    @Autowired
//    private WateryieldService wateryieldService;
//    @Autowired
//    private BwrkdetailService bwrkdetailService;
//
//    private final static String WATER_YIELD_TABLE = "TF_YWPN_WATERYIELD_W";
//
//    private final static Integer MAX_INSERT_NUM = 1000;
//    @Value("${meter.factory.oneFactory}")
//    private long oneFactory;
//    @Value("${meter.factory.twoFactory}")
//    private long twoFactory;
//    @Value("${meter.factory.sumFactory}")
//    private long factory;


    @Override
    public boolean handleWaterYield() {
        //获取配置信息
        List<Wateryieldconfig> configList = wateryieldconfigService.list();
        if (CollectionUtil.isNotEmpty(configList)) {
            List<String> itcdList = configList.stream().map(Wateryieldconfig::getItem).collect(Collectors.toList());


            DateTime date = DateUtil.date();
            //获取前一个小时
            DateTime beforeHour = DateUtil.offsetHour(date, -1);
            //获取前一个天
            DateTime beforeDay = DateUtil.offsetDay(date, -1);
            //获取前一个月
            DateTime beforeMonth = DateUtil.offsetMonth(date, -1);

            Daywateryield day;
            Hourwateryield hour;
            Monthwateryield month;

            for (String itcd : itcdList) {

                Hourwateryield hourwateryield = hourwateryieldService.getOne(Wrappers.<Hourwateryield>lambdaQuery()
                        .eq(Hourwateryield::getItcd, itcd)
                        .eq(Hourwateryield::getTime, DateUtil.format(beforeHour, "yyyy-MM-dd HH")));

                hour = new Hourwateryield();
                hour.setFlow(wateryieldMapper.queryWaterNumber(itcd, beforeHour, FORMAT_HOUR));
                hour.setItcd(itcd);
                hour.setTime(DateUtil.format(beforeHour, "yyyy-MM-dd HH"));

                if (hourwateryield != null) {
                    hour.setId(hourwateryield.getId());
                }
                hourwateryieldService.saveOrUpdate(hour);

                Daywateryield daywateryield = daywateryieldService.getOne(Wrappers.<Daywateryield>lambdaQuery()
                        .eq(Daywateryield::getItcd, itcd)
                        .eq(Daywateryield::getTime, DateUtil.format(beforeDay, "yyyy-MM-dd")));

                day = new Daywateryield();
                day.setFlow(wateryieldMapper.queryWaterNumber(itcd, beforeDay, FORMAT_DAY));
                day.setItcd(itcd);
                day.setTime(DateUtil.format(beforeDay, "yyyy-MM-dd"));
                if (daywateryield != null) {
                    day.setId(daywateryield.getId());
                }
                daywateryieldService.saveOrUpdate(day);

                Monthwateryield monthwateryield = monthwateryieldService.getOne(Wrappers.<Monthwateryield>lambdaQuery()
                        .eq(Monthwateryield::getItcd, itcd)
                        .eq(Monthwateryield::getTime, DateUtil.format(beforeMonth, "yyyy-MM")));

                month = new Monthwateryield();
                month.setFlow(wateryieldMapper.queryWaterNumber(itcd, beforeMonth, FORMAT_MONTH));
                month.setItcd(itcd);
                month.setTime(DateUtil.format(beforeMonth, "yyyy-MM"));

                if (monthwateryield != null) {
                    month.setId(monthwateryield.getId());
                }
                monthwateryieldService.saveOrUpdate(month);
            }
        }
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateMeterReadingInfo(String month) {
        List<Map> meterInfo = baseMapper.getMeterInfo(month);
        if (meterInfo != null && !meterInfo.isEmpty()) {
            for (Map map : meterInfo) {
                String yhbh = map.get("yhbh").toString();
                long scds = Long.parseLong(map.get("sl").toString());
                Long bcds = !StringUtils.isEmpty(map.get("bcds")) ? Long.parseLong(map.get("bcds").toString()) : null;
                long waterVal = 0L;
                if (bcds != null) {
                    waterVal = bcds - scds;
                }
                readmeterhisService.update(Wrappers.<Readmeterhis>lambdaUpdate()
                        .set(Readmeterhis::getScds, scds)
                        .set(bcds != null, Readmeterhis::getBcjjl, waterVal)
                        .set(bcds != null, Readmeterhis::getCbsl, waterVal)
                        .set(bcds != null, Readmeterhis::getUpdateBcds, bcds)
                        .set(Readmeterhis::getUpdateScds, scds)
                        .set(bcds != null, Readmeterhis::getUpdateWater, waterVal)
                        .eq(Readmeterhis::getYhbh, yhbh)
                        .eq(Readmeterhis::getIsGeneral, "1")
                        .eq(Readmeterhis::getMonthPlan, month));
            }
        }
        return true;
    }
}
