package com.tofly.scada.mq;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.tofly.scada.entity.Scada;
import com.tofly.scada.entity.ScadaMonitor;
import com.tofly.scada.entity.vo.ScadaDeviceData;
import com.tofly.scada.service.ScadaMonitorService;
import com.tofly.scada.service.ScadaService;
import com.tofly.scada.util.StrUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;

import java.util.Base64;

/**
 * 物联网监听数据配置
 */
@Component
@Slf4j
public class ReceiveHandler {

    @Autowired
    private ScadaService scadaService;
    @Autowired
    private ScadaMonitorService scadaMonitorService;

    //监听scada队列
    @RabbitListener(queues = {RabbitmqConfig.QUEUE_INFORM_SCADA})
    public void receive_scada(Message message) {
        byte[] body = message.getBody();
        String encode = Base64.getEncoder().encodeToString(body);
        byte[] decode = Base64.getDecoder().decode(encode);
        String s = new String(decode);
        ScadaDeviceData scadaDeviceData = JSON.parseObject(StrUtils.underlineToHump(s), ScadaDeviceData.class);
        if (!ObjectUtils.isEmpty(scadaDeviceData) && StringUtils.isNotBlank(scadaDeviceData.getFilePath())) {
            Scada scada = new Scada();
            ScadaMonitor scadaMonitor = new ScadaMonitor();
            scada.setCode(scadaDeviceData.getDeviceId());
            scadaMonitor.setCode(scadaDeviceData.getDeviceId());
            scada.setValue(scadaDeviceData.getFilePath());
            scadaMonitor.setValue(scadaDeviceData.getFilePath());
            scada.setScadaTime(DateUtil.parse(scadaDeviceData.getGenerationTime(), "yyyy-MM-dd HH:mm:ss"));
            scadaMonitor.setScadaTime(DateUtil.parse(scadaDeviceData.getGenerationTime(), "yyyy-MM-dd HH:mm:ss"));
            try {
                scadaService.save(scada);

                scadaMonitorService.remove(new QueryWrapper<ScadaMonitor>().lambda().eq(ScadaMonitor::getCode, scadaMonitor.getCode()));
                scadaMonitorService.save(scadaMonitor);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

}
