package com.tofly.spire.entity.dto;


import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;

/**
 * 大表（管网）(SpireBigGrid)表实体类
 *
 * @author HaiQiu
 * @since 2022-11-11 09:16:07
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "SpireBigGrid对象", description = "大表（管网）")
public class SpireBigGridDto implements Serializable {

    private static final long serialVersionUID = -75133457144472198L;

    /**
     * 主键ID
     */
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;

    /**
     * 累计量(m³)
     */
    @Size(max = 255, message = "累计量(m³)长度不能超过255")
    @ApiModelProperty(value = "累计量(m³)")
    private String accFlow;

    /**
     * 反向累积量 (m³)
     */
    @Size(max = 255, message = "反向累积量 (m³)长度不能超过255")
    @ApiModelProperty(value = "反向累积量 (m³)")
    private String decAccFlow;

    /**
     * 瞬时量 (m³/h)
     */
    @Size(max = 255, message = "瞬时量 (m³/h)长度不能超过255")
    @ApiModelProperty(value = "瞬时量 (m³/h)")
    private String flow;

    /**
     * 进水温度 (℃)
     */
    @Size(max = 255, message = "进水温度 (℃)长度不能超过255")
    @ApiModelProperty(value = "进水温度 (℃)")
    private String inTemp;

    /**
     * 纬度
     */
    @Size(max = 255, message = "纬度长度不能超过255")
    @ApiModelProperty(value = "纬度")
    private String latitude;

    /**
     * 经度
     */
    @Size(max = 255, message = "经度长度不能超过255")
    @ApiModelProperty(value = "经度")
    private String longitude;

    /**
     * 表名
     */
    @Size(max = 255, message = "表名长度不能超过255")
    @ApiModelProperty(value = "表名")
    private String meterName;

    /**
     * 表具状态
     */
    @Size(max = 255, message = "表具状态长度不能超过255")
    @ApiModelProperty(value = "表具状态")
    private String meterStatus;

    /**
     * 表具类型
     */
    @Size(max = 255, message = "表具类型长度不能超过255")
    @ApiModelProperty(value = "表具类型")
    private String meterType;

    /**
     * 压力值 (K_PA)
     */
    @Size(max = 255, message = "压力值 (K_PA)长度不能超过255")
    @ApiModelProperty(value = "压力值 (K_PA)")
    private String pressure;

    /**
     * 表具时间
     */
    @ApiModelProperty(value = "表具时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date readTime;

    /**
     * 表号
     */
    @Size(max = 255, message = "表号长度不能超过255")
    @ApiModelProperty(value = "表号")
    private String sn;

    /**
     * 出水温度 (℃)
     */
    @Size(max = 255, message = "出水温度 (℃)长度不能超过255")
    @ApiModelProperty(value = "出水温度 (℃)")
    private String temp;

    /**
     * 上传时间
     */
    @ApiModelProperty(value = "上传时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date uploadTime;

}

