package com.tofly.water.entity;


import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.haiqiu.tools.excel.annotation.Excel;
import com.tofly.common.constant.SqlCondition;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import org.codehaus.jackson.map.annotate.JsonSerialize;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Null;
import java.util.Date;

/**
 * 安防监控（门禁管理）(SafeMonitor)表实体类
 *
 * @author HaiQiu
 * @since 2022-11-22 13:57:02
 */
@Data
@TableName(value = "TF_WATER_REPORT", resultMap = "waterReportMap")
@ApiModel(value = "WaterReport对象", description = "污水厂排污报表")
public class WaterReport extends Model<WaterReport> {

    private static final long serialVersionUID = -34727060177508749L;

    /**
     * 主键ID
     */
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;

    @ApiModelProperty(value = "水厂Id")
    @NotNull(message = "水厂Id不能为空")
    private Long waterId;


    @ApiModelProperty(value = "水厂名称")
    @TableField(exist = false)
    private String waterName;

    /**
     * 记录日期
     */
    @ApiModelProperty(value = "记录日期")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @NotNull(message = "记录日期不能为空")
    private Date recordDate;

    /**
     * 日进水量(万吨)
     */
    @ApiModelProperty(value = "日进水量(万吨)")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String totalFlowNow;

    /**
     * 上日日进水量(万吨)
     */
    @ApiModelProperty(value = "上日流量累计流量(万吨)")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String totalFlowLast;

    /**
     * 日处理水量(万吨)，出水
     */
    @ApiModelProperty(value = "日处理水量(万吨)")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String dealWaterVolume;

    /**
     * cod_进水(mg/L)
     */
    @ApiModelProperty(value = "cod_进水(mg/L)")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String codWaterIn;

    /**
     * cod_出水(mg/L)
     */
    @ApiModelProperty(value = "cod_出水(mg/L)")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String codWaterOut;

    /**
     * bod3_进水(mg/L)
     */
    @ApiModelProperty(value = "bod3_进水(mg/L)")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String bodWaterIn;

    /**
     * bod3_出水(mg/L)
     */
    @ApiModelProperty(value = "bod3_出水(mg/L)")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String bodWaterOut;


    /**
     * 氨氮_进水(mg/L)
     */
    @ApiModelProperty(value = "氨氮_进水(mg/L)")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String adWaterIn;

    /**
     * 氨氮_出水(mg/L)
     */
    @ApiModelProperty(value = "氨氮_出水(mg/L)")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String adWaterOut;

    /**
     * SS_进水(mg/L)
     */
    @ApiModelProperty(value = "SS_进水(mg/L)")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String ssWaterIn;

    /**
     * SS_出水(mg/L)
     */
    @ApiModelProperty(value = "SS_出水(mg/L)")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String ssWaterOut;

    /**
     * 总磷_进水(mg/L)
     */
    @ApiModelProperty(value = "总磷_进水(mg/L)")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String zlWaterIn;

    /**
     * 总磷_出水(mg/L)
     */
    @ApiModelProperty(value = "总磷_出水(mg/L)")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String zlWaterOut;


    /**
     * 总氮_进水(mg/L)
     */
    @ApiModelProperty(value = "总氮_进水(mg/L)")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String zdWaterIn;

    /**
     * 总氮_出水(mg/L)
     */
    @ApiModelProperty(value = "总氮_出水(mg/L)")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String zdWaterOut;


    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    @TableField(updateStrategy=FieldStrategy.NOT_NULL)
    private String remark;


    /**
     * 创建人
     */
    @ApiModelProperty(value = "创建人")
    private Long createUser;

    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;
    /**
     * 修改时间
     */
    @ApiModelProperty(value = "修改时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updateTime;
    /**
     * 更新人员
     */
    @ApiModelProperty(value = "更新人员")
    private Long updateUser;

    /**
     * 更新人员
     */
    @ApiModelProperty(value = "创建人员")
    @TableField(exist = false)
    private String createUserName;
    /**
     * 更新人员
     */
    @ApiModelProperty(value = "更新人员")
    @TableField(exist = false)
    private String updateUserName;


    @ApiModelProperty(value = "记录日期，转换为天数")
    @TableField(exist = false)
    private String recordDay;

}

