package com.tofly.analyse.controller;


import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.analyse.entity.ThematicMap;
import com.tofly.analyse.service.ThematicMapService;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/thematicMap")
@Api(tags = "专题地图管理")
public class ThematicMapController {
	
	@Autowired
	private ThematicMapService thematicMapService;
	
    /**
    * 分页查询
    * @param page 分页对象
    * @param projectInfo 存储排水项目
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "专题地图分页查询")
    public ResultRespone getProjectInfoPage(Page page, @ModelAttribute ThematicMap thematicMap) {
      return ResultRespone.success(thematicMapService.page(page, Wrappers.query(thematicMap)));
    }
    

    /**
     * 通过ID查询单个专题地图
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询单个专题地图")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("id" ) String id) {
      return ResultRespone.success(thematicMapService.getById(id));
    }

    /**
     * 新增专题地图
     * @param thematicMap 专题地图
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增专题地图信息" )
    @ApiOperation(value = "新增专题地图信息")
    @PostMapping
    public ResultRespone save(@RequestBody ThematicMap thematicMap) {
      return ResultRespone.success(thematicMapService.save(thematicMap));
    }

    /**
     * 修改专题地图信息
     * @param thematicMap 专题地图
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改专题地图信息" )
    @ApiOperation(value = "修改专题地图信息")
    @PutMapping
    public ResultRespone updateById(@RequestBody ThematicMap thematicMap) {
      return ResultRespone.success(thematicMapService.updateById(thematicMap));
    }
    
    
    /**
     * 通过id删除专题地图信息
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除专题地图信息" )
    @ApiOperation(value = "通过id删除专题地图信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable String id) {
      return ResultRespone.success(thematicMapService.removeById(id));
    }
    
    
    @ToFlyAppLog(title="通过ids批量删除专题地图信息" )
    @ApiOperation(value = "通过isd批量删除专题地图信息")
    @DeleteMapping("/removeByIds" )
    public ResultRespone removeByIds(@ApiParam(value = "通过ID拼接的字符串,以逗号隔开.如,'1,2,3,4,5,6,7",name = "ids",required = true) String ids) {
        String[] idss=ids.split(",");
        List<Long> list=new ArrayList();
        for(String id:idss) {
        	list.add(Long.valueOf(id));
        }
        return ResultRespone.success(thematicMapService.removeByIds(list));
    }

    
    
}