package com.tofly.analyse.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.analyse.entity.TitleLabel;
import com.tofly.analyse.service.TitleLabelService;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.util.SecurityUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

@RestController
@RequestMapping("/titleLabel")
@Api(tags = "书签管理")
public class TitileLabelController {
	
   @Autowired	
   private TitleLabelService titleLabelService;

   /**
    * 书签信息新增
    *
    * @param publicHouse 书签信息
    * @return ResultRespone
    */
   @ToFlyAppLog(title = "书签信息新增")
   @ApiOperation(value = "书签信息新增")
   @PostMapping
   public ResultRespone saveTitleLabel(HttpServletRequest servletRequest,@RequestBody TitleLabel titleLabel) {
	   
	   try {
		   titleLabel.setUserId(SecurityUtils.getUserId());
		   titleLabel.setCreateTime(new Date());
		   titleLabelService.saveTitleLabel(servletRequest,titleLabel);
	   }catch (Exception e) {
		   e.printStackTrace();
	   }
	   
       return ResultRespone.success();
   }
   /**
    * 书签信息按条件分页查询
    *
    * @param page
    * @param houseMortgage
    * @return
    */
   @ApiOperation(value = "书签信息按条件分页查询")
   @GetMapping("/page")
   public ResultRespone getTitleLabelPage(Page page, @ModelAttribute TitleLabel titleLabel) {
	   
	   Page<Map<String, Object>> pageList = null;
	   try {
	   		Page<Map<String, Object>> pageParam = new Page<>(page.getCurrent(), page.getSize());
	        Map<String, Object> map = new HashMap<>();
	        map.put("markName", titleLabel.getMarkName());
	        map.put("userId", SecurityUtils.getUserId());
	        pageList = titleLabelService.getTitleLabelPage(pageParam, map);
	   }catch (Exception e) {
			e.printStackTrace();
	   }
   	
       return ResultRespone.success(pageList);
   }
   
   /**
    * 根据ID查询书签情况详情
    *
    * @param id 根据ID查询书签情况详情
    * @return ResultRespone
    */
   @ToFlyAppLog(title = "根据ID查询书签情况详情")
   @ApiOperation(value = "根据ID查询书签情况详情")
   @GetMapping("/{id}")
   public ResultRespone getTitleLabelById(@PathVariable("id") Long id) {
       return ResultRespone.success(titleLabelService.getById(id));
   }
      
   @ToFlyAppLog(title="通过ids批量删除书签信息" )
   @ApiOperation(value = "通过isd批量删除书签信息")
   @DeleteMapping("/removeByIds" )
   public ResultRespone removeByIds(@ApiParam(value = "通过ID拼接的字符串,以逗号隔开.如,'1,2,3,4,5,6,7",name = "ids",required = true) String ids) {
       String[] idss=ids.split(",");
       List<Long> list=new ArrayList();
       for(String id:idss) {
       	list.add(Long.valueOf(id));
       }
       return ResultRespone.success(titleLabelService.removeByIds(list));
   }
   
}