package com.tofly.analyse.service.impl;


import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supermap.data.*;
import com.tofly.analyse.entity.DataListDto;
import com.tofly.analyse.entity.DataSet;
import com.tofly.analyse.mapper.DataSetMapper;
import com.tofly.analyse.service.DataSetService;
import com.tofly.analyse.util.analysis.CamelToUtil;
import com.tofly.analyse.util.analysis.DatasourceConfig;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.StringUtil;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author zhoux
 */
@AllArgsConstructor
@Service("dataSetService")
public class DataSetServiceImpl extends ServiceImpl<DataSetMapper, DataSet> implements DataSetService {

	@Autowired
	private DataSetMapper dataSetMapper;

	@Autowired
	private DatasourceConfig datasourceConfig;

	@Override
	public List<String> getColData(Map<String, String> queryParams) {
		return dataSetMapper.getColData(queryParams);
	}

	@Override
	public ResultRespone getFieldValue(String dataSetName) {
		List<Map> result=new ArrayList<>();
		datasourceConfig.loginSuperMap();
		Workspace workspace = new Workspace();
		Datasource datasource = null;
		if (StringUtil.isNotEmpty(dataSetName)) {
			try {
				datasource = datasourceConfig.getDatasource(workspace);
				Datasets datasets = datasource.getDatasets();
				if (datasets.contains(dataSetName)) {
					DatasetVector datasetVector = (DatasetVector) datasets.get(dataSetName);
					Recordset recordset = datasetVector.getRecordset(false, CursorType.STATIC);
					FieldInfos fieldInfos = recordset.getFieldInfos();
					FieldInfo[] fieldInfosArray = fieldInfos.toArray();
					if (fieldInfosArray != null && fieldInfosArray.length > 0) {
						for (FieldInfo f : fieldInfosArray) {
							Map<String, Object> data=new HashMap<>();
							data.put("SmFieldName",f.getName());
							data.put("SmFieldCaption",f.getCaption());
							data.put("SmFieldType",f.getType().value());
							data.put("isRequired",f.isRequired());
							result.add(data);
						}
					}
				} else {
					return ResultRespone.failed("不存在此图层");
				}
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				if (datasource.isOpened())
					datasource.close();
				workspace.close();
			}
		} else {
			return ResultRespone.failed("数据集不能为空");
		}
		return ResultRespone.success(result);
	}

	@Override
	public List<Map> getDataSetData(Map<String, String> queryParams) {
		return dataSetMapper.getDataSetData(queryParams);
	}

	@Override
	public Page<Map> getDataList(Page page, DataListDto param) {
		List<Map> result=new ArrayList<>();
		List<Map> dataList = dataSetMapper.getDataList(page, param);
		if(CollUtil.isNotEmpty(dataList)){
			Map<String,String>  queryParams = new HashMap<String,String>();
			queryParams.put("dataSetName", param.getDataSet());
			List<Map> dataSetData = dataSetMapper.getDataSetData(queryParams);
			if(CollUtil.isNotEmpty(dataSetData)){
				Map<String, Object> map=new HashMap<>();
				dataSetData.forEach(d->{
					Object smFieldName = d.get("smfieldname");
					String s = CamelToUtil.underlineToCamel(smFieldName.toString());
					map.put(s.toLowerCase(),smFieldName);
				});
				if(CollUtil.isNotEmpty(map)){
					dataList.forEach(d->{
						Map<String,Object> colMap=new HashMap<>();
						d.forEach((k,v)->{
							if(map.get(k.toString().toLowerCase())!=null){
								colMap.put(map.get(k.toString().toLowerCase()).toString(),v);
							}
						});
						result.add(colMap);
					});
				}
			}
		}
		return page.setRecords(result);
	}

}