package com.tofly.analyse.service.impl;


import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supermap.data.*;
import com.tofly.analyse.entity.LayerFiledConfig;
import com.tofly.analyse.entity.Pipeline;
import com.tofly.analyse.entity.em.LayerTypeEnum;
import com.tofly.analyse.entity.em.PipeLineQueryEnum;
import com.tofly.analyse.entity.em.PipeNodeQueryEnum;
import com.tofly.analyse.mapper.PipelineMapper;
import com.tofly.analyse.service.LayerFiledConfigService;
import com.tofly.analyse.service.PipelineService;
import com.tofly.analyse.util.analysis.CamelToUtil;
import com.tofly.analyse.util.analysis.DatasourceConfig;
import com.tofly.common.core.entity.ResultRespone;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhoux
 */
@AllArgsConstructor
@Service("pipelineService")
public class PipelineServiceImpl extends ServiceImpl<PipelineMapper, Pipeline> implements PipelineService {

    @Autowired
    private PipelineMapper pipelineMapper;

    @Autowired
    private LayerFiledConfigService layerFiledConfigService;

    @Autowired
    private DatasourceConfig datasourceConfig;

    /**
     * 管线追踪性分析业务方法
     */
    @Override
    public Map<String, Object> getPipelineConnect(Map<String, String> pipeParams) {
        Map<String, Object> map = new HashMap<>();
        List<Map<String, Object>> edges = new ArrayList<>();
        List<Object> nodes = new ArrayList<>();
        List<Map<String, Object>> returnList = new ArrayList<>();
        LayerFiledConfig layfiledConfig = layerFiledConfigService.getOne(Wrappers.<LayerFiledConfig>query().lambda().eq(LayerFiledConfig::getLayerName, pipeParams.get("tableName")));
        if (layfiledConfig != null) {
            String sp = CamelToUtil.underlineToCamel(layfiledConfig.getSPoint());
            String ep = CamelToUtil.underlineToCamel(layfiledConfig.getEPoint());
            String no = CamelToUtil.underlineToCamel(layfiledConfig.getNo());
            String dr = CamelToUtil.underlineToCamel(layfiledConfig.getDirFlow());

            Map<String, String> queryMap = new HashMap<String, String>();
            queryMap.put("pipePointNo", pipeParams.get("pipePointNo"));
            queryMap.put("tableName", pipeParams.get("tableName"));
            queryMap.put("no", layfiledConfig.getNo());
            if(StringUtils.isNotBlank(dr)){
                queryMap.put("dir", layfiledConfig.getDirFlow());
            }
            queryMap.put("sPoint", layfiledConfig.getSPoint());
            queryMap.put("ePoint", layfiledConfig.getEPoint());
            List<Map<String, Object>> gisInfo = pipelineMapper.getPipeInfos(queryMap);
            if (gisInfo.size() != 0) {
                returnList.addAll(gisInfo);
                Map<String, Object> pipeInfo = gisInfo.get(0);
                String sPoint = null;
                String ePonit = null;
                String drs =null;
                if(StringUtils.isNotBlank(dr)){
                     drs = pipeInfo.get(dr).toString();
                }
                if (pipeParams.get("tableName").contains("PS")) {
                    if (drs.equals("0")) {
                        sPoint = pipeInfo.get(sp).toString();
                        ePonit = pipeInfo.get(ep).toString();
                    } else {
                        sPoint = pipeInfo.get(ep).toString();
                        ePonit = pipeInfo.get(sp).toString();
                    }
                } else {
                    sPoint = pipeInfo.get(sp).toString();
                    ePonit = pipeInfo.get(ep).toString();
                }
                String sid = pipeInfo.get(no).toString();
                List<Map<String, Object>> nextMaps=new ArrayList<>();
                Map<String, Object> newValueMap = new HashMap<String, Object>();
                newValueMap.put("nextPoint", sPoint);
                if(StringUtils.isNotBlank(dr)){
                    newValueMap.put("drs", drs);
                }
                newValueMap.put("sid", sid);
                Map<String, Object> newValueMap2 = new HashMap<String, Object>();
                newValueMap2.put("nextPoint", ePonit);
                if(StringUtils.isNotBlank(dr)){
                    newValueMap2.put("drs", drs);
                }
                newValueMap2.put("sid", sid);
                String type=null;
                if (pipeParams.get("type") != null) {
                     type = pipeParams.get("type").toString();
                    if (type.equals("0")) {
                        nextMaps.add(newValueMap);
                    } else {
                        nextMaps.add(newValueMap2);
                    }
                } else {
                    type="1";
                    //获取出水口管线
                    nextMaps.add(newValueMap2);
                }
                checkConnect(type, queryMap.get("tableName"), nextMaps, returnList, layfiledConfig);
            }
            if (CollUtil.isNotEmpty(returnList)) {
                edges = returnList.stream().distinct().collect(Collectors.toList());
                List<String> points = new ArrayList<>();
                edges.forEach(d -> {
                    Object sPoint = d.get(sp);
                    Object ePoint = d.get(ep);
                    if (sPoint != null) {
                        String s = sPoint.toString();
                        if (!points.contains(s)) {
                            points.add(s);
                        }
                    }
                    if (ePoint != null) {
                        String s = ePoint.toString();
                        if (!points.contains(s)) {
                            points.add(s);
                        }
                    }
                });
                if (CollUtil.isNotEmpty(points)) {
                    Map<String, Object> nodeMap = new HashMap<String, Object>();
                    nodeMap.put("points", points);
                    nodeMap.put("tableName", layfiledConfig.getNodeLayerName());
                    nodeMap.put("no", layfiledConfig.getNodeNo());
                    nodeMap.put("code", layfiledConfig.getCode());
                    List<String> pointByExpNos = pipelineMapper.getPointByExpNos(nodeMap);
                    if (CollUtil.isNotEmpty(pointByExpNos)) {
                        nodes.addAll(pointByExpNos);
                    }
                }
            }
        }
        map.put("edges", edges);
        map.put("nodes", nodes);
        return map;
    }



    public void checkConnect(String type, String tableName,
                                                            List<Map<String, Object>> nextMaps, List<Map<String, Object>> list, LayerFiledConfig layfiledConfig) {

        List<String> sids = nextMaps.stream().map(d -> d.get("sid").toString()).collect(Collectors.toList());
        List<String> nextPoints = nextMaps.stream().map(d -> d.get("nextPoint").toString()).collect(Collectors.toList());
        List<Map<String, Object>> nextMap = new ArrayList<>();

        String sp = CamelToUtil.underlineToCamel(layfiledConfig.getSPoint());
        String ep = CamelToUtil.underlineToCamel(layfiledConfig.getEPoint());
        String no = CamelToUtil.underlineToCamel(layfiledConfig.getNo());
        String model = CamelToUtil.underlineToCamel(layfiledConfig.getSubtype());

        Map<String, Object> startMap = new HashMap<String, Object>();
        startMap.put("tableName", tableName);
        startMap.put("nextPoints", nextPoints);
        startMap.put("no", layfiledConfig.getNo());
        startMap.put("sPoint", layfiledConfig.getSPoint());
        startMap.put("ePoint", layfiledConfig.getEPoint());
        startMap.put("model", layfiledConfig.getSubtype());
        startMap.put("tableNameNode", layfiledConfig.getNodeLayerName());
        if (tableName.contains("PS")) {
            startMap.put("type", type);
            startMap.put("dir", layfiledConfig.getDirFlow());
        }
        //查找管线对应start和end节点
        List<Map<String, Object>> result = pipelineMapper.getPipeInfos(startMap);

        //删除查到的重复管线
        Iterator<Map<String, Object>> nextEndIterator = result.iterator();
        while (nextEndIterator.hasNext()) {
            Map<String, Object> ob = nextEndIterator.next();
            if(sids.contains(ob.get(no).toString())){
                nextEndIterator.remove();
            }
        }

        if(!result.isEmpty()){
            for (int i = 0; i < result.size(); i++) {
                String sid = result.get(i).get(no).toString();
                String startSid = null;
                if(nextPoints.contains(result.get(i).get(sp))){
                    startSid = result.get(i).get(ep).toString();
                }else{
                    startSid = result.get(i).get(sp).toString();
                }
                if(!list.contains(result.get(i))){
                    Map<String, Object> newValueMap = new HashMap<String, Object>();
                    if (result.get(i).get(model) != null) {
                        newValueMap.put("model",  result.get(i).get(model));
                    }
                    newValueMap.put("sid", sid);
                    newValueMap.put("nextPoint", startSid);
                    nextMap.add(newValueMap);
                    list.add(result.get(i));
                }
            }
        }

        if(CollUtil.isNotEmpty(nextMap)){
            List<Map<String, Object>> nextMapListNew = new ArrayList<>();
            nextMap.forEach(d->{
                Object existModel = d.get("model");
                if(existModel!=null){
                    if (type.equals("0")) {
                        if (layfiledConfig.getSySubtypeValue().contains(existModel.toString())) {
                            return;
                        } else {
                            nextMapListNew.add(d);
                        }
                    } else {
                        if (layfiledConfig.getXySubtypeValue().contains(existModel.toString())) {
                            return;
                        } else {
                            nextMapListNew.add(d);
                        }
                    }
                }else{
                    nextMapListNew.add(d);
                }
            });
            if(CollUtil.isNotEmpty(nextMapListNew)){
                checkConnect(type,tableName, nextMapListNew, list, layfiledConfig);
            }
        }
    }

    @Override
    public Map<String, Object> getPointByExpNo(String expNo) {
        return baseMapper.getPointByExpNo(expNo);
    }

    @Override
    public ResultRespone getDataBykeyValue(String layerName,String keyValue) {
        List<Map<String, Object>> resultMap = new ArrayList<>();
        Workspace workspace = new Workspace();
        Datasource datasource = null;
        try {
            datasource = datasourceConfig.getDatasource(workspace);
            Datasets datasets = datasource.getDatasets();
            if(StringUtils.isNotBlank(layerName)){
                DatasetVector datasetVector = (DatasetVector) datasets.get(layerName);
                String name = datasetVector.getName();
                queryByKeyValue(keyValue, resultMap, datasetVector, name);
            }else{
                int count = datasets.getCount();
                if (count > 0) {
                    for (int i = 0; i < count; i++) {
                        DatasetVector datasetVector = (DatasetVector) datasets.get(i);
                        String name = datasetVector.getName();
                        if (datasetVector.getType() != DatasetType.NETWORK3D  && !name.contains("SM")  && name.endsWith("B") && (name.contains("NODE") || name.contains("PIPE"))) {
                            queryByKeyValue(keyValue, resultMap, datasetVector, name);
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (datasource.isOpened())
                datasource.close();
            workspace.close();
        }
        return ResultRespone.success(resultMap);
    }

    private static void queryByKeyValue(String keyValue, List<Map<String, Object>> resultMap, DatasetVector datasetVector, String name) {
        Recordset recordset = datasetVector.query("(SID like '%"+ keyValue +"%') or (ADDRESS like '%"+ keyValue +"%')", CursorType.STATIC);
        recordset.moveFirst();
        while (!recordset.isEOF()) {
            Map<String,Object> dataMap=new HashMap<>();
            if(name.contains("NODE")){
                for (PipeNodeQueryEnum e : PipeNodeQueryEnum.values()){
                    Object fieldValue = recordset.getFieldValue(e.getCode());
                    dataMap.put(e.getCode(), fieldValue);
                }
                if(dataMap.get(PipeNodeQueryEnum.FTTY.getCode())!=null){
                    dataMap.put("type",LayerTypeEnum.getNameForValue(name)+dataMap.get(PipeNodeQueryEnum.FTTY.getCode()));
                }else{
                    dataMap.put("type",LayerTypeEnum.getNameForValue(name)+dataMap.get(PipeNodeQueryEnum.APTY.getCode()));
                }
            }else{
                dataMap.put("type",LayerTypeEnum.getNameForValue(name)+"管线");
                for (PipeLineQueryEnum e : PipeLineQueryEnum.values()){
                    Object fieldValue = recordset.getFieldValue(e.getCode());
                    dataMap.put(e.getCode(), fieldValue);
                }
            }
            resultMap.add(dataMap);
            recordset.moveNext();
        }
        recordset.dispose();
    }
}