package com.tofly.analyse.util.analysis;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.supermap.analyst.networkanalyst.*;
import com.supermap.data.CursorType;
import com.supermap.data.DatasetVector;
import com.supermap.data.Recordset;
import com.tofly.analyse.entity.LayerFiledConfig;
import com.tofly.analyse.entity.analyse.BurstResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * 关阀分析
 *
 * @author zhoux
 */
@Component
public class FormShutoffValveAnalyst {

    @Autowired
    public ConnectionAnalysis connectionAnalysis;

    @Autowired
    private AnalystConfig facilityAnalystConfig;


    //关阀分析
    public BurstResult formShutoffValveAnalyst(FacilityAnalyst facilityAnalyst, LayerFiledConfig layfiledConfig, String sid, List<Integer> sourceIds) {
        BurstResult brResult = new BurstResult();
        List<String> affectedEdge = new ArrayList<>();
        List<Object> affectedNode = new ArrayList<>();
        List<Object> mustCloseNode = new ArrayList<>();

        if (StringUtils.isNotBlank(sid)) {
            List<String> iValveNodeIds = Arrays.asList(sid.split(","));
            //有流向边集合
            List<Integer> lstValidEdgeIds = new ArrayList<Integer>();
            //不确定流向边集合
            List<Map<String,String>> lstValidEdgeIds2 = new ArrayList<Map<String,String>>();
            //分析阀门是否会关闭后产生影响，转换得到模拟爆管分析的边Id
            for (String iValveNodeId : iValveNodeIds) {
                String strSQL = "START_SID= '" + iValveNodeId + "' or END_SID = '" + iValveNodeId + "' ";
                Recordset recConnectedPipe = facilityAnalyst.getAnalystSetting().getNetworkDataset().query(strSQL, CursorType.STATIC);
                recConnectedPipe.moveFirst();
                //记录关阀后，受影响的待分析源管线
                while (recConnectedPipe.isEOF() == false) {
                    //只取无流向、和下游管线(阀门为起点流向为0正向，阀门为终点流向1)
                    int dir = recConnectedPipe.getInt32(layfiledConfig.getDir());
                    String spoint = recConnectedPipe.getString(layfiledConfig.getSPoint());
                    String epoint = recConnectedPipe.getString(layfiledConfig.getEPoint());
                    String no = recConnectedPipe.getString(layfiledConfig.getNo());
                    int smfnode = recConnectedPipe.getInt32("smfnode");
                    int smtnode = recConnectedPipe.getInt32("smtnode");
                    if(dir!=3 ){
                        if(dir==2){
                            Map<String, String> map = new HashMap<>();
                            map.put("sid",no);
                            if(iValveNodeId.equals(spoint)){
                                map.put("nodeId",smfnode+"");
                            }else{
                                map.put("nodeId",smtnode+"");
                            }
                            lstValidEdgeIds2.add(map);
                        }else{
                            if((iValveNodeId.equals(spoint) && dir==0) || (iValveNodeId.equals(epoint) && dir==1)) {
                                lstValidEdgeIds.add(recConnectedPipe.getInt32("smid"));
                                affectedEdge.add(no);
                                affectedNode.add(spoint);
                                affectedNode.add(epoint);
                            }
                        }
                    }
                    recConnectedPipe.moveNext();
                }
                recConnectedPipe.close();
                recConnectedPipe.dispose();
            }
            //如果确定流向，则直接做下游分析
            if (CollUtil.isNotEmpty(lstValidEdgeIds)) {
                downAnalyse(facilityAnalyst, layfiledConfig, affectedEdge, affectedNode, lstValidEdgeIds);
            }
            if (CollUtil.isNotEmpty(lstValidEdgeIds2)) {
                //如果不确定流向，先做有障碍连通性分析，找出除去关阀的障碍点之外的障碍点
                for(Map<String,String> d:lstValidEdgeIds2){
                    String sid1 = d.get("sid");
                    String nodeId = d.get("nodeId");
                    List<Map<String, String>> collect1 = lstValidEdgeIds2.stream().filter(s -> !s.get("sid").equals(d.get("sid"))).collect(Collectors.toList());
                    List<String> barrEdge=new ArrayList<>();
                    collect1.forEach(s->{
                        barrEdge.add(s.get("sid"));
                    });
                    Map<String, Object> arrAftEdges = connectionAnalysis(facilityAnalyst.getAnalystSetting().getNetworkDataset().getName(), layfiledConfig, sid1, true, barrEdge);

                    List<Integer> barrierNodeIds = (List<Integer>) arrAftEdges.get("barrierNodeIds");
                    List<String> edgeSids = (List<String>) arrAftEdges.get("edgeSids");
                    List<Object> tracedNode = (List<Object>) arrAftEdges.get("tracedNode");
                    Map<Integer, Object> barrierNodeConc = (Map<Integer, Object>) arrAftEdges.get("barrierNodeConc");
                    List<Integer> collect = barrierNodeIds.stream().filter(s -> !s.toString().equals(nodeId)).collect(Collectors.toList());
                    List<Object> mustCloseNode1 = new ArrayList<>();
                    List<Integer> needAnalyNodeIds = new ArrayList<>();
                    //判断是否连通水源，如果连通说明不影响，如果都不连通水源，说明该区域形成了闭合区域，则该区域为影响范围
                    getMustNode(facilityAnalyst.getAnalystSetting().getNetworkDataset(),sourceIds, mustCloseNode1, collect, needAnalyNodeIds, barrierNodeConc,new int[]{Integer.valueOf(nodeId)});
                    if(CollUtil.isEmpty(mustCloseNode1)){
                        affectedEdge.addAll(edgeSids);
                        affectedNode.addAll(tracedNode);
                        getAnalystData(facilityAnalyst, layfiledConfig, affectedEdge, affectedNode, edgeSids, needAnalyNodeIds);
                    }
                }
            }
        }
        if (CollUtil.isNotEmpty(affectedEdge)) {
            affectedEdge = affectedEdge.stream().distinct().collect(Collectors.toList());
        }
        if (CollUtil.isNotEmpty(affectedNode)) {
            affectedNode = affectedNode.stream().distinct().collect(Collectors.toList());
        }
        if (CollUtil.isNotEmpty(mustCloseNode)) {
            mustCloseNode = mustCloseNode.stream().distinct().collect(Collectors.toList());
        }
        brResult.setAffectedEdge(affectedEdge);
        brResult.setAffectedNode(affectedNode);
        brResult.setMustCloseNode(mustCloseNode);
        return brResult;
    }

    private static void downAnalyse(FacilityAnalyst facilityAnalyst, LayerFiledConfig layfiledConfig, List<String> affectedEdge, List<Object> affectedNode, List<Integer> lstValidEdgeIds) {
        List<Integer> downList=new ArrayList<>();
        for (Integer lstValidEdgeId : lstValidEdgeIds) {
            //确定流向则做下游分析
            FacilityAnalystResult resultSourceFromEdge = facilityAnalyst.traceDownFromEdge(Integer.valueOf(lstValidEdgeId).intValue(), "Length", true);
            if (resultSourceFromEdge != null && resultSourceFromEdge.getEdges().length>0) {
                List<Integer> collect = Arrays.stream(resultSourceFromEdge.getEdges()).boxed().collect(Collectors.toList());
                downList.addAll(collect);
            }
        }
        if(CollUtil.isNotEmpty(downList)){
            List<Integer> collect = downList.stream().distinct().collect(Collectors.toList());
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < collect.size(); i++) {
                sb.append(collect.get(i) + ",");
            }
            String strEdgeIds = sb.toString().substring(0, sb.toString().length() - 1);
            Recordset recValveNode = facilityAnalyst.getAnalystSetting().getNetworkDataset().query("smid in ("+strEdgeIds+")", CursorType.STATIC);
            recValveNode.moveFirst();
            while (recValveNode.isEOF() == false) {
                String no = recValveNode.getString(layfiledConfig.getNo());
                String startSid = recValveNode.getString(layfiledConfig.getSPoint());
                String endSid = recValveNode.getString(layfiledConfig.getEPoint());
                affectedEdge.add(no);
                affectedNode.add(startSid);
                affectedNode.add(endSid);
                recValveNode.moveNext();
            }
            recValveNode.close();
            recValveNode.dispose();
        }
    }

    private void getMustNode(DatasetVector vecPNetwork, List<Integer> sourceIds, List<Object> mustCloseNode,
                             List<Integer> barrierNodeIds, List<Integer> needAnalyNodeIds, Map<Integer, Object> barrierNodeConc, int[] barrId) {
        TransportationAnalyst m_transAnalyst = facilityAnalystConfig.loadModel(vecPNetwork);
        if (CollUtil.isNotEmpty(barrierNodeIds)) {

            barrierNodeIds.forEach(d -> {
                Object o = barrierNodeConc.get(d);
                AtomicReference<Boolean> flag = new AtomicReference<>(false);
                sourceIds.forEach(s -> {
                    int[] nodeIDs = new int[] { d,s };
                    TransportationAnalystParameter parameter = new TransportationAnalystParameter();
                    parameter.setBarrierNodes(barrId);
                    parameter.setNodes(nodeIDs);
                    parameter.setNodesReturn(true);
                    parameter.setEdgesReturn(true);
                    TransportationAnalystResult taResult = m_transAnalyst.findTSPPath(parameter, false);
                    if (null!=taResult ) {
                        if(null!=taResult.getEdges() && taResult.getEdges().length>0){
                            if (o != null) {
                                if (!mustCloseNode.contains(o)) {
                                    mustCloseNode.add(o);
                                }
                            }
                            flag.set(true);
                            return;
                        }
                    }
                });
                if (!flag.get()) {
                    if(!needAnalyNodeIds.contains(d)){
                        needAnalyNodeIds.add(d);
                    }
                }
            });

        }
    }

    private void getAnalystData(FacilityAnalyst facilityAnalystParam, LayerFiledConfig layfiledConfig, List<String> affectedEdge, List<Object> affectedNode,
                                List<String> edgeSids, List<Integer> needAnalyNodeIds) {
        //找出需要补充的阀门不存在于连通范围内的边
        DatasetVector networkDataset = facilityAnalystParam.getAnalystSetting().getNetworkDataset();
        List<Map<String, String>> edgeList = new ArrayList<>();
        for (Integer d : needAnalyNodeIds) {
            List<Map<String,String>> sidList = new ArrayList<>();
            Recordset recValveNode = networkDataset.query("smfnode="+d+" or smtnode=" + d , CursorType.STATIC);
            recValveNode.moveFirst();
            while (recValveNode.isEOF() == false) {
                Map<String,String> map=new HashMap<>();
                String sid1 = recValveNode.getString(layfiledConfig.getNo());
                int dir = recValveNode.getInt32(layfiledConfig.getDir());
                String smID = recValveNode.getInt32("SmID")+"";
                String startSid = recValveNode.getString(layfiledConfig.getSPoint());
                String endSid = recValveNode.getString(layfiledConfig.getEPoint());
                int startSmid = recValveNode.getInt32("smfnode");
                int endSmid = recValveNode.getInt32("smtnode");
                map.put(layfiledConfig.getNo(),sid1);
                map.put("smid",smID);
                map.put(layfiledConfig.getDir(),dir+"");
                map.put(layfiledConfig.getSPoint(),startSid);
                map.put(layfiledConfig.getEPoint(),endSid);
                //只取无流向、和下游管线(阀门为起点流向为0正向，阀门为终点流向1)
                if(dir!=3 ){
                    if(dir==2){
                        sidList.add(map);
                    }else{
                        if((d==startSmid && dir==0) || (d==endSmid && dir==1)) {
                            sidList.add(map);
                        }
                    }
                }
                recValveNode.moveNext();
            }
            recValveNode.close();
            recValveNode.dispose();
            //排除掉已存在连通性分析
            List<Map<String, String>> filterEdge = sidList.stream().filter(s -> !edgeSids.contains(s.get(layfiledConfig.getNo()))).collect(Collectors.toList());
            edgeList.addAll(filterEdge);
        }
        if (CollUtil.isNotEmpty(edgeList)) {
            List<Integer> downList=new ArrayList<>();
            for (Map<String,String> s : edgeList) {
                String sid1 = s.get(layfiledConfig.getNo());
                Integer smId = Integer.valueOf(s.get("smid"));
                String direction = s.get(layfiledConfig.getDir());
                String startSid = s.get(layfiledConfig.getSPoint());
                String endSid = s.get(layfiledConfig.getEPoint());
                //如果是不确定流向，则做无障碍扩展连通性分析， 0:正向 1:反向 2:不确定流向 3:无流向
                if(direction.equals("2") || direction.equals("3")){
                    Map<String, Object> stringObjectMap1 = connectionAnalysis(facilityAnalystParam.getAnalystSetting().getNetworkDataset().getName(), layfiledConfig, sid1, false, edgeSids);
                    List<String> connectedEdgesList1 = (List<String>) stringObjectMap1.get("edgeSids");
                    List<Object> tracedNode1 = (List<Object>) stringObjectMap1.get("tracedNode");
                    affectedEdge.addAll(connectedEdgesList1);
                    affectedNode.addAll(tracedNode1);
                }else{
                    //确定流向则做下游分析(下游管线(阀门为起点流向为0正向，阀门为终点流向1))
                    FacilityAnalystResult resultSourceFromEdge = facilityAnalystParam.traceDownFromEdge(smId, "Length", true);
                    if (resultSourceFromEdge != null && resultSourceFromEdge.getEdges().length>0) {
                        List<Integer> collect = Arrays.stream(resultSourceFromEdge.getEdges()).boxed().collect(Collectors.toList());
                        downList.addAll(collect);
                    }
                }
                affectedEdge.add(sid1);
                affectedNode.add(startSid);
                affectedNode.add(endSid);
            }
            if(CollUtil.isNotEmpty(downList)){
                List<Integer> collect = downList.stream().distinct().collect(Collectors.toList());
                StringBuffer sb = new StringBuffer();
                for (int i = 0; i < collect.size(); i++) {
                    sb.append(collect.get(i) + ",");
                }
                String strEdgeIds = sb.toString().substring(0, sb.toString().length() - 1);
                Recordset recValveNode = networkDataset.query("SMID in ("+strEdgeIds+")", CursorType.STATIC);
                recValveNode.moveFirst();
                while (recValveNode.isEOF() == false) {
                    String sid = recValveNode.getString(layfiledConfig.getNo());
                    String startSid = recValveNode.getString(layfiledConfig.getSPoint());
                    String endSid = recValveNode.getString(layfiledConfig.getEPoint());
                    affectedEdge.add(sid);
                    affectedNode.add(startSid);
                    affectedNode.add(endSid);
                    recValveNode.moveNext();
                }
                recValveNode.close();
                recValveNode.dispose();
            }
        }
    }

    private Map<String, Object> connectionAnalysis(String tablename, LayerFiledConfig layfiledConfig, String sid, Boolean useBarrier, List<String> barrEdge) {
        Map<String, Object> arrAftEdges = connectionAnalysis.connectionAnalysis2(sid, tablename, layfiledConfig, useBarrier, barrEdge);//进行有障碍连通性分析
        return arrAftEdges;
    }
}