package com.tofly.analyse.util.analysis;

import com.supermap.analyst.networkanalyst.TransportationAnalyst;
import com.supermap.analyst.networkanalyst.TransportationAnalystParameter;
import com.supermap.analyst.networkanalyst.TransportationAnalystResult;
import com.supermap.data.*;
import com.tofly.analyse.entity.analyse.Features;
import com.tofly.analyse.entity.analyse.PathAnalysisResult;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;


/**
 * 路径分析
 *
 * @author zhoux
 */
@Component
public class PathAnalysis {


    /// <summary>
    /// 路径分析
    /// </summary>
    /// <param name="node1"></param>
    /// <param name="node2"></param>
    /// <returns></returns>
    public TransportationAnalystResult PathAnalysis(TransportationAnalyst m_transAnalyst, List<Integer> param) {
        TransportationAnalystParameter transParameter = new TransportationAnalystParameter();
        transParameter.setNodes(param.stream().mapToInt(i->i).toArray());
        transParameter.setRoutesReturn(true);
        transParameter.setNodesReturn(true);
        transParameter.setEdgesReturn(true);
        TransportationAnalystResult taResult = m_transAnalyst.findPath(transParameter, false);
        return taResult;
    }


    public PathAnalysisResult PathAnalysis(TransportationAnalyst m_transAnalyst, DatasetVector vecPNetwork, Point2Ds point2Ds) {
        PathAnalysisResult pathAnalysisResult=new PathAnalysisResult();
        TransportationAnalystParameter transParameter = new TransportationAnalystParameter();
        transParameter.setPoints(point2Ds);
        transParameter.setNodesReturn(true);
        transParameter.setEdgesReturn(true);
        TransportationAnalystResult taResult = m_transAnalyst.findPath(transParameter, false);
        if(null!=taResult.getEdges() && taResult.getEdges().length>0){
            StringBuffer stringBuffer = getStringBuffer(taResult.getEdges());
            Recordset recordset = vecPNetwork.query("smid in ( "+stringBuffer+" )", CursorType.STATIC);
            List<Features> recordData = getRecordData(recordset);
            pathAnalysisResult.setEdgeFeatures(recordData);
        }
        if(null!=taResult.getNodes() && taResult.getNodes().length>0){
            StringBuffer stringBuffer = getStringBuffer(taResult.getNodes());
            Recordset recordset = vecPNetwork.getChildDataset().query("smid in ( "+stringBuffer+" )", CursorType.STATIC);
            List<Features> recordData = getRecordData(recordset);
            pathAnalysisResult.setNodeFeatures(recordData);
        }
        return pathAnalysisResult;
    }

    private static StringBuffer getStringBuffer(int[][] data) {
        StringBuffer stringBuffer=new StringBuffer();
        for(int i = 0; i< data.length; i++){
            for (int j = 0; j < data[i].length; j++) {
                stringBuffer.append(data[i][j]);
                if(j!= data[i].length-1){
                    stringBuffer.append(",");
                }
            }
        }
        return stringBuffer;
    }

    private static List<Features> getRecordData( Recordset recordset) {
        List<Features> edgeFeatures=new ArrayList<>();
        List<String> fieldNames=new ArrayList<>();
        recordset.moveFirst();
        FieldInfos fieldInfos = recordset.getFieldInfos();
        FieldInfo[] fieldInfos1 = fieldInfos.toArray();
        for (FieldInfo f : fieldInfos1) {
            if(f.getName().contains("id") || !f.getName().contains("sm")){
                fieldNames.add(f.getName());
            }
        }
        while (!recordset.isEOF()) {
            Features features=new Features();
            List<String> fieldValues=new ArrayList<>();
            for (String s : fieldNames) {
                Object fieldValue = recordset.getFieldValue(s);
                fieldValues.add(fieldValue!=null?fieldValue.toString():null);
            }
            features.setFieldNames(fieldNames);
            features.setFieldValues(fieldValues);
            String s = Toolkit.GeometryToGeoJson(recordset.getGeometry());
            features.setGeometry(s);
            edgeFeatures.add(features);
            recordset.moveNext();
        }
        recordset.dispose();
        return edgeFeatures;
    }


}
